/*
Rules:

  FAIL 
       issued by the MIDDLE node upon detection of a failure  
       MIDDLE node may issue a single or a multiple packette FAIL 
       a multiple packette FAIL splits into multiple FAIL messages at
          each node where the paths in the FAIL diverge
       move upstream/downstream to the START/FINISH nodes
          the START node is the first node on a path
          the FINISH node is the last node on a path
          to move upstream along a path, means to move closer to the 
             START node for the path, 
          to move downstream along a path, means to move closer to the
             FINISH node for the path
       upon receipt of a FAIL, channels on active links referenced in 
          the FAIL are released
       FAIL ID == PACK ID of twoprong 

  PACK 
       issued upon detection of failure by the START node for path(s)
          or upon receipt of a FAIL by the START node for path(s) 
       if issued upon receipt of a FAIL, PACK ID == FAIL ID, since
          one node may issue more than one FAIL message all with
          the same ID and since FAIL messages are split as the paths 
          in them diverge in different directions, may have more than 
          one PACK message with the same ID
       if issued upon detection of failure, may also have more than one
          PACK message with same ID
        upon receipt of a PACK message Connection Table is updated
          based on information in the PACK
       upon receipt of a given PACK message for the first time, a
          node broadcasts the PACK to all its immediate neighbor
          except the message's sender
       upon receipt of a duplicate PACK message (same PACK ID, same
          paths), the PACK message is NOT broadcast
       if START node is lost, no PACKs will be issued for the 
          unrestorable path
       if FINISH node is lost, path is unrestorable, PACKs will never
          reach their destination, use TRAIL field, do
          not let a PACK be forwarded to a node in the TRAIL,  when a
          node is unable to forward the PACK, it does nothing 
       packette(s) in PACK includes bandwidth of lost path(s)
       
   NO NACKs, CREQs, CRACKs, CREQFs, CRAKFs, or CONFCs
   
   priority
       first come, first served 

   protection
      ACKed connections are protected
      CONFed connections are protected and usable

   ACKs
      initiated by the FINISH node upon receipt of a:
          PACK if the FINISH node does NOT already have 'path 
             bandwidth' of acked connections for the path in the PACK 
          CONF, if receipt of the CONF results in the release of 
             reserved channels
      as travels towards START node,  MIDDLE nodes make connections and
         forward ACK for as much of the bandwidth requested in the ACK
         as possible
      
   CONF/CONFF
      reply to an ACK, initiated by the START node  
      CONFF is used by the START node when:
         1)  all its connections are confirmed connections and it has 
             no spares left OR 
         2)  the lost path has been fully restored 
      otherwise, a CONF is used

   PDONE
      initiated by any node issuing or receiving a CONF or CONFF if that
         node is incapable of making additional connections & that node is 
         an end node for a previously 'not done' path
      broadcast by any node upon just learning that the path is done



Functions (in order of occurence of their definition in the program):

	void fillUnderline(char *name);
	void removeUnderline(char *name);
   static char *int2str_(char *Result, int i)
   int decideFailureType(char *scenario);
	void setupNodeFailureEvent(float time, char *nodeName);
	void setupLinkBreakEvent(float time, char *break1str, char *break2str);
   void  parse_packet(char *newPacket, int *path, int *qos, int *up, int *bw, 
		int *down)
   void  parse_ack_packet(char *newPacket, int *path, int *qos, int *up, 
		int *bw, int *down, int *ackhop)
   int Get_Node_ID(NODENAME the_node)
   void Get_Node_Name(int node_id, char node_name[4])
   int searchNodeID(char **p);
   tablerec * get_start_finger(node *cnode, int path)
   tablerec * get_end_finger(node *cnode, int path)
   tablerec * get_ack_finger(node *cnode, int path)
   tablerec * get_unconnect_finger(node *cnode, int path)
   tablerec * get_unconnect_upfinger(node *cnode, int path, int up)
   void Append(char str1[], char str2[]) 
   void Delete(char str1[], char str2[])
   static char *simtime2str(char * Result, v)
   static float noise()
   static void simlog(char *s)
   void Init_Tablerec(int path, int qos, int up, int hopcount, int path_bw, int down, 
		int ackhop, int whichPart, float time_received, tablerec *newrec)
   void Copy_Tablerec(tablerec *oldrec, tablerec *newrec) 
   void Clean_Table(node *cnode)
   void Sort_Table(node *cnode)
   void Pack_Table(node *cnode) 
   void Print_Table(node *cnode)    
   void Print_Node_Table(node *cnode)
   void Add_To_Table(node *cnode, int path, int qos, int up, int bw,
                     int down, int ackhop, int whichPart,
                     int confirmed, boolean forced, int path_bw)
   void Forward_Msg(node *cnode, int n, erec *ev, char mtype[],
                    int destination, char packet[])             
   void Take_Spares(node *cnode, int first, int second, int bw)
   void Give_Spares(node *cnode, int first, int second, int bw)
   void Insert_Before_Tablerec(node *cnode, tablerec *intablerec, 
                               tablerec *addrec)
   void Insert_After_Tablerec(node *cnode, tablerec *intablerec, 
                              tablerec *addrec)
   void Remove_Tablerec(node *cnode, tablerec *obsolete)
   static void SIM_Processing(erec *ev, int n, node *cnode)
   static void FAIL_Processing(char *packID, erec *ev, int n,
                               node *cnode)
   static void PACK_Processing(char *packID, erec *ev, int n, 
                                 node *cnode)
   void init_one_packette_msg(erec *ev, node *cnode, int *msg_num_ptr,
                              int *path_ptr, int *qos_ptr, 
										int *up_ptr,
										int *bw_ptr, 
                              int *down_ptr, tablerec **start_finger_ptr,
                              int *whichPart_ptr, int *path_bw_ptr) 
   void init_ack_packette_msg(erec *ev, node *cnode, int *msg_num_ptr,
                              int *path_ptr, int *qos_ptr, 
										int *up_ptr,
										int *bw_ptr, 
                              int *down_ptr, 
										int *ackhop_ptr, tablerec **start_finger_ptr,
                              int *whichPart_ptr, int *path_bw_ptr) 
   void generate_nodelist(node *cnode, nodelist **ptr_ptr, boolean up,
                          boolean ack, int path, int root, int msg_num,
                          int msg_sender, int still_waiting)
   void free_nodelist(nodelist *ptr)
   void zero_up_entries(node *cnode, int path, int up)
   boolean up_available(int up, int num_unavailable, int *unavailables)
   boolean spares_remaining(cnode)
   int num_spares(node *cnode, int node_num)
   int get_ack_num(node *cnode, int path, int up)
   int get_ack_bw(node *cnode, int path, int up)
   void ACK_Processing(erec *ev, int n, node *cnode)
   void CONF_Processing(erec *ev, int n, node *cnode, char type)
   static void PDONE_Processing(erec *ev, int n, node *cnode)
   static void Process(erec * ev, int n, node *cnode, int *downs)
   void Do_Path_Connection(int path_id, int qos, int hopcount, int bw, 
		int pathlength, int prevNode, int thisNode, int nextNode, int whichPart)
   void Make_Path(int path_id, int qos, int bw, pathArray *pap)
	int FindNodeName(char *nodeName, char **ap)
		 void ParsePathStr(pathArray *pap, char *pathstr)
		 void Setup_Paths(FILE *path_data)
		 static void init_nodes(char *s)
		 void Follow_Path(int node_id, int path_id, int path_bw,
								char thePath[], FILE *path_data, float time,
								int last_node)   
		 void Trace_Paths(char filename[])
		 void Trace_Lost_Paths (char filename[])
		 void Follow_Restored_Path(int node_id, int path_id, int path_bw,
											char thePath[], FILE *path_data,
											int last_node)            
		 void Trace_Restored_Paths(char filename[], float endtime)
		 void log_network()

	 */

#include "decl.h"
#include <math.h>
#include <time.h>
#include <sys/timeb.h>

#ifndef STRINGS_H
#include "strings.h"
#endif

#ifndef NETSIM_H
#include "netsim.h"
#endif

#ifndef LISTS_H
#include "lists.h"
#endif

#ifndef MATH_H
#include "mathfunc.h"
#endif





#define KMPERDEG        111.3194   /* km/deg */
#define LIGHTSPEED      299796.0   /* km/sec */
#define NO_NODE         0
#define MAX_CHANNELS    256
#define MAX_NODES       256
#define PACKET_SIZE     25
#define PACKET_FIELD_SIZE     5
#define MSG_BASE_SIZE   7
#define MSG_HEADER   	4
#define SINGLE_PACKETTE_PAYLOAD   	7.0
#define FINAL           -1
#define CONFIRMED       1
#define ACKED           2
#define MaxPathLength 25
#define NodeNameLength 4  /* include the \0 */
#define MaxPathStringLength 121
#define MAX_PACKETTES   10  /* default allows PACK to send 25 packettes */
#define MAX_PACKET_LEN  PACKET_SIZE*MAX_PACKETTES
/* because MAX_PACKET_LEN is used in string declaration can't change it
	dynamically unless packet_info field are dynamically allocated/replaced
	a lot of code needed. But it will be interesting to see the impact of
	this length on performance, in 28-29cut, path 825'sPACK msg won't arrive
	at 29 until 1.7 seconds, that is amazing  Chow */


/* net file format types */
#define V_H3 				5
#define V_H3wSpanMiles 	6
#define L_L3				7
#define L_L3wSpanKms		8

enum { 
	START, MIDDLE, FINISH };

typedef char NODENAME[NodeNameLength];

NODENAME ID_Table[MAX_NODES + 1];



/* timeval struct for win32*/
typedef struct timeval{
	long tv_sec;
	long tv_usec;
}timeval;

/* Used in "links" field of node structure */
typedef struct linkdata {
	int         dest;  /* each node record contains a list of linkdata, the
										 dest field contains the node at the other end of 
										 the link */
	boolean     active;  /* if true, link is usable, NOT broken */
	int         work, spare;  /* all channels are modeled as having the
												  same bandwidth 
												  'work' is the number of working channels
												  in the link
												  'spare' is the number of spare channels
												  in the link */

	float       delay;   /* transmission time over the link */
	float       last_trans;  
} 
linkdata;


/* This structure is used to maintain the information that allows a node 
		 to forward a CONF or CONFF downstream with the correct message
		 number.
	 */
typedef struct message_history
{
	int recv;        /* id number of last ACK received */

	int down;        /* id number of last ACK from the downstream
										 node for which an ACK was forwarded */

	int up;          /* id number of last ACK forwarded to the
										 upstream node */
} 
message_history;


/* Used in node structure to keep track of state of the connections. */
typedef struct confirm
{
	int value;               /* 0 unconnected 
													CONFIRMED if entry has been confirmed
													ACKED if entry has been acked but not
													  confirmed, ACKED > 1 
												*/
	message_history ack;
	int conf_recv;           /*  'FINAL' after path has been
														completely restored  
													 0 no CONFs received for this 
														replacement connection 
													>0 message number of the last CONF
														received 
												*/

	boolean conf_sent;       /* true if CONF/CONFF has been forwarded to
													the downstream node
												*/
} 
confirm;


/* Connection Table Entry */
typedef struct tablerec {
	int       path;            /* internal path identification # */

	int       qos;             /* QoS with lowest digit indicate priority */
	/* with 1 as highest priority */
	int       up;              /* internal identification number of the
													upstream node */

	int       hopcount;             /* hop count from start node via up node */
	int       bw;              /* if entry references a connection,
														bandwidth of the connection
													else bandwidth requested  
												*/

	int       down;            /* internal identification number of the
													downstream node */

	int       ackhop;          /* hop count from finish node via down node */
	int       whichPart;       /* location of node on 'path'--START,
													MIDDLE or FINISH */
	confirm   confirmed;       
	float     time_connected;  /* If the connection has been acked but
													not confirmed, time the connection was
													acked. 
													If connection has been confirmed, time 
													the connection was confirmed.  */
	float     time_received;   /* when information in entry was received */
	struct tablerec *next, *prev;
	int       path_bw;         /* Total bandwidth of the original path--after
													a failure the original path may be
													replaced by several restoration paths
													and the sum of the bandwidths of the 
													restoration paths for 'path' should not
													exceed the path bandwidth.
											  */

} 
tablerec;


typedef struct int_node 
{
	int num;                 
	int up;
	struct int_node *next;
} 
int_node;  


/* Used to maintain list of PACK messages forwarded by a node. */
typedef struct packrec {
	char   id[NodeNameLength*2];              /* PACK identification number */
	int_node *paths;           /* Id numbers of Paths in the PACK */
	struct packrec *next;
} 
packrec;


/* Used for collecting statistics ONLY. */
typedef struct disruptrec {
	int path_num;
	int bw;
	int links;
	boolean restorable;
	int restored_bw;
	float restoration_time;
	float disrupt_time;
	char path[MaxPathStringLength];
	boolean in2FullyRestored;
	int in2bw;
	int totalRestoredBw;
	struct disruptrec *next;
} 
disruptrec;

typedef struct statistics {
	int	totalDisruptedBw;
	int	totalRestorableBw;
	int	totalRestoredBw;
	int	totalDisruptedPath;
	int	totalRestorablePath;
	int	fullyRestoredPath;
	int	partiallyRestoredPath;
	int	notRestoredPath;
	int	in2FullyRestoredPath;
	int	in2PartiallyRestoredPath;
	int	in2NotRestoredPath;
	int	in2RestoredBw;
	int	links_used;
	double	wamean;
	double	hamean;
	double	lastRestoreTime;
} statistics;

typedef struct Packet_info
{
	int receiver_id;
	char thePacket[MAX_PACKET_LEN+1];
	struct Packet_info *next;
} 
Packet_info;

typedef struct up_info {
   int up;
   int ack_num;
   boolean retry;
   struct up_info *next;
} up_info;

typedef struct down_info {
   int down;
   int bw;
   struct down_info *next;
} down_info;


typedef struct path_ack_info
{
	int path;
	int num_acks;
   up_info *uplist;
   down_info *downlist;
	struct path_ack_info *next;
} 
path_ack_info;


/* Timer_rec structure */
/* Used to keep track of timer inside node */
typedef struct TimerRecord{
	int initiated;
	float expire_time;
	int path;
	float delay;
	struct TimerRecord *next, *prev;
}
TimerRecord;







typedef struct node 
{
	int 			id;      /* node ID */
	float      time;     /* Time that the node last processed a message */
	float      dxc_time;     /* Time that DXC finishes last request 
										 or current node time whichever later*/
	list       links;    /* Linked list of "linkdata" */
	char       name[4];  /* Name of node as read in from network
											description file, 'netname.net'. */

	float      msg_process_delay; /* Comes from third column of the first line
														for each node in netname.net */
	float      lat, long_;  /* Comes from columns 5 and 6 of the first line for
												each node in netname.net */
	tablerec  *tablehead, *tabletail;   /* table == Connection Table */
	packrec   *packlist;
	int        creq;  /* Number assigned to any message without a special
										identification number.  Increments each
										time a new message number is required. */
	path_ack_info *acks_sent;  /* Each node is allowed to forward an ACK
													for a given path to the same uptream
													node twice.  Used to keep track of which
													nodes have been requested to participate
													in which restoration paths.
												*/
	char packbuf[MAX_PACKET_LEN+1]; /* buffer for accumulate pack msgs*/




	TimerRecord *RecordHead;		/* record of initiated timers at a node */


} 
node;


typedef struct nodelist
{
	int      num;             /* identification number of the node */

	int      acked_bw;        /* For ACK processing: 
													 acked bandwidth--summed over all the
													 Connection Table entries for node num
	
													 For CONF processing:
													 confirmed bandwidth--summed over all
													 the Connection Table entries for num 
													 node
												 */

	boolean  acked;           /* For ACK processing: 
														 if true means that the just recieved
														 ACK was responsible for some of the
														 acked bandwidth 
													 For CONF processing:
														 if true, some confirmed bandwidth
														 exists for the connection
												 */

	int msg_num;
	struct nodelist *next;
} 
nodelist;


typedef struct pathArray {
	int length;
	NODENAME name[MaxPathLength+1];
	boolean glassThrough[MaxPathLength+1];
	char path[MaxPathStringLength];
} pathArray;

float Forward_Delay = (float)0.0;
float lastRestoreTime = (float)0.0;
int   pathAffected = 0;
int	pathRestorable = 0;
int   pathPartiallyRestored = 0;
int   pathFullyRestored = 0;
int   pathNotRestored = 0;
static int PARALLEL_DCS = 1; 
static float CROSS_CONNECT_TIME = (float)0.02000;   /* time for digital cross-
																	  connect switch to make
																	  connections in seconds */
static float MPTOVERRIDE = (float)0.004;       /* msg/signature processing time*/
static float PPTOVERRIDE = (float)0.100;              /* in seconds 
																	  if >= 0 used as node's 
																	  message processing 
																	  delay
																	  if < 0 node's message
																	  processing time is
																	  obtained from the net 
																	  file 
																  */

static float TX_SPEED = (float)8.0;                  /* transmission speed in
																	  kilobits per second */
static float NOISE_SIZE = (float)0.00001;            /* multiplied by random 0..100 */
static float LINK_CHECK_SCALEFACTOR = (float)0.05;   /* fraction of node msg process 
																	  delay */
static float STOP_TIME = (float)300.0;                /* sec -- total run time of 
																	  simulation */
static float REFRACINDEX = (float)1.4;               /* Light travels more
																	  slowly in fiber than in air.  
																	  REFRACINDEX makes the 
																	  ajustment.  
																  */
static int MAX_HOPS = 11;                    /* Maximum number of hops
																	 that a broadcast message will 
																	 be sent.
																  */

static int TYPE = 1;

static int ACK_LIMIT = 1;                     /* Number of time a node
																	  may request the same
																	  upstream node to
																	  participate in a restoration
																	  path for the same lost path.
																  */
static erec *ee;                              /* from netsim.h:
																	  typedef struct erec {
																			 float time;
																			 int done;
																			 long next,src,dst;
																			 symbol_node *msg
									 } erec;
																	  from strings.h
																	  typedef struct symbol_node{
																			 struct sumbol_node *next
																			 char * sstr, * dstr;
									 } symbol_node;
																	*/
static int i, 
j;
static boolean SaveEvents;
static list net;
static int netsize;                           /* Number of nodes in
																	  network */

static symbol_node *nodename;
static FILE *log_;
static char pathlist[256];
static node *nextnode, 
*thisnode;
static boolean stop;
static symbol_node *options;
static char result[256];
static char scenario[256];
static char outfilename[81],        /* default: NETSIM.OUT */ 
infilename[81];         /* First parameter on the command
														 line.  Name of the network
														 description file, i.e.
														 'netname.net'.
													 */

static int break1,                  /* internal identification number:
															  in case of a node failure:
																	node lost
															  in case of a link failure:
																	first node entered on the
																	command line in
																	description of the link 
																	failure
													 */ 
break2, 
theNode, 
choosen,                 /* internal identification number
														 of node being studied when 
														 DEBUG_NODE is true
													 */   
selected;                /* internal identification number of
														 the path being studied when
														 DEBUG_PATH is true
													 */

NODENAME break1str, break2str, failureNode;
int failureNodeID;
char failureEventFile[128];
static char  tempStr[8];
static int min_path = INT_MAX;  
static int max_path = 0;
static int msgCount = 0;             /* Number of messages sent. */

static int FAILCount = 0;            /* TYPECount number of messages of
														  TYPE sent.  If debug, message
														  count history is reported in
														  file 'EVENT.OUT'. */

static int PACKCount = 0;
static int ACKCount = 0;
static int CONFCount = 0;
static int PDONECount = 0;
static int eventCount = 0;
static int bwCount = 0;
static disruptrec *disrupted = NULL;   /* To keep track of restoration
															 statistics. */
static int uplim_rbw = 999;              /* Upper limit on restorable 
															 bandwidth */
static int src_rbw = 0, dest_rbw = 0;  /* Number of remaining spares. */

static int lost = 0;                   /* Number of path links lost. */
static int lost_bw = 0;
static int debug;                      /* If debug and a link break, 
															 program generates a Connection Table
															 and message history for the nodes
															 at the ends of the link break. 
															 Records information in files
															 'BREAK1.OUT' and 'BREAK2.OUT'.
															 If debug, records a message
															 count history in 'EVENT.OUT'.
														 */
static int debug_node;                 /* If debug_node, program generates a 
															 Connection Table and message
															 history for the choosen node.
															 Recorded in file
															 'CHOOSEN.OUT'. */

static int debug_path;                 /* If debug_path, program
															 generates a Connection Table and
															 message history showing every
															 message involved in the
															 restoration of the selected path
															 and the effect of each message
															 on the receiving node. 
															 Recorded in file
															 'SELECTED.OUT'. */

static int length_scenario=0;            /* Used to determine whether a
															 link or a node failure is being 
															 simulated. */

int V_Hcoord;
int OVERWRITE_LINK_DISTANCE;
boolean GlassThrough = false;
boolean reuse = false;
static FILE *break1_out = NULL, 
*break2_out = NULL, 
*choosen_out = NULL,
*selected_out = NULL, 
*event_out = NULL;      
static boolean selected_path = false;

/* Timer Delay variables */
int RANDOMDELAY=1;
float RAND_DEL_LIMIT= (float)1.0;
float RAND_DEL_DOWN_LIMIT = (float)0.0;

/* used in function CheckTableForErrors to keep track of the bw */
typedef struct uprecord
{
	int up;
	int acc_bw;
	struct uprecord *next;
}uprecord;


typedef struct pathrecord
{
int path;
int lost_bw;
struct uprecord *list;
struct pathrecord *next;
}pathrecord;




float DELAY=(float)0.000000;




void Get_Node_Name (int node_id,
NODENAME node_name) 
{
	strcpy (node_name, ID_Table[node_id]);
} /* end Get_Node_Name */


/* in:  pointer to node information
		 Writes node information to output file 'outfilename' and 
		 if (debug_path and selected_path) also writes node information to
			  file 'SELECTED.OUT'. 
	 */ 
void Print_Table(node *cnode) 
{
	tablerec *finger;
	char      upstr[4], downstr[4], part;

	if (debug_path && selected_path)
	{
		fprintf(selected_out, "\n%s:\n", cnode->name);
		fprintf(selected_out, 
		" PATH  QOS   UP   HOP BW   DOWN ACKHOP   CONFIRMED   PART    ACKS    CONF_RECV\n");

	}
	else
	{
		printf("\n%s:\n", cnode->name);
		printf(
		" PATH  QOS   UP   HOP BW   DOWN ACKHOP   CONFIRMED   PART    ACKS    CONF_RECV\n");
	}
	finger = cnode->tablehead;
	while (finger) 
	{
		if (finger->up == NO_NODE) 
			sprintf (upstr, "%s", "   ");
		else 
		    Get_Node_Name (finger->up, upstr);
		if (finger->down == NO_NODE) 
			sprintf (downstr, "%s", "   ");
		else 
		    Get_Node_Name (finger->down, downstr);
		if (finger->whichPart == START)
			part = 'S';
		else if (finger->whichPart == MIDDLE)
			part = 'M';
		else 
		    part = 'F';
		if (debug_path && selected_path)
		{
			fprintf(selected_out,
			"%5d %3d %5s %2d  %3d  %5s   %2d          %2d     ", 
			finger->path, finger->qos, upstr, finger->hopcount, finger->bw, 
			downstr,  finger->ackhop, finger->confirmed.value);
			fprintf(selected_out,
			"   %c    %2d/%2d/%2d      %2d\n", part, 
			finger->confirmed.ack.down, 
			finger->confirmed.ack.up, 
			finger->confirmed.ack.recv, 
			finger->confirmed.conf_recv); 
		}
		else
		{
			printf (
			"%5d %3d %5s %2d  %3d  %5s   %2d          %2d     ", 
			finger->path, finger->qos, upstr, finger->hopcount, finger->bw, 
			downstr,  finger->ackhop, finger->confirmed.value);
			printf (
			"   %c    %2d/%2d/%2d      %2d\n", part, 
			finger->confirmed.ack.down, 
			finger->confirmed.ack.up, 
			finger->confirmed.ack.recv, 
			finger->confirmed.conf_recv); 

		}
		finger = finger->next; 
	}
	if (debug_path && selected_path)
		fprintf(selected_out,"\n");
} /* end Print_Table */



/* function to check tablerec for bad data */
/* Returns 0 if OK, else 1 */

int CheckTableRecForError(tablerec *t)
{
if(t->path < 0)
	{
	printf("BAD TABLE REC \n");
	printf("tablerec->path < 0\n");
	return 1;
	}
if(t->up < 0)
	{
	printf("BAD TABLE REC \n");
	printf("tablerec->path=%d,  up=%d (<0)\n", t->path, t->up);
	return 1;
	}
if(t->hopcount < 0)
	{
	printf("BAD TABLE REC \n");
	printf("tablerec->path=%d,  hopcount=%d (<0)\n", t->path, t->hopcount);
	return 1;
	}
if(t->bw < 0)
	{
	printf("BAD TABLE REC \n");
	printf("tablerec->path=%d,  bw=%d (<0)\n", t->path, t->bw);
	return 1;
	}
if(t->down < 0)
	{
	printf("BAD TABLE REC\n");
	printf("tablerec->path=%d,  down=%d (<0)\n", t->path, t->down);
	return 1;
	}
if(t->ackhop < 0)
	{
	printf("BAD TABLE REC\n");
	printf("tablerec->path=%d,  ackhop=%d (<0)\n", t->path, t->ackhop);
	return 1;
	}
if(t->path_bw < 0)
	{
	printf("BAD TABLE REC \n");
	printf("tablerec->path=%d,  path_bw=%d (<0)\n", t->path, t->path_bw);
	return 1;
	}
return 0;
}



void add_pathrecord(pathrecord **headlist, tablerec *finger)
{
pathrecord *newrecord, *pointer, *head;
uprecord *up, *pek, *prev;

head = *headlist;

	if(*headlist == NULL) /* list of pathrecords is empty */
	{
	newrecord = (pathrecord*)malloc(sizeof(pathrecord));
	newrecord->path = finger->path;
	newrecord->lost_bw = finger->path_bw;
	newrecord->next = NULL;
	up = (uprecord*)malloc(sizeof(uprecord));
	up->up = finger->up;
	up->acc_bw = finger->bw;
	up->next =NULL;
	newrecord->list = up;
	*headlist = newrecord;
	}
	else /* if(headlist), list of pathrecords is NOT empty */
	{
		if((*headlist)->path == finger->path)
		{
			pek = (*headlist)->list;
			
			if(pek->up == finger->up)
			{
				pek->acc_bw+=finger->bw;
				return;
			}
			prev=pek;
			pek = pek->next;
			while(pek && pek->up != finger->up)
			{
				prev = pek;
				pek = pek->next;
			}
			if(!pek)
			{
				if(prev->up == finger->up)
				{
					prev->acc_bw+=finger->bw;
					return;
				}
				else
				{
					prev->next = (uprecord*)malloc(sizeof(uprecord));
					pek = prev->next;
					pek->up = finger->up;
					pek->next = NULL;
					pek->acc_bw = finger->bw;
					return;
				}
			}
			if(pek->up == finger->up)
			{
				pek->acc_bw+=finger->bw;
				return;
			}
		}
		pointer = (*headlist);
		head = pointer->next;
		while(head && head->path != finger->path)
		{
			pointer = head;
			head = head->next;
		}
		if(!head)
		{
			if(pointer->path == finger->path)
			{
				pek = pointer->list;
			
				if(pek->up == finger->up)
				{
					pek->acc_bw+=finger->bw;
					return;
				}
				prev=pek;
				pek = pek->next;
				while(pek && pek->up != finger->up)
				{
					prev = pek;
					pek = pek->next;
				}
				if(!pek)
				{
					if(prev->up == finger->up)
					{
						prev->acc_bw+=finger->bw;
						return;
					}
					else
					{
						prev->next = (uprecord*)malloc(sizeof(uprecord));
						pek = prev->next;
						pek->up = finger->up;
						pek->next = NULL;
						pek->acc_bw = finger->bw;
						return;
					}
				}
				if(pek->up == finger->up)
				{
					pek->acc_bw+=finger->bw;
					return;
				}
			}

			newrecord = (pathrecord*)malloc(sizeof(pathrecord));
			newrecord->path = finger->path;
			newrecord->lost_bw = finger->path_bw;
			newrecord->next = NULL;
			up = (uprecord*)malloc(sizeof(uprecord));
			up->up = finger->up;
			up->acc_bw = finger->bw;
			up->next =NULL;
			newrecord->list = up;
			pointer->next = newrecord;
			return;
		}
		if(finger->path == head->path)
		{

			pek = head->list;
			
			if(pek->up == finger->up)
			{
				pek->acc_bw+=finger->bw;
				return;
			}
			prev=pek;
			pek = pek->next;
			while(pek && pek->up != finger->up)
			{
				prev = pek;
				pek = pek->next;
			}
			if(!pek)
			{
				if(prev->up == finger->up)
				{
					prev->acc_bw+=finger->bw;
					return;
				}
				else
				{
					prev->next = (uprecord*)malloc(sizeof(uprecord));
					pek = prev->next;
					pek->up = finger->up;
					pek->next = NULL;
					pek->acc_bw = finger->bw;
					return;
				}
			}
			if(pek->up == finger->up)
			{
				pek->acc_bw+=finger->bw;
				return;
			}
		}
			
	}
}


void Free_Pathrecords(pathrecord **headlist)
{
	pathrecord *head;
	pathrecord *prev, *finger;
	uprecord *pointer, *prevpointer;

	head = *headlist;
	finger = head;
	/* if list is empty */
	if(!head)
		return;
	/* there is one item in pathrecord list */
	if(head->next == NULL)
	{
		pointer = head->list;

		/* there is only one item in uprecord list */
		if(pointer->next == NULL)
		{
			free(pointer);
		
		}
		/* there exists several items in uprecord list */
		else
		{
			prevpointer = head->list;
			pointer = pointer->next;
			while(pointer)
			{
				free(prevpointer);
				prevpointer = pointer;
				pointer = pointer->next;
			}
			free(prevpointer);
		}
	
		head->list = NULL;
	
	} /* end only one item in pathrecord list */

	/* there exists several items in pathrecord list */
	else
	{
		prev = finger;
		finger = finger->next; 
		while(finger)
		{
			pointer = prev->list;
			if(pointer->next == NULL)
				free(pointer);
			else
			{
				prevpointer = prev->list;
				pointer = pointer->next;
				while(pointer)
				{
					free(prevpointer);
					prevpointer = pointer;
					pointer = pointer->next;
				}
				free(prevpointer);
			}
			prev->list = NULL;

			prev = finger;
			finger = finger->next;
		}
		

		/* now only prev is left */
		pointer = prev->list;
		if(pointer->next == NULL)
			free(pointer);	
		else
		{
			prevpointer = prev->list;
			pointer = pointer->next;
			while(pointer)
			{
				free(prevpointer);
				prevpointer = pointer;
				pointer = pointer->next;
			}
			free(prevpointer);
		}
		free(prev);
	}
	*headlist = NULL;
}


int CheckPathRecords(pathrecord *head)
{
	pathrecord *finger;
	uprecord *upfinger;
	int totbw, value = 0;

	if(!head)
		return 0;
	finger = head;
	while(finger)
	{
		totbw = finger->lost_bw;
		upfinger = finger->list;
		while(upfinger)
		{
			if( ((upfinger->acc_bw)%totbw) != 0)
			{
				fprintf(stderr,"ERROR CheckPathRecords()\n");
				fprintf(stderr,"Sum(bw) for entries with upnode:%d and path:%d\n", upfinger->up, finger->path);
				fprintf(stderr,"is equal to %d != total lost bw %d\n\n", upfinger->acc_bw, finger->lost_bw);
				value = 1;
			}
			upfinger = upfinger->next;
		}
		finger = finger->next;
	}

	return value;
}




/* Function to search table for bad data */
/* Return 0 if table OK, else 1 (if bad data exists ) */
int CheckTableForError(node *cnode)
{

tablerec *t;
int value=0;
pathrecord  *head=NULL;

t=cnode->tablehead;

if(t == NULL) return 0;
while(t)
	{



	if(t->path < 0)
		{
		printf("BAD TABLE REC node:%d\n", cnode->id);
		printf("tablerec->path < 0\n");
		value = 1;
		}
	if(t->up < 0)
		{
		printf("BAD TABLE REC node:%d\n", cnode->id);
		printf("tablerec->path=%d,  up=%d (<0)\n", t->path, t->up);
		value = 1;
		}
	if(t->hopcount < 0)
		{
		printf("BAD TABLE REC node:%d\n", cnode->id);
		printf("tablerec->path=%d,  hopcount=%d (<0)\n", t->path, t->hopcount);
		value = 1;
		}
	if(t->bw < 0)
		{
		printf("BAD TABLE REC node:%d\n", cnode->id);
		printf("tablerec->path=%d,  bw=%d (<0)\n", t->path, t->bw);
		value = 1;
		}
	if(t->down < 0)
		{
		printf("BAD TABLE REC node:%d\n", cnode->id);
		printf("tablerec->path=%d,  down=%d (<0)\n", t->path, t->down);
		value = 1;
		}
	if(t->ackhop < 0)
		{
		printf("BAD TABLE REC node:%d\n", cnode->id);
		printf("tablerec->path=%d,  ackhop=%d (<0)\n", t->path, t->ackhop);
		value = 1;
		}
	if(t->path_bw < 0)
		{
		printf("BAD TABLE REC node:%d\n", cnode->id);
		printf("tablerec->path=%d,  path_bw=%d (<0)\n", t->path, t->path_bw);
		value = 1;
		}
	t = t->next;
	}


if(value)
	Print_Table(cnode);
return value;

}






void collect_statistics(statistics *s)
{


	int path_num;
	disruptrec *p = disrupted;
	disruptrec *q;
	boolean search;

	s->totalDisruptedBw = 0;
	s->totalRestorableBw = 0;
	s->totalRestoredBw = 0;
	s->totalDisruptedPath = 0;
	s->totalRestorablePath = 0;
	s->fullyRestoredPath = 0;
	s->partiallyRestoredPath = 0;
	s->notRestoredPath = 0;
	s->in2FullyRestoredPath = 0;
	s->in2PartiallyRestoredPath = 0;
	s->in2NotRestoredPath = 0;
	s->in2RestoredBw = 0;
	s->links_used = 0;
	s->wamean = 0.0;
	s->hamean = 0.0;
	s->lastRestoreTime = 0.0;

	while (p != NULL) 
	{
		s->totalDisruptedBw += p->bw;
		s->totalDisruptedPath++;
		if (p->restorable) 
		{
			p->in2FullyRestored = false; /* initialize */
			p->in2bw= 0;
			p->totalRestoredBw = 0;
			if (s->lastRestoreTime < p->restoration_time)
				s->lastRestoreTime = p->restoration_time;
			path_num = p->path_num;
			q = p;
			while (q != NULL) 
			{
				if (q->path_num == path_num) 
				{
					p->totalRestoredBw += q->restored_bw;
					s->links_used += q->links;
					s->wamean += (double) q->restored_bw*q->restoration_time;
					if (q->restored_bw != 0) 
					s->hamean += (double) q->restored_bw * (1.0/q->restoration_time);
					if (q->restoration_time <= 2.0) 
						p->in2bw += q->restored_bw;
				}
				q = q->next;
			}
			s->totalRestorableBw += p->bw;
			s->totalRestorablePath++;
			s->totalRestoredBw += p->totalRestoredBw;
			s->in2RestoredBw += p->in2bw;
			if (p->in2bw) 
			{
				p->in2FullyRestored = true;
				s->in2FullyRestoredPath++;
			} else if (p->in2bw == 0) 
			{
				s->in2NotRestoredPath++;
			} else 
			{
				s->in2PartiallyRestoredPath++;
			}
			if (p->totalRestoredBw == p->bw) 
			{
				s->fullyRestoredPath++;
			} else if (p->totalRestoredBw == 0) 
			{
				s->notRestoredPath++;
			} else 
			{
				s->partiallyRestoredPath++;
			}
		}
		/* search for the next path_num */
		p = p->next;
		search = true;
		while (p != NULL && search) 
		{
			if (p->path_num == path_num) p = p->next;
			else search = false;
		}
	}
}

float dxc_time(node *cnode, int bw) {
	if (PARALLEL_DCS==0) {
		cnode->dxc_time += CROSS_CONNECT_TIME;
	} else if (PARALLEL_DCS==1) {
		cnode->dxc_time += bw*CROSS_CONNECT_TIME;
	} else {
	
	cnode->dxc_time += ( (float)ceil( (float)(bw*1.0/(PARALLEL_DCS*1.0)) ))*CROSS_CONNECT_TIME;
	}
	return cnode->dxc_time;
}

int all_up_ack_exceed_limit(node *cnode, int path) {
   path_ack_info *pa = cnode->acks_sent;
	boolean flag = true;
   up_info *ui;
   if (pa) {
      /* search for the path */
      while (pa) {
         if (pa->path == path) {
            /* search for up info record */
            ui = pa->uplist;
            if (ui) {
               while (ui) {
                  if (ui->ack_num <= ACK_LIMIT) {
                     return 0;
                  } else ui = ui->next;
               }
					return 1;
            }
            return -1;
         } else pa = pa->next;
      }
   }
   return -1;
	
}

int up_ack(node *cnode, int path, int up) 
{
   path_ack_info *pa = cnode->acks_sent;
   up_info *ui;
   if (pa) 
   {
      /* search for the path */
      while (pa) 
	  {
         if (pa->path == path) 
		 {
            /* search for down info record */
            ui = pa->uplist;
            if (ui) 
			{
               while (ui) 
			   {
                  if (ui->up == up) 
				  {
                     return ui->ack_num;
                  } 
				  else 
					  ui = ui->next;
               }
            }
            return -1;
         } else 
			 pa = pa->next;
      }
   }
   return -1;
}

void add_up_ack(node *cnode, int path, int up) {
   path_ack_info *pa = cnode->acks_sent;
   up_info *ui;
   boolean first_pa = true;
   boolean first_ui = true;
   if (pa) {
      /* search for the path */
      while (pa) {
         if (pa->path == path) {
            /* search for down info record */
            ui = pa->uplist;
            if (ui) {
               while (ui) {
                  if (ui->up == up) {
                     ui->ack_num++;
                     return;
                  } else ui = ui->next;
               }
               first_ui = false;
            }
            ui = (up_info *) malloc(sizeof(up_info));
            ui->up = up;
            ui->ack_num = 1;
            if (first_ui) {
               ui->next = NULL;
               pa->uplist = ui;
            } else {
               ui->next = pa->uplist;
               pa->uplist = ui;
            }
            return;
         } else pa = pa->next;
      }
      first_pa = false;
   }
   pa = (path_ack_info *) malloc(sizeof(path_ack_info));
   pa->path = path;
   pa->num_acks = 1;
   pa->uplist = NULL;
   ui = (up_info *) malloc(sizeof(up_info));
   ui->up = up;
   ui->ack_num = 1;
   ui->next = NULL;
   pa->uplist = ui;
   if (first_pa) {
      cnode->acks_sent = pa;
      pa->next = NULL;
   } else {
      /* insert pa */
      pa->next = cnode->acks_sent;
      cnode->acks_sent = pa;
   }

}

void decrease_up_ack(node *cnode, int path, int up) {
   path_ack_info *pa = cnode->acks_sent;
   up_info *ui;
   boolean first_pa = true;
   boolean first_ui = true;
   if (pa) {
      /* search for the path */
      while (pa) {
         if (pa->path == path) {
            /* search for down info record */
            ui = pa->uplist;
            if (ui) {
               while (ui) {
                  if (ui->up == up) {
                     if (ui->ack_num > 0) ui->ack_num--;
                     return;
                  } else ui = ui->next;
               }
            }
            return;
         } else pa = pa->next;
      }
      return;
   }
	return;
}

void create_up_ack(node *cnode, int path, int up) {
   path_ack_info *pa = cnode->acks_sent;
   up_info *ui;
   boolean first_pa = true;
   boolean first_ui = true;
   if (pa) {
      /* search for the path */
      while (pa) {
         if (pa->path == path) {
            /* search for down info record */
            ui = pa->uplist;
            if (ui) {
               while (ui) {
                  if (ui->up == up) {
                     return;
                  } else ui = ui->next;
               }
               first_ui = false;
            }
            ui = (up_info *) malloc(sizeof(up_info));
            ui->up = up;
            ui->ack_num = 0;
            if (first_ui) {
               ui->next = NULL;
               pa->uplist = ui;
            } else {
               ui->next = pa->uplist;
               pa->uplist = ui;
            }
            return;
         } else pa = pa->next;
      }
      first_pa = false;
   }
   pa = (path_ack_info *) malloc(sizeof(path_ack_info));
   pa->path = path;
   pa->num_acks = 0;
   pa->uplist = NULL;
   ui = (up_info *) malloc(sizeof(up_info));
   ui->up = up;
   ui->ack_num = 0;
   ui->next = NULL;
   pa->uplist = ui;
   if (first_pa) {
      cnode->acks_sent = pa;
      pa->next = NULL;
   } else {
      /* insert pa */
      pa->next = cnode->acks_sent;
      cnode->acks_sent = pa;
   }
}

/* in:  integer 'i'
		 out: string represntation of 'i mod 1000'
	 */
static char *int2str_( char *Result, int i)
{
	char s[6];

	i %= 1000;
	sprintf(s, "%0.3ld", i);
	return strcpy(Result, s);
} /* end int2str */

/* Input: char string
	Output: remove underline from string */
void removeUnderline(char *name) {
	char *p, *q;
	p = name;
	q = name;
	while (*p != '\0') {
		while (*p != '\0' && *p != '_') {
			*q = *p; q++; p++;
		}
		while (*p != '\0' && *p == '_') p++;
	}
	*q = *p;
}

/* Input: char string 
	Output: fill underline and make string length = NodeNameLength */

void fillUnderline(char *name) {
	int i;
	i = strlen(name);
	if (i> NodeNameLength-1) {
		printf("name %s longer than NodeNameLenght=%d\n", name, NodeNameLength);
		exit(1);
	}
	if (i == NodeNameLength-1) return;
	while (i < NodeNameLength-1) {
		name[i++] = '_';
	}
	name[NodeNameLength-1] = '\0';
}
/* Input: char string scenario, the value in command line option /b=, 
		 Ouput: the type of failure, also 
		 setup break1str, break2str for single link break,
		 setup failureNode,
		 setup failureEventFile

		 Assume single link break if there is '-' in scenario string,
			multiple failure event in a event file, if there is ".event" at the
				 end of scenario stirng.
				  otherwise it is a signle node failure */

/* failure scenario */
#define SINGLE_LINK_BREAK    1
#define SINGLE_NODE_FAILURE  2
#define MULTIPLE_FAILURE     3

int failureScenario;

int decideFailureType(char *scenario) {
	char *p;
	char *bp;
	int failureType;
	p= scenario;
	bp = break1str;
	failureType = 0;
	while (*p != '\0') {
		if (*p == '-')  {
			failureType = SINGLE_LINK_BREAK;
			bp = '\0';
			p++;
			strcpy(break2str, p);
			fillUnderline(break1str);
			fillUnderline(break2str);
			if (debug) printf("decideFailureType: break1str=%s, break2str=%s\n",
			break1str, break2str);
			return failureType;
		} 
		else if (*p == '.') {
			if (strcmp(p, ".event") == 0) {
				failureScenario = MULTIPLE_FAILURE;
				strcpy(failureEventFile, scenario);
				return failureType;
			} 
			else {
				printf("decideFailureType: /b=%s illegal value\n", scenario);
				exit(1);
			}
		} 
		else {
			*bp = *p; 
			bp++; 
			p++;
		}
	}
	strcpy(failureNode, scenario);
	fillUnderline(failureNode);
	failureType = SINGLE_NODE_FAILURE;
	return failureType;
}

/* in:  name of node
		 returned: internal identification number for 'the_node'
	 */ 
int Get_Node_ID (NODENAME the_node) 
{
	int i;

	if (strlen(the_node) == 0) return NO_NODE; 
	/* Chow: the above line is needed, otherwise it may return # of node +1, 
			depending on whether the memory get zero out initially */
	for (i = 1; i < MAX_NODES; i++)
		if (strncmp (ID_Table[i], the_node, NodeNameLength-1) == 0) return i;
	return NO_NODE;
} /* end Get_Node_ID */


/* Input: failure node name 
		 Output: event put into the event queue */

void	 setupNodeFailureEvent(float time, char *nodeName) 
{
	theNode = Get_Node_ID (nodeName);
	failureNodeID = theNode;

	ee = (erec *)malloc(sizeof(erec));
	ee->time = time;
	ee->src = 0;
	ee->dst = theNode;
	ee->next = theNode;
	ee->msg = NULL;
	init_symbol(&ee->msg, "ACTION", " DEAD");
	init_symbol(&ee->msg, "TYPE", " SIM");
	init_symbol(&ee->msg, "PATH", " INTERNAL FAULT DETECTION");
	queue_event(ee);
}

/* Input: time of the event, two break node names,
		 Ouput: two event record put into event queue */

void setupLinkBreakEvent(float time, char *break1str, char *break2str) {
	char STR1[256];

	break1 = Get_Node_ID (break1str);
	break2 = Get_Node_ID (break2str);

	ee = (erec *)malloc(sizeof(erec));
	ee->time = time;
	ee->src = 0;
	ee->dst = break2;
	ee->next = break2;
	ee->msg = NULL;
	init_symbol(&ee->msg, "ID", int2str_(STR1, break1));
	init_symbol(&ee->msg, "ACTION", "BREAK");
	init_symbol(&ee->msg, "TYPE", " SIM");
	init_symbol(&ee->msg, "PATH", " INTERNAL FAULT DETECTION");
	queue_event(ee);

	ee = (erec *)malloc(sizeof(erec));
	ee->time = time;
	ee->src = 0;
	ee->dst = break1;
	ee->next = break1;
	ee->msg = NULL;
	init_symbol(&ee->msg, "ID", int2str_(STR1, break2));
	init_symbol(&ee->msg, "ACTION", "BREAK");
	init_symbol(&ee->msg, "TYPE", " SIM");
	init_symbol(&ee->msg, "PATH", " INTERNAL FAULT DETECTION");
	queue_event(ee);
}

/* parse_packet() converts the four fields of a packet from character strings 
		 to integers */
void  parse_packet(char *newPacket, int *path, int *qos, int *up, int *bw, 
	int *down){
	char pathField[PACKET_FIELD_SIZE+1];
	char qosField[PACKET_FIELD_SIZE+1];
	char upField[PACKET_FIELD_SIZE+1];
	char bwField[PACKET_FIELD_SIZE+1];
	char downField[PACKET_FIELD_SIZE+1];
	newPacket[PACKET_SIZE] = '\0';
	for (j = 0; j < PACKET_FIELD_SIZE; j++) {
		pathField[j] = newPacket[j];
		qosField[j] = newPacket[j + PACKET_FIELD_SIZE];
		upField[j] = newPacket[j + 2*PACKET_FIELD_SIZE];
		bwField[j] = newPacket[j + 3*PACKET_FIELD_SIZE];
		downField[j] = newPacket[j + 4*PACKET_FIELD_SIZE];
	}
	pathField[PACKET_FIELD_SIZE] = '\0';
	qosField[PACKET_FIELD_SIZE] = '\0';
	upField[PACKET_FIELD_SIZE] = '\0';
	bwField[PACKET_FIELD_SIZE] = '\0';
	downField[PACKET_FIELD_SIZE] = '\0';
	*path = atoi (pathField);
	*qos = atoi (qosField);
	if (*qos != 1) printf("qos=%d\n", *qos);
	*up = atoi (upField);
	*bw = atoi (bwField);
	*down = atoi (downField);
}

/* parse_ack_packet() converts the four fields of a packet from character 
	strings to integers */
void  parse_ack_packet(char *newPacket, int *path, int *qos, int *up, int *bw, 
	int *down, int *ackhop){
	char pathField[PACKET_FIELD_SIZE+1];
	char qosField[PACKET_FIELD_SIZE+1];
	char upField[PACKET_FIELD_SIZE+1];
	char bwField[PACKET_FIELD_SIZE+1];
	char downField[PACKET_FIELD_SIZE+1];
	char ackhopField[3];
	newPacket[PACKET_SIZE+2] = '\0';
	for (j = 0; j < PACKET_FIELD_SIZE; j++) {
		pathField[j] = newPacket[j];
		qosField[j] = newPacket[j + PACKET_FIELD_SIZE];
		upField[j] = newPacket[j + 2*PACKET_FIELD_SIZE];
		bwField[j] = newPacket[j + 3*PACKET_FIELD_SIZE];
		downField[j] = newPacket[j + 4*PACKET_FIELD_SIZE];
	}
	for (j=0; j<2; j++) 
		ackhopField[j] = newPacket[j + 5*PACKET_FIELD_SIZE];
	pathField[PACKET_FIELD_SIZE] = '\0';
	qosField[PACKET_FIELD_SIZE] = '\0';
	upField[PACKET_FIELD_SIZE] = '\0';
	bwField[PACKET_FIELD_SIZE] = '\0';
	downField[PACKET_FIELD_SIZE] = '\0';
	ackhopField[2] = '\0';
	*path = atoi (pathField);
	*qos = atoi (qosField);
	if (*qos != 1) printf("qos=%d\n", *qos);
	*up = atoi (upField);
	*bw = atoi (bwField);
	*down = atoi (downField);
	*ackhop = atoi(ackhopField);
}



/* in:  internal identification number for node
		 out: name of node # 'node_id'
	 */ 

/* in: address of pointer point to the path character string
		 return: the nodeID of the first nodeName in the path string if found,
			 0, if can not find name in Node ID_table,
			 -1, if can not find '-' delimiter or exceed max node name length 
			 the address of pointer is updated to point to the next node name */
int searchNode(char **p)
{
	char *np;
	NODENAME name;

	np = *p;
	i= 0;
	while (*np != '-' && *np != '\0' && i<NodeNameLength) {
		name[i++] = *np;
		np++;
	}
	if (i >= NodeNameLength) return -1;
	else name[i]='\0';
	if (*np == '-') *p = np++;
	else *p = np;
	return Get_Node_ID(name);
}

/* in:  pointer to node information (Connection Table),
				internal path identification number  
		 returned: a pointer to the first entry in the Connection Table for
					  'path' or NULL if this node's Connection Table does NOT 
					  have an entry for 'path'.
	 */
tablerec * get_start_finger(node *cnode, int path)
{
	tablerec *finger = cnode->tablehead;

	while (finger && finger->path < path)
		finger = finger->next;

	if (!finger || finger->path != path)
		return NULL;
	else 
	    return finger;
} /* end get_start_finger */


/* in:  pointer to node information,
				internal path identification number  
		 returned: a pointer to the last entry in the Connection Table for
					  'path' or NULL if the Connection Table does NOT have an entry for
					  'path'.
	 */
tablerec * get_end_finger(node *cnode, int path)
{
	tablerec *finger = cnode->tabletail;

	while (finger && finger->path > path)
		finger = finger->prev;

	if (!finger || finger->path != path)
		return NULL;
	else 
	    return finger; 
} /* end get_end_finger */


/* in:  pointer to node information,
				internal path identification number  
		 returned: a pointer to the first acked entry in the Connection Table for
					  'path' or NULL if the Connection Table does NOT have an acked
					  entry for 'path'.
	 */
tablerec * get_ack_finger(node *cnode, int path)
{
	tablerec *finger = cnode->tablehead;

	while (finger && finger->path < path)
		finger = finger->next;

	while (finger && finger->path == path && 
	    finger->confirmed.value == CONFIRMED)
		finger = finger->next;

	if (finger && finger->path == path && finger->confirmed.value > 1)
		return finger;
	else 
	    return NULL;
} /* end get_ack_finger */


/* in:  pointer to node information,
				internal path identification number  
		 returned: a pointer to the first unconnected entry in the Connection Table 
					  for 'path' or NULL if the Connection Table does NOT have an 
					  unconnected entry for 'path'.
	 */
tablerec * get_unconnect_finger(node *cnode, int path)
{
	tablerec *finger = cnode->tablehead;

	while (finger && finger->path < path)
		finger = finger->next;

	while (finger && finger->path == path && finger->confirmed.value)
		finger = finger->next;

	if (!finger || finger->path != path)
		return NULL;
	else 
	    return finger;
} /* end get_unconnect_finger */

boolean Only_available_entry(node * cnode, tablerec *finger){
	tablerec *p = get_start_finger(cnode, finger->path);

	 while (p && p->path == finger->path) {
		if (p->confirmed.value == 0 && p->up != NO_NODE && p->up != finger->up
			&& up_ack(cnode, p->path, p->up) <= ACK_LIMIT)
			return false;
		p = p->next;
	 }
	 return true;
}

/* in:  pointer to node information,
				internal path identification number,
				internal identifrication number for upstream node  
		 returned: a pointer to the first unconnected entry for 'path' with 'up' 
					  as the upstream node in the Connection Table 
					  for 'path' or NULL if the Connection Table does NOT have an 
					  unconnected entry for 'path' with 'up' as the upstream
					  node.
	 */
tablerec * get_unconnect_upfinger(node *cnode, int path, int up)
{
	tablerec *finger = cnode->tablehead;

	while (finger && finger->path < path)
		finger = finger->next;

	while (finger && finger->path == path && finger->confirmed.value)
		finger = finger->next;

	if (!finger || finger->path != path)
		return NULL;
	else 
	    {
		while (finger && finger->path == path && finger->up != up)
			finger = finger->next;
		if (finger && finger->path == path && finger->up == up)
			return finger;
		else
			return NULL;
	}
} /* end get_unconnect_upfinger */


/* in:  strings, str1 and str2
		 out: string: "str1-str2"
	 */     
void Append (char str1[], char str2[]) 
{
	sprintf (str1, "%s-%s", str1, str2);
}  /* end Append */


/* in: strings, str1 and str2
		 out:  if the last three characters of str1 == str2,  
				 str1 minus its last four characters  
				 else
					 the empty string
	 */
void Delete(char str1[], char str2[])
{
	int length = strlen(str1);

	if (strcmp(&(str1[length - 3]), str2))
		return;
	else
	{
		str1[length - 4] = '\0';
	} 
} /* end Delete */


/* in:  a floating point number, 'v'
		 out: the string equivalent of 'v'
	 */
static char *simtime2str(char *Result, float v)
{
	char s[256];

	sprintf(s, "%8.5f", v);
	return strcpy(Result, s);
} /* end simtime2str */



/* returned:  A random value between 0 and 100 * NOISE_SIZE--added 
						to the message processing delay.
						default for NOISE_SIZE:  0.00001
	 */
static float noise()
{
	float value = (float)0.0;
	return value;
	/* chow: comment out can not find random() on Sun
			float n;
			n = random() * 100.0 * NOISE_SIZE / (pow((double)2, (double) 32)-1.0);
			return (n);
			*/
} /* end noise */


/* in:  string 's' 
		 String 's' is appended to the file 'outfilename'  
	 */
static void simlog( char *s)
{
	if (debug) 
	{

		fprintf(log_, "%s\n", s);




	}
} /* end simlog */


/* in:  values to be assigned to fields of the tablerec
				pointer to tablerec to be assigned the given values
		 out: tablerec initialized with input values
	 */
static void Init_Tablerec(int path, int qos, int up, int hopcount, 
int path_bw, int down, int ackhop, int whichPart, float time_received, tablerec *newrec)
{
	/* Check in values */
	if(path_bw < 0 || down < 0 || ackhop < 0 || path < 0 || qos < 0 || up < 0|| hopcount < 0 )
		{
		printf("ERROR Init_Tablerec \n");
		exit(1);
		}



	newrec->path = path;
	newrec->qos = qos;
	newrec->up = up;
	newrec->hopcount = hopcount;
	newrec->bw = path_bw;
	newrec->down = down;
	newrec->ackhop = ackhop;
	newrec->whichPart = whichPart;
	newrec->confirmed.value = 0;
	newrec->confirmed.ack.recv = 0;
	newrec->confirmed.ack.down = 0;
	newrec->confirmed.ack.up = 0;
	newrec->confirmed.conf_recv = 0;
	newrec->confirmed.conf_sent = false;
	newrec->time_connected = (float)0.0; 
	newrec->time_received = time_received; 
	newrec->path_bw = path_bw;
	newrec->next = NULL;
	newrec->prev = NULL;
} /* end Init_Tablerec */

/* in:  pointer to a nodelist
		 Memory occupied by nodelist pointed to by 'ptr' is released.
	 */ 


void free_nodelist (nodelist *ptr)
{
	nodelist * prev_ptr = NULL;

	while (ptr)
	{
		prev_ptr = ptr;
		ptr = ptr->next;
		free(prev_ptr);
	}
} /* end free_nodelist */


/* in:  pointer to two tablerecs
		 out: two tablrecs with identical values in all their fields, the
				values of each field are equal to the input value of the 
				field in 'oldrec'
	 */
void Copy_Tablerec(tablerec *oldrec, tablerec *newrec)
{


	/* check in values */
	if(CheckTableRecForError(oldrec))
		{
		printf("ERROR Copy_Tablerec   oldrec\n");
		exit(1);
	}


	newrec->path = oldrec->path;
	newrec->qos = oldrec->qos;
	newrec->up = oldrec->up;
	newrec->hopcount = oldrec->hopcount;
	newrec->bw = oldrec->bw;
	newrec->down = oldrec->down;
	newrec->ackhop = oldrec->ackhop;
	newrec->whichPart = oldrec->whichPart;
	newrec->confirmed.value = oldrec->confirmed.value;
	newrec->confirmed.ack.recv = oldrec->confirmed.ack.recv;
	newrec->confirmed.ack.down = oldrec->confirmed.ack.down;
	newrec->confirmed.ack.up = oldrec->confirmed.ack.up;
	newrec->confirmed.conf_recv = oldrec->confirmed.conf_recv;
	newrec->confirmed.conf_sent = oldrec->confirmed.conf_sent;
	newrec->time_connected = oldrec->time_connected;
	newrec->time_received = oldrec->time_received;
	newrec->next = oldrec->next;
	newrec->prev = oldrec->prev;
	newrec->path_bw = oldrec->path_bw;
} /* end Copy_Tablerec */



/*  in:  Pointer to node information
		  out: Node information minus zero bandwidth and invalid entries
	 */
void Clean_Table (node *cnode) 
{
	tablerec *finger, *temp;
	boolean   elim_from_table;


	/* check in values */
	if(CheckTableForError(cnode))
	{
		fprintf(stderr,"ERROR Clean_Table()\n");
		fprintf(stderr,"Bad values passed to function\n");
		exit(1);
	}


	finger = cnode->tablehead;
	while (finger != NULL) 
	{
		elim_from_table = false;
		if (finger->bw == 0)
			elim_from_table = true;
		else if (!finger->confirmed.value &&
		    finger->whichPart == START &&
		    finger->up != NO_NODE)
			elim_from_table = true;
		else if (!finger->confirmed.value &&
		    finger->whichPart == FINISH &&
		    finger->down != NO_NODE)
			elim_from_table = true;

		if (elim_from_table) 
		{
			if (cnode->tablehead == cnode->tabletail) 
			{
				cnode->tablehead = NULL;
				cnode->tabletail = NULL;
				free (finger);
				finger = NULL;
			}
			else if (finger->prev == NULL)  
			{
				/* finger at head of table */
				cnode->tablehead = finger->next;
				free (finger);
				finger = cnode->tablehead;
				finger->prev = NULL;
			}
			else 
			    {  
				/* finger NOT at head of table */
				temp = finger->prev;
				temp->next = finger->next;
				if (finger->next != NULL)  /* NOT at end of table */
					finger->next->prev = temp;
				else cnode->tabletail = temp;
				free(finger);
				finger = temp->next;
			}
		}
		else finger = finger->next;
	}


/* check out values */
	if(CheckTableForError(cnode))
	{
		fprintf(stderr,"ERROR Clean_Table()\n");
		fprintf(stderr,"Bad values created by function\n");
		exit(1);
	}




}  /* end Clean_Table */


/* in:  pointer to node information
		 out: node information sorted in order:
				  lowest numbered path listed first
				  withing a path listing for a given path:
					 confirmed connections listed first
					 acked connections listed next
					 unconnected entries listed last
				  within a connection type (confirmed, acked or unconnected) for
				  a given path:
					 oldest entry listed first
	 */
void Sort_Table(node *cnode) 
{
	tablerec  *finger, *thumb;
	tablerec   temp;




	/* check in values */
	if(CheckTableForError(cnode))
		{
		fprintf(stderr,"ERROR Sort_Table()\n");
		fprintf(stderr,"Bad values passed to function, table contained errors cnode->id:%d\n",cnode->id);
		exit(1);
		}
	




	finger = cnode->tablehead;
	if (finger == NULL) return;
	while (finger->next != NULL) 
	{
		thumb = finger->next;
		while (thumb != NULL) 
		{
			/* Conditions for swapping entries. */
			if 
			    /* 1 */  ((finger->path > thumb->path) ||
			    /* 3 */  (finger->path == thumb->path && 
			    !finger->confirmed.value &&
			    !thumb->confirmed.value
			    && (finger->time_connected > thumb->time_connected)) ||
			    /* 4 */  (finger->path == thumb->path &&
			    !finger->confirmed.value 
			    && thumb->confirmed.value) ||
			    /* 5 */  (finger->path == thumb->path && 
			    finger->confirmed.value > 1 &&
			    thumb->confirmed.value > 1 && 
			    finger->time_connected > thumb->time_connected) || 
			    /* 6 */  (finger->path == thumb->path && 
			    finger->confirmed.value > 1 &&
			    thumb->confirmed.value == CONFIRMED) ||
			    /* 7 */  (finger->path == thumb->path && 
			    finger->confirmed.value == CONFIRMED  &&
			    thumb->confirmed.value == CONFIRMED &&
			    finger->time_connected > thumb->time_connected) ||
			    /* 8 */  (finger->path == thumb->path && !finger->confirmed.value  
			    && !thumb->confirmed.value && thumb->up == NO_NODE 
			    && thumb->down == NO_NODE) ||
			    /* 9 */  (finger->path == thumb->path && !finger->confirmed.value 
			    && !thumb->confirmed.value && 
				 /* Chow: the following additional condition sort entry with 
					(finger->hopcount == thumb->hopcount &&)
					hopcount but it does not help, several cases in layer3.net 
					lost 100% restoration 
					 try to find out why */
				 finger->hopcount == thumb->hopcount &&
			    finger->time_received > thumb->time_received)  ||
				 /* 10 */ (finger->path == thumb->path && 
				 /* Chow replace the following 
					finger->confirmed.value == thumb->confirmed.value &&
					finger->up == thumb->up &&
					finger->hopcount > thumb->hopcount) )
				*/
               finger->confirmed.value == 0 &&
               thumb->confirmed.value == 0 &&
               finger->down == NO_NODE &&
               thumb->down == NO_NODE &&
               finger->hopcount > thumb->hopcount) )

			{

			

				temp.path = thumb->path;
				temp.up = thumb->up;
				temp.hopcount = thumb->hopcount;
				temp.bw = thumb->bw;
				temp.down = thumb->down;
				temp.ackhop = thumb->ackhop;
				temp.whichPart = thumb->whichPart;
				temp.confirmed.value = thumb->confirmed.value;
				temp.confirmed.ack.down = thumb->confirmed.ack.down; 
				temp.confirmed.ack.up = thumb->confirmed.ack.up; 
				temp.confirmed.ack.recv = thumb->confirmed.ack.recv; 
				temp.confirmed.conf_recv = thumb->confirmed.conf_recv; 
				temp.confirmed.conf_sent = thumb->confirmed.conf_sent; 
				temp.time_connected = thumb->time_connected;
				temp.time_received = thumb->time_received;
				temp.path_bw = thumb->path_bw;

			



				thumb->path = finger->path;
				thumb->up = finger->up;
				thumb->hopcount = finger->hopcount;
				thumb->ackhop = finger->ackhop;
				thumb->bw = finger->bw;
				thumb->down = finger->down;
				thumb->whichPart = finger->whichPart;
				thumb->confirmed.value = finger->confirmed.value;
				thumb->confirmed.ack.down = finger->confirmed.ack.down; 
				thumb->confirmed.ack.up = finger->confirmed.ack.up; 
				thumb->confirmed.ack.recv = finger->confirmed.ack.recv; 
				thumb->confirmed.conf_sent = finger->confirmed.conf_sent; 
				thumb->confirmed.conf_recv = finger->confirmed.conf_recv; 
				thumb->time_connected = finger->time_connected;
				thumb->time_received = finger->time_received;
				thumb->path_bw = finger->path_bw;

				finger->path = temp.path;
				finger->up = temp.up;
				finger->hopcount = temp.hopcount;
				finger->ackhop = temp.ackhop;
				finger->bw = temp.bw;
				finger->down = temp.down;
				finger->whichPart = temp.whichPart;
				finger->confirmed.value = temp.confirmed.value;
				finger->confirmed.ack.down = temp.confirmed.ack.down; 
				finger->confirmed.ack.up = temp.confirmed.ack.up; 
				finger->confirmed.ack.recv = temp.confirmed.ack.recv; 
				finger->confirmed.conf_sent = temp.confirmed.conf_sent; 
				finger->confirmed.conf_recv = temp.confirmed.conf_recv; 
				finger->time_connected = temp.time_connected;
				finger->time_received = temp.time_received;
				finger->path_bw = temp.path_bw;
			}  /* if swap */
			thumb = thumb->next;
		} /* end while thumb */
		finger = finger->next;
	} /* end while finger */




	/* check out values */
	if(CheckTableForError(cnode))
	{
		fprintf(stderr,"ERROR Sort_Table()\n");
		fprintf(stderr,"Bad values created by function\n");
		exit(1);
	}



}  /* end Sort_Table */


/* in:  pointer to node information
		 out: node information having been updated by:
				  1)  having any 2 entries for the same path and in the same 
						state involving identical upstream and downstream nodes
						replaced with one entry with the bandwidth = the combined
						bandwidth of the original 2 entries
				 2)   sorting
	 */
void Pack_Table(node *cnode) 
{
	tablerec *finger, *thumb, *temp;

	
	/* check in values*/
	if(CheckTableForError(cnode))
		{
		fprintf(stderr, "ERROR Pack_Table()\n");
		fprintf(stderr,"Bad values passed to function, table contained errors cnode->id:%d\n",cnode->id);
		exit(1);
		}

	


	finger = cnode->tablehead;
	while (finger) 
	{
		thumb = finger->next;
		while (thumb) 
		{
			if (thumb->path == finger->path &&
			    thumb->qos == finger->qos &&
			    thumb->up == finger->up &&
			    thumb->down == finger->down &&
			    thumb->time_connected == finger->time_connected &&
			    thumb->confirmed.value == finger->confirmed.value)
			{
				/* Fix confirm ack and conf_recv fields. */
				if (thumb->confirmed.conf_recv > finger->confirmed.conf_recv)
					finger->confirmed.conf_recv = thumb->confirmed.conf_recv;
				if (thumb->confirmed.ack.recv > finger->confirmed.ack.recv)
					finger->confirmed.ack.recv = thumb->confirmed.ack.recv;
				if (thumb->confirmed.ack.down > finger->confirmed.ack.down)
					finger->confirmed.ack.down = thumb->confirmed.ack.down;
				if (thumb->confirmed.ack.up > finger->confirmed.ack.up)
					finger->confirmed.ack.up = thumb->confirmed.ack.up;
				

			


				finger->bw += thumb->bw;
				temp = thumb->prev;
				temp->next = thumb->next;
				if (thumb->next != NULL)
					thumb->next->prev = temp;
				else cnode->tabletail = temp;
				free (thumb);
				thumb = temp;
			}
			thumb = thumb->next;
		}
		finger = finger->next;
	}
	Sort_Table (cnode);




	/* check out values */
	if(CheckTableForError(cnode))
	{
		fprintf(stderr,"ERROR Pack_Table()\n");
		fprintf(stderr,"Bad values created by function\n");
		exit(1);
	}



}  /* end Pack_Table */





/* in:  pointer to node information
		 Writes node information to files.  Depending values of on debug, 
		 debug_node and type of failure may or may not write to one of the
		 files:
				  BREAK1.OUT
				  BREAK2.OUT
				  CHOOSEN.OUT 
	 */
void Print_Node_Table(node *cnode) 
{
	tablerec *finger;
	char      upstr[NodeNameLength], downstr[NodeNameLength], part;
	FILE * file_ptr;
	
	if(debug)
	{
	printf("break1str=%s break2str=%s\n",break1str,break2str );
	printf("break1_out=%s break2_out=%s\n",break1_out,break2_out );
	}

	if (debug && !strcmp(cnode->name, break1str))
		file_ptr = break1_out;
	else if (debug && !strcmp(cnode->name, break2str))
		file_ptr = break2_out;
	else if (debug_node && choosen == Get_Node_ID(cnode->name))
		file_ptr = choosen_out;
	else 
	    return; 





	fprintf (file_ptr, "\nPATH   UP   HOP BW   DOWN ACKHOP    CONFIRMED");




	fprintf (file_ptr, "   PART     ACKS    CONF_RECV\n");
	finger = cnode->tablehead;
	while (finger != NULL) 
	{
		if (finger->up == NO_NODE) 
			sprintf (upstr, "%s", "   ");
		else 
		    Get_Node_Name (finger->up, upstr);
		if (finger->down == NO_NODE) 
			sprintf (downstr, "%s", "   ");
		else 
		    Get_Node_Name (finger->down, downstr);
		if (finger->whichPart == START)
			part = 'S';
		else if (finger->whichPart == MIDDLE)
			part ='M';
		else 
		    part = 'F';




		fprintf (file_ptr, " %2d   %s   %2d  %2d    %s   %2d         %3d",
		finger->path, upstr, finger->hopcount, finger->bw, downstr,  
		finger->ackhop,
		finger->confirmed.value);




		fprintf (file_ptr, "        %c    %2d/%2d/%2d     %2d\n", part, 
		finger->confirmed.ack.down, 
		finger->confirmed.ack.up,
		finger->confirmed.ack.recv,
		finger->confirmed.conf_recv); 
		finger = finger->next;
	}
} /* end Print_Node_Table */


/* in:  pointer to node information
				values for new Connection Table entry
		 out: updated node information--a new Connection Table new entry 
				is generated using the supplied values and the new entry
				is inserted in its correctly sorted location in the 
				node information.
	 */
void Add_To_Table(node *cnode, int path, int qos, int up, int hopcount, int bw, int down, int ackhop,
int whichPart, int confirmed, boolean forced, int path_bw)

{


	tablerec *temp;


	/* Check in values*/
	if( path < 0 || hopcount < 0 || bw < 0 || up < 0 || down < 0 )
		{
		printf(" ERROR Add_To_Table \n");
		exit(1);
		}

	/* Generate first entry in table. */
	if (cnode->tablehead == NULL) 
		{
		cnode->tablehead  = (tablerec *)malloc(sizeof(tablerec));
		cnode->tablehead->path = path;
		cnode->tablehead->qos = qos;
		cnode->tablehead->up = up;
		cnode->tablehead->hopcount = hopcount;
		
	
		cnode->tablehead->ackhop= 0;
		
		cnode->tablehead->bw = bw;
		cnode->tablehead->down = down;
		cnode->tablehead->whichPart = whichPart;
		cnode->tablehead->confirmed.value = confirmed;
		cnode->tablehead->confirmed.ack.recv = 0;
		cnode->tablehead->confirmed.ack.down = 0;
		cnode->tablehead->confirmed.ack.up = 0;
		if (confirmed == CONFIRMED)
			{
			cnode->tablehead->confirmed.conf_recv = FINAL;
			cnode->tablehead->confirmed.conf_sent = true;
			}
		else
			{
			cnode->tablehead->confirmed.conf_recv = 0;
			cnode->tablehead->confirmed.conf_sent = false;
			}
		if (confirmed)
			cnode->tablehead->time_connected = cnode->time;
		else
			cnode->tablehead->time_connected = (float)0.0;
		cnode->tablehead->time_received = cnode->time;
		cnode->tablehead->path_bw = path_bw;
		cnode->tablehead->next = NULL;
		cnode->tablehead->prev = NULL;
		cnode->tabletail = cnode->tablehead;
		}

	/* Add new entry to end of the table. */
	else 
		{
		temp = cnode->tabletail;
		cnode->tabletail  = (tablerec *)malloc(sizeof(tablerec));
		cnode->tabletail->path = path;
		cnode->tabletail->up = up;
		

		cnode->tabletail->ackhop= 0;
		
		
		cnode->tabletail->hopcount = hopcount;
		cnode->tabletail->qos = qos;
		cnode->tabletail->bw = bw;
		cnode->tabletail->down = down;
		cnode->tabletail->whichPart = whichPart;
		cnode->tabletail->confirmed.value = confirmed;
		cnode->tabletail->confirmed.ack.recv = 0;
		cnode->tabletail->confirmed.ack.down = 0;
		cnode->tabletail->confirmed.ack.up = 0;
		if (confirmed == CONFIRMED)
			{
			cnode->tabletail->confirmed.conf_recv = FINAL;
			cnode->tabletail->confirmed.conf_sent = true;
			}
		else
			{
			cnode->tabletail->confirmed.conf_recv = 0;
			cnode->tabletail->confirmed.conf_sent = false;
			}
		if (confirmed)
			cnode->tabletail->time_connected = cnode->time;
		else
			cnode->tabletail->time_connected = (float)0.0;
		cnode->tabletail->time_received = cnode->time;
		cnode->tabletail->path_bw = path_bw;
		temp->next = cnode->tabletail;
		cnode->tabletail->next = NULL;
		cnode->tabletail->prev = temp;    
	}
	Pack_Table (cnode);
} /* end Add_To_Table */

/*  in:  pointer to node information
				 internal identifcation number for node sending the message
				 pointer to event record containing prior message, the
					message received by 'n'  
				 message type for message 
				 internal identification number for node to receive messsage
				 path information for inclusion in message
		  Converts message previously received by 'n' to message to be sent 
		  by 'n' to 'destination'.  Sends message to 'destination'.
	 */ 
void Forward_Msg (node *cnode, int n, erec *ev, char mtype[], int destination, char packet[])
{
	erec     *hold;
	linkdata *clink;
	char      str1[256];
	char		packet_copy[MAX_PACKET_LEN+1],pack_copy[PACKET_SIZE+1];
	char      temp[256], aPacket[PACKET_SIZE+3];
	char      destination_name[4], msg_id[25]; 
	int       hops, num_packetts, i=0,path,qos,up,bw,down,msg_num, ack_hop;
	boolean   message_sent = true;




	if (strcmp(mtype, "PACK") == 0) 
	{
		if (n == destination) 
		{
			printf("self loop on PACK\n");
			exit(1);
		}
		strcpy(packet_copy, packet);
		num_packetts = strlen(packet_copy) / PACKET_SIZE;

		if(debug)
		{
			printf("Sending PACKmsg from %d to %d at time %f containing the following\n",n,destination, cnode->time);
			for(i=0;i < num_packetts; i++)
			{
				strncpy(pack_copy, &(packet_copy[PACKET_SIZE*i]), PACKET_SIZE);
				parse_packet(pack_copy, &path, &qos,&up,&bw,&down);
				printf("packette: path%d, bw %d\n", path,bw);
			}
		}


	}
	if (debug_node && choosen == n) {
		fprintf(choosen_out, 
			"at %f, %d sent %d %s=%s\n", cnode->time, n, destination, 
			mtype, packet);
		Print_Node_Table(cnode);
		}

	hold = (erec *)malloc(sizeof(erec));
	*hold = *ev;
	copy_symbols(&ev->msg, &hold->msg);
	clink = (linkdata *)head(&cnode->links);
	while (clink && clink->dest != destination)
		clink = (linkdata *)next(&cnode->links);
	if (!clink)
	{
		fprintf(stderr,
		"In Forward_Msg, %d trying to send %s to %d at time %f\n",
		n, mtype, destination, cnode->time);
		exit(1);
	}

	hold->time = cnode->time + PPTOVERRIDE; /* tx side  Port-CPU delay */
	hold->time += PPTOVERRIDE + MPTOVERRIDE; /* rx side delay */

	if (!strncmp(mtype, "CONF", 4))
	{

		/* Get number of message   */ 
		find_symbol_def(msg_id, ev->msg, "ID");
		msg_num = atoi(msg_id);

		strcpy(packet_copy,packet);

		parse_packet(packet_copy, &path, &qos, &up, &bw, &down);


		CONFCount++;
		if (mtype[4] == 'F')
		{
			hold->time += (float)(8.0 * 
				( MSG_HEADER + SINGLE_PACKETTE_PAYLOAD) / 
				(TX_SPEED * 1000.0));
			if(debug)
				printf("Sending CONFF#%d(bw=%d) from %d to %d for path:%d at time %f\n",msg_num, bw, n, destination, path, cnode->time);



		} 
		else 
		{
			find_symbol_def(temp, ev->msg, "TRAIL");
			hold->time += (float)( 8.0 * 
				( strlen(temp) + MSG_HEADER + SINGLE_PACKETTE_PAYLOAD) / 
				(TX_SPEED * 1000.0));
			if(debug)
			{
				if(mtype[4] == 'X')
					printf("Sending CONFX#%d(bw=%d) from %d to %d for path:%d at time %f\n",msg_num, bw, n, destination, path, cnode->time);
				else
					printf("Sending CONF#%d(bw=%d) from %d to %d for path:%d at time %f\n",msg_num, bw, n, destination, path, cnode->time);
			}

		}
	}
	else if (!strncmp(mtype, "FAIL", 4))
	{
		FAILCount++; 
		hold->time += (float)(8.0 * 
		(MSG_HEADER + (strlen(packet)*SINGLE_PACKETTE_PAYLOAD/PACKET_SIZE)) / 
		(TX_SPEED * 1000.0));
	}
	/* Add additional transmission time needed for TRAIL and ACK_BW */
	else if (!strncmp(mtype, "PACK", 4))
	{


		find_symbol_def(temp, ev->msg, "HOPS");
		hops = atoi(temp);
		hops++;
		if (hops > MAX_HOPS)
		{
			message_sent = false;
			return;
		}
		else
		{
			PACKCount++;
			find_symbol_def(temp, ev->msg, "TRAIL");
			hold->time += (float)(8.0 * 
				(MSG_HEADER + strlen(temp)) /
				 (TX_SPEED * 1000.0));
			sprintf(temp, "%d", hops);
			init_symbol(&hold->msg, "HOPS", temp);
			hold->time += (float)(8.0 *  
			     (strlen(temp) + 
				 strlen(packet)*SINGLE_PACKETTE_PAYLOAD/PACKET_SIZE)
				/ (TX_SPEED * 1000.0));
		}
	} 
	else if (!strncmp(mtype, "ACK", 3))
	{
		find_symbol_def(temp, ev->msg, "TRAIL");
		hold->time += (float)(8.0 * 
			(MSG_HEADER + strlen(temp) + SINGLE_PACKETTE_PAYLOAD) / 
			(TX_SPEED * 1000.0));
		ACKCount++;

		/* Get number of message received. */   
		find_symbol_def(msg_id, ev->msg, "ID");
		msg_num = atoi(msg_id);
		strcpy(aPacket,packet);
		parse_ack_packet(aPacket, &path, &qos, &up, &bw, &down, &ack_hop);


		if(debug)
			printf("Sending ACK#%d(bw=%d) from %d to %d for path:%d at time %f\n",msg_num,bw,n,destination, path,cnode->time);

	} 
	else if (!strncmp(mtype, "PDONE", 5))
	{
		PDONECount++; 
		hold->time += (float)(8.0 * 
			(MSG_HEADER + strlen(temp) + SINGLE_PACKETTE_PAYLOAD) / 
			(TX_SPEED * 1000.0));
	}
	if (message_sent)
		msgCount++;

	if (hold->time > clink->last_trans) 
		hold->time += clink->delay;
	else 
	    hold->time = clink->last_trans + clink->delay;

	/* Chow: a quick way to imlement the delay of the forward of a msg
	   by Forward_Delay amount */
	

	hold->time += Forward_Delay;
	clink->last_trans = hold->time - clink->delay;
	init_symbol(&hold->msg, "TYPE", mtype);
	sprintf (str1, " %s", cnode->name);
	init_symbol(&hold->msg, "PATH", str1);
	init_symbol(&hold->msg, "PACK", packet);
	hold->next = destination;
	hold->src = n;
	hold->dst = destination;
	queue_event(hold);

	Get_Node_Name(destination, destination_name);
	find_symbol_def(temp, ev->msg, "ID");
	if(debug)
		printf("%f\t%s #%s to %s:  %s\n", hold->time, mtype, temp, destination_name,
	packet);
	if (debug && length_scenario > 4) 
		if (hold->src == break1)
			fprintf(break1_out, "%f\t%s #%s to %s:  %s\n", hold->time, mtype,
			temp, destination_name, packet);
		else if (hold->src == break2) 
			fprintf(break2_out, "%f\t%s #%s to %s:  %s\n", hold->time, mtype,
			temp, destination_name, packet);
	if (debug_node && hold->src == choosen)
		fprintf(choosen_out, "%f\t%s #%s to %s:  %s\n", hold->time, mtype,
		temp, destination_name, packet);
	if (debug_path && selected_path)
		fprintf(selected_out, "%f\t%s #%s to %s:  %s\n", hold->time, mtype,
		temp, destination_name, packet);

} /* end Forward_Msg */


/*  in:  pointer to node information
				 internal identifcation number for node sending the message
				 pointer to event record containing prior message, the
					message received by 'n'  
				 message type for message 
				 internal identification number for node to receive messsage
				 path information for inclusion in message
			forward delay time in seconds 
		  Converts message previously received by 'n' to message to be sent 
		  by 'n' to 'destination'.  Sends message to 'destination'.
	 */ 
void Forward_Msg_Delay (node *cnode, int n, erec *ev, char mtype[], int destination, char packet[], float delay)
{
	Forward_Delay = (float)delay;
	Forward_Msg(cnode, n, ev, mtype, destination, packet);
	Forward_Delay = (float)0.0;
}

/* in:  node information pointer
				internal identification numbers for nodes which are to
					  have spares converted to working channels
				number of spares to be converted to working channels
		 out: node information updated to show conversion of 'bw' spares
				to 'first' and 'bw' spares to 'second' to working channels
	 */
void Take_Spares (node *cnode, int first, int second, int bw)
{
	linkdata *clink;

	clink = (linkdata *)head(&cnode->links);
	while ((clink->dest != first) && (clink->dest != second))
		clink = (linkdata *)next(&cnode->links);
	clink->work += bw;
	clink->spare -= bw;
	clink = (linkdata *)next(&cnode->links);
	if ((first != NO_NODE) && (second != NO_NODE)) 
	{
		while ((clink->dest != first) && (clink->dest != second))
			clink = (linkdata *)next(&cnode->links);
		clink->work += bw;
		clink->spare -= bw;
	}
} /* end Take_Spares */


/* in:  node information pointer
				internal identification number for nodes with which the
					 number of spares will increase
				number of channels to be converted from working channels to
					 spare channels
		 out: node information updated to show 'bw' channels to 'first' and
				'bw' channels to 'second' converted from working to spare
				channels
	 */
void Give_Spares (node *cnode, int first, int second, int bw)
{
	linkdata *clink;
	int flag = 1;

	clink = (linkdata *)head(&cnode->links);
	while ((clink->dest != first) && (clink->dest != second))
		clink = (linkdata *)next(&cnode->links);
	/* Chow: here clink becomes null on Simple net N0A failure
				 the following is a tempoary fix, need to find out why 
			if (clink == NULL) return;
			while ((clink->dest != first) && (clink->dest != second)) {
				clink = (linkdata *)next(&cnode->links);
				if (clink == NULL) return;
			  }
			*/
	clink->work -= bw;
	clink->spare += bw;
	clink = (linkdata *)next(&cnode->links);
	if ((first != NO_NODE) && (second != NO_NODE)) 
	{
		while ((clink->dest != first) && (clink->dest != second)) {
			clink = (linkdata *)next(&cnode->links);
		}
		clink->work -= bw;
		clink->spare += bw;
	}
} /* end Give_Spares */


/*  in:  node information pointer
				 pointer to tablerec 'intablerec' in the node information
				 pointer to tablerec 'addrec' NOT in the node information
		  out: node information with 'addrec' inserted before 'intablerec'
	 */
void Insert_Before_Tablerec(node *cnode, tablerec *intablerec,
tablerec *addrec)
{
	

	/* Check in values */

	if(CheckTableRecForError(addrec))
		{
		fprintf(stderr,"ERROR Insert_Before_Tablerec()\n");
		fprintf(stderr,"Bad tablerec passed to function, \"addrec\" contained errors \n");
		exit(1);
		}
	if(CheckTableRecForError(intablerec))
		{
		fprintf(stderr,"ERROR Insert_Before_Tablerec()\n");
		fprintf(stderr, "Bad tablerec passed to function, \"intablerec\" contained errors \n");
		exit(1);
		}
	



	if (intablerec->prev)
		intablerec->prev->next = addrec;
	else
		cnode->tablehead = addrec;
	addrec->prev = intablerec->prev;
	intablerec->prev = addrec;
	addrec->next = intablerec;

	if(CheckTableForError(cnode))
		{
		fprintf(stderr,"ERROR End of Insert_Before_Tablerec()\n");
		fprintf(stderr, "table contained errors\n");
		exit(1);
		}

} /* end Insert_Before_Tablerec */


/*  in:  node information pointer
				 pointer to tablerec 'intablerec' in the node information
				 pointer to tablerec 'addrec' NOT in the node information
		  out: node information with 'addrec' inserted after 'intablerec'
	 */
void Insert_After_Tablerec(node *cnode, tablerec *intablerec, 
tablerec *addrec)
{
	

	/* check in values */

	if(CheckTableRecForError(addrec))
		{
		fprintf(stderr,"ERROR Insert_Before_Tablerec()\n");
		fprintf(stderr,"Bad tablerec passed to function, \"addrec\" contained errors \n");
		exit(1);
		}
	if(intablerec)
	{
		if(CheckTableRecForError(intablerec))
			{
			fprintf(stderr,"ERROR Insert_Before_Tablerec()\n");
			fprintf(stderr, "Bad tablerec passed to function, \"intablerec\" contained errors \n");
			exit(1);
			}
	}

	if (cnode->tablehead == NULL) {
		cnode->tablehead = addrec;
		cnode->tabletail = addrec;
		addrec->next = NULL;
		addrec->prev = NULL;
		return;
	}
	if (intablerec->next)
		intablerec->next->prev =  addrec;
	else
		cnode->tabletail = addrec;

	addrec->next = intablerec->next;
	addrec->prev = intablerec;
	intablerec->next = addrec;
} /* end Insert_After_Tablerec */


/* in:  node information pointer
				pointer to tablerec 'obsolete' in node information
		 out: node information without 'obsolete' tablerec
	 */
void Remove_Tablerec(node *cnode, tablerec *obsolete)
{
	if (cnode->tablehead == obsolete)
	{
		cnode->tablehead = obsolete->next;
		if (obsolete->next != NULL) obsolete->next->prev = NULL;
	}
	else if (cnode->tabletail == obsolete)
	{
		cnode->tabletail = obsolete->prev;
		obsolete->prev->next = NULL;
	}
	else
	{
		obsolete->prev->next = obsolete->next;
		obsolete->next->prev = obsolete->prev;
	}

	free(obsolete);
} /* end Remove_Tablerec */

boolean In_Trail(int n, char *trail) {
   int trail_length;
   int index;

   trail_length = (strlen(trail) + 1) / NodeNameLength;
   for (index = 0; index < trail_length; index++){
      if (n == Get_Node_ID(&(trail[NodeNameLength * index]))) return true;
   }
   return false;
}

/*
		in:  array of node identification numbers for nodes that may not
				  be used as upstream nodes
			  length of the array
			  identification number of potential upstream node  
		returned:  true if 'up' is available for use as upstream node,
							 i.e. up is not found in the unavailables array
					  else
						  false 
	 */
boolean up_available(int up, int num_unavailable, int *unavailables)
{
	int i;

	for (i = 0; i < num_unavailable; i++)
	{
		if (up == unavailables[i])
			return false;
	} /* end for i */
	return true;
} /* end up_available */


int searchWhichPart(node *cnode, int path) 
{
	tablerec *t;
	t = cnode->tablehead;
	while (t && t->path != path) t = t->next;
	if (t == NULL) return -1;
	return t->whichPart;
}

int accumulated_bw(node *cnode, int path, int up) {
	tablerec *finger = get_start_finger(cnode, path);
	int bw = 0;
	while (finger && finger->path == path) {
		if (finger->up == up && finger->confirmed.value > 0) 
			bw += finger->bw;
		finger =finger->next;
	}
	return bw;
}



/* function to create a random value between downlimit and uplimit */
float random_delay(float uplimit, float downlimit )
{
	int random;
	float result;
	float range;

	range = uplimit - downlimit;
	random = rand();
	result = (float)random / (float)RAND_MAX;
	return (downlimit + result * range);
}




/* Function to initiate and create a timer record
        if it already exists a timer record for path, the function returns FALSE
        if it does not exists a timer record, a record is created / initiated and
        put at the end of the list, the function returns TRUE 
        
        IN: node, path
        OUT: FALSE if it exists a record. if not TRUE
*/

boolean Init_timer_rec( node *cnode, int Path)
{
TimerRecord *finger;
boolean value = TRUE;
TimerRecord	*timer_rec;

/* set finger to point at timer list */
finger = cnode -> RecordHead;

/* if there exists no records, create/init one and set timer to point at it */
if(!finger)
        {
        /* create record */
		if(RANDOMDELAY)
			DELAY = random_delay(RAND_DEL_LIMIT, RAND_DEL_DOWN_LIMIT);



        timer_rec = (TimerRecord *)malloc(sizeof(TimerRecord));
        
        /* init record */
        timer_rec -> path = Path;
        timer_rec -> initiated = TRUE;
        timer_rec -> expire_time = cnode->time + DELAY;
        timer_rec -> delay = DELAY;
        timer_rec -> next = NULL;
        timer_rec -> prev = NULL;
        cnode -> RecordHead = timer_rec;
        
		if(debug)
		{
			printf("\nInit_timer_rec\n");
			printf("path=%d node=%d\n", Path, cnode->id);
			printf("delay = %f, current time = %f expire time =%f\n", DELAY, cnode->time, timer_rec->expire_time);
		}

        }
        
/* there exist records, go through list and append to the end of the list */
else
        {
        while (finger && finger -> path != Path && finger -> next != NULL)
                finger = finger -> next;
                
        /* it already exists a record of path */
        if(finger -> path == Path)
                {
			printf("It already exists a record for path: %d at node:%d\n",Path,cnode->id);
                value = FALSE; 
                return value;
                }
                
        /* create record */
		if(RANDOMDELAY)
			DELAY = random_delay(RAND_DEL_LIMIT, RAND_DEL_DOWN_LIMIT);


        timer_rec = (TimerRecord *)malloc(sizeof(TimerRecord));
        
        /* init record */
        timer_rec -> path = Path;
        timer_rec -> initiated = TRUE;
        timer_rec -> expire_time = cnode->time + DELAY;
        timer_rec -> delay = DELAY;
        timer_rec -> next = NULL;
        timer_rec -> prev = finger;
        finger -> next = timer_rec;

		if(debug)
		{
			printf("\nInit_timer_rec\n");
			printf("path=%d node=%d\n", Path, cnode->id);
			printf("delay = %f, current time = %f expire time =%f\n", DELAY, cnode->time, timer_rec->expire_time);
		}

        }
        
        
/* return status value */        
return value;
}
   



/* Function to check if the timer for a path at a node has expired */
/* returns TRUE if the timer has expired, else FALSE */
boolean Timer_expired( node *cnode, int path)
{
	TimerRecord *finger;
	boolean expired = FALSE;
	float present_time = cnode->time;

	/* set finger to point at timer list */
	finger = cnode -> RecordHead;

	/* find record for path */
	while (finger && finger -> path != path)
        finger = finger -> next;

	/* check if timer has expired */
	if( finger && finger -> path == path && finger -> initiated == TRUE)
        {
        if(finger -> expire_time <= present_time )
                expired = TRUE;
        }
	/* return value */
	return expired;
}           




/* function to check if timer has been initiated
        IN:     node, path
        OUT:    returns TRUE if timer has been initiated for path at node, else FALSE
        */
boolean Timer_initiated( node *cnode, int path)
{
	TimerRecord *finger;
	boolean initiated = FALSE;
 
	/* set finger to point at timer list */
	finger = cnode -> RecordHead;

	/* find record for path */
	while (finger && finger -> path != path)
        finger = finger -> next;

	/* check if there exists an entry and if the timer is initiated */
	if( finger && finger -> path == path && finger -> initiated == TRUE) 
			initiated = TRUE;
        
	return initiated;
}	   



/* dummy packet used by Initiate_timer_event() to keep track of
path,bw and qos */
void create_dummy_packet(char *string, int path, int qos, int bw)
{
	sprintf(string, "%5d%5d%5d%5d%5d",path,qos,0,bw,0);
}




/* parse the dummy packet */
void parse_dummy_packet( char *string, int *path, int *qos, int *bw)
{
	int i;
	char pathfield[PACKET_FIELD_SIZE+1];
	char qosfield[PACKET_FIELD_SIZE+1];
	char bwfield[PACKET_FIELD_SIZE+1];
	string[PACKET_SIZE]='\0';
	for(i = 0; i < 5; i++)
	{
		pathfield[i] = string[i];
		qosfield[i] = string[i +PACKET_FIELD_SIZE];
		bwfield[i] = string[i +3*PACKET_FIELD_SIZE];
	}
	pathfield[PACKET_FIELD_SIZE] = '\0';
	qosfield[PACKET_FIELD_SIZE] = '\0';
	bwfield[PACKET_FIELD_SIZE] = '\0';
	*path = atoi(pathfield);
	*qos = atoi(qosfield);
	*bw = atoi(bwfield);
}





/* function for getting the delay time for path at node 
        (Used when the delay is set by random)
   IN:  node, path
   OUT: delay, if timer has been initiated and there exists a record
               else -1.0
*/
float Timer_delay( node *cnode, int path)
{
	TimerRecord *finger;
	float delay = (float)(-1.0);

	/* set finger to point at timer list */
	finger = cnode -> RecordHead;

	/* find record for path */
	while (finger && finger -> path != path)
        finger = finger -> next;

	/* check if there exists an entry and if the timer is initiated */
	if( finger && finger -> path == path && finger -> initiated == TRUE) 
        delay = finger -> delay;        

	/* returns value, if there exists no record or timer has not been initiated
	the function return -1.0 */
	return delay;
}

/* Function used for debugging, prints the status on every outgoing link from cnode */
void Print_links_variables( node *cnode)
{
	inode *first, *nextitem, *last;
	linkdata *test;
	int t,i=2;

	t=cnode->links.count;
	printf("\ncnode->links.count = %hu\n",cnode->links.count); 
	printf("=================\n");

	first = cnode->links.head;
	last = cnode->links.tail;
	nextitem = first->next;
	

	test = (linkdata*)(first->e);
	printf("node%d - node %d\n",cnode->id, test->dest );
	printf("link->active =%hu\n", test->active);
	printf("link->work = %d\n",test->work);
	printf("link->spare = %d\n\n", test->spare);
	
	

	while(nextitem!= last)
	{
	test = (linkdata*)(nextitem->e);
	printf("node%d - node%d\n",cnode->id, test->dest );
	printf("link%d->active =%hu\n", i, test->active);
	printf("link%d->work = %d\n",i ,test->work);
	printf("link%d->spare = %d\n\n", i, test->spare);
	i=i+1;
	nextitem = nextitem->next;
	}

	test = (linkdata*)(last->e);
	printf("node%d - node%d\n",cnode->id, test->dest );
	printf("link->active =%hu\n", test->active);
	printf("link->work = %d\n",test->work);
	printf("link->spare = %d\n\n", test->spare);
}




/* Function to test if the link node-up got available spares */
/* if so return TRUE, else FALSE */
int Test_for_spares( node *cnode, int up)
{
inode *first,*nextitem,*last;
linkdata *data;

first = cnode->links.head;
last = cnode->links.tail;
nextitem = first->next;

/* Test first item in list */
data = (linkdata*)(first->e);
if(data->spare && data->active && data->dest == up)
	return TRUE;
/* Test last item in list */
data = (linkdata*)(last->e);
if(data->spare && data->active && data->dest == up)
	return TRUE;	

/* Go through the "middle" nodes in list */
while(nextitem!= last)
	{
	data = (linkdata*)(nextitem->e);
	if(data->spare && data->active && data->dest == up)
		return TRUE;
	nextitem = nextitem->next;
	}

return FALSE;
}



tablerec *Get_suitable_entry_with_min_hop_for_ack(node *cnode, int path, 
								int ackhop, int *unavailables, int down, 
								int trail_length)
{
	linkdata *clink=NULL;
	tablerec *startfinger, *minhopfinger=NULL,*finger = cnode->tablehead;

	/* Get first entry with spares available */


	

	while (finger && finger->path < path)
		finger = finger->next;

	startfinger = finger;

	while(finger && finger->path == path)
	{
		if(finger->confirmed.value == 0 && finger->up != NO_NODE &&
			(up_ack(cnode, path, finger->up) <= ACK_LIMIT ) &&
			(ackhop + finger->hopcount <= MAX_HOPS) )
		{
			if(up_available(finger->up, trail_length, unavailables) 
				&& finger->up != down)
			{
				if( Test_for_spares( cnode, finger->up) )
					minhopfinger = finger;
			}
		}
	finger = finger->next;
	}
	
	/* There exists no entry with available spares */
	if(!minhopfinger)
		return NULL;


	finger = startfinger;

	while (finger && finger->path == path)
	{
		if(finger->confirmed.value == 0&& finger->up != NO_NODE &&
			(up_ack(cnode, path, finger->up) <= ACK_LIMIT ) &&
			(ackhop + finger->hopcount <= MAX_HOPS) ) 
		{

			if(up_available(finger->up, trail_length, unavailables) 
				&& finger->up != down)
			{

				if( Test_for_spares(cnode, finger->up) )
				{
					if(finger->hopcount <= minhopfinger->hopcount)
						minhopfinger = finger;
				}	
			}
		}
		finger = finger->next;
	}


	return minhopfinger;
}









tablerec *Get_entry_min_hop_and_spares_suitable_for_retry(node *cnode, int upnode, int entry_ackhop, int path, int entrydown, char *trail)
{
	linkdata *clink=NULL;
	tablerec *minhopfinger=NULL, *startfinger, *finger = cnode->tablehead;

	/* Get first entry with spares available */




	while (finger && finger->path < path)
		finger = finger->next;

	startfinger = finger;
	while(finger && finger->path == path)
	{
		if(finger->confirmed.value == 0 && finger->up != NO_NODE && 
		!In_Trail(finger->up, trail) &&finger->up != entrydown && 
		finger->confirmed.conf_recv == 0)
		{
			if(up_ack(cnode, path, finger->up) <= ACK_LIMIT)
			{
				if( (finger->up==upnode && Only_available_entry(cnode, finger)) || finger->up != upnode)
				{
					if( Test_for_spares( cnode, finger->up) && 
						finger->hopcount+entry_ackhop<=MAX_HOPS )
						minhopfinger = finger;
				}
			}
		}
	finger = finger->next;
	}
	
	/* There exists no entry with available spares */
	if(!minhopfinger)
		return NULL;

 


	/* Start looking for entry with min hop */
	
	finger = startfinger;


	while (finger && finger->path == path)
	{
		if(finger->confirmed.value == 0 && finger->up != NO_NODE && 
		!In_Trail(finger->up, trail) && finger->up != entrydown 
		&&finger->confirmed.conf_recv == 0)
		{

			if(up_ack(cnode, path, finger->up) <= ACK_LIMIT)
			{
				if( (finger->up==upnode && Only_available_entry(cnode, finger)) || finger->up != upnode)
				{
					if( Test_for_spares(cnode, finger->up)&& 
						finger->hopcount+entry_ackhop<=MAX_HOPS )
					{
						if(finger->hopcount <= minhopfinger->hopcount)
							minhopfinger = finger;
					}
				}
			}	
		}
		finger = finger->next;
	}


	return minhopfinger;
}





/* Function to get the unconnected entry with spares available for the up node
        
        IN: node, path
        OUT: pointer to table entry with min number of hops, returns NULL if no entry or 
		it has no spare left
        */
tablerec *Get_unconnected_entry_with_min_hop_and_spares(node *cnode, int path)
{
	linkdata *clink=NULL;
	tablerec *minhopfinger=NULL,*finger = cnode->tablehead;

	/* Get first entry with spares available */


	while (finger && finger->path < path)
		finger = finger->next;
	while(finger && finger->path == path)
	{
		if(finger->confirmed.value == 0 && finger->up != NO_NODE)
		{
		if( Test_for_spares( cnode, finger->up) )
			minhopfinger = finger;
		}
	finger = finger->next;
	}
	
	/* There exists no entry with available spares */
	if(!minhopfinger)
		return NULL;

	/* Reset finger */
	finger = cnode->tablehead;


	/* Start looking for entry with min hop */
	while (finger && finger->path < path)
		finger = finger->next;


	while (finger && finger->path == path)
	{
		if(finger->confirmed.value == 0)
		{
			if( Test_for_spares(cnode, finger->up) )
			{
				if(finger->hopcount <= minhopfinger->hopcount)
					minhopfinger = finger;
			}	
		}
		finger = finger->next;
	}

	/* check return data */
	if(CheckTableRecForError(minhopfinger))
	{
		fprintf(stderr,"ERROR Get_unconnected_entry_with_min_hop_and_spares()\n");
		fprintf(stderr,"minhopfinger contained bad data \n");
		exit(1);
	}

	return minhopfinger;
}


/* function to init the "actual bw field" (not acc bw) */
/* used by ACK Processing                                                               */
/* IN: pointer to eventrecord, "actual bw"                              */
/* OUT: no out values                                                                   */
void InitActualBw(erec *ev, int bw)
{
char buffer[25];

        sprintf(buffer, "%3d", bw);
        init_symbol(&ev->msg,"ABW", buffer);
}


/* function to get the actual bw */
/* IN: pointer to event record   */
/* OUT: "actual bw"                              */
int GetActualBw(erec *ev)
{
char buffer[25];
int bw;

        find_symbol_def(buffer, ev->msg, "ABW");
        bw = atoi(buffer);
        return bw;
}




/* Function to initiate timer, setup Timer_expire_event for path.

        IN: node, path
        OUT: TRUE if init was successfull , else FALSE
        */                                             
boolean Initiate_timer_event(node *cnode, erec *ev, int path, int bw, int qos) 
{
boolean status=TRUE;
char aPacket[PACKET_SIZE+1];
float dlay;
int dlystat;

if( !Init_timer_rec( cnode, path) )
        {
		printf("ERROR Init_timer_rec \n");
        status = FALSE;      
        return status;
        } 

/* Timer_delay returns -1 if error */
dlay = Timer_delay( cnode, path);

dlystat = (int) dlay;
if(dlystat == -1)
	{
	printf("ERROR getting delay\n");
	status = FALSE;
	return status;
	}
ee = (erec *)malloc(sizeof(erec));
ee -> time = cnode->time + dlay;
ee -> src = 0;
ee -> dst = cnode -> id;
ee -> next = cnode -> id;
copy_symbols(&ev->msg, &ee->msg);
init_symbol(&ee ->msg, "TYPE", "TIEX");
create_dummy_packet(aPacket,path,qos,bw);
init_symbol(&ee->msg, "PACK", aPacket);        
queue_event(ee);
return status;
}


void TIMER_EXP_Processing(erec *ev,int  n, node *cnode)
{
char thePacket[PACKET_SIZE+1], aPacket[PACKET_SIZE+3];
linkdata *clink;
int up, bw, path,qos, ack_bw, MoreBwNeeded, fingerBw;
path_ack_info *ack_sent;
tablerec *startfinger, *tempfinger, *finger;


/* INIT */
MoreBwNeeded = TRUE;
startfinger = NULL;
tempfinger = NULL;
finger = NULL;
ack_sent = cnode->acks_sent;

/* get information from dummy packet */ 
find_symbol_def(thePacket, ev->msg, "PACK");
parse_dummy_packet(thePacket, &path, &qos, &bw);





if(debug)
	printf("TIMER EXPIRED for path:%d at node:%d time:%f\n",path,cnode->id,ev->time);


ack_bw = bw;




init_symbol(&ev->msg, "TRAIL", cnode->name);

/* if MoreBwNeeded becomes TRUE it means that we need to find another
table entry for path and send another ACK along that "path"
to fulfill the total amount of bw requested, some bw can have been taken
by others during the delay of the ACK */  
while(MoreBwNeeded)
	{
		MoreBwNeeded=FALSE;

		finger = Get_unconnected_entry_with_min_hop_and_spares(cnode,  path);
		if(!finger)
		{
		/* No up nodes with spares left available */
		printf("No links with available spares available\n");


		}



		/* There existed "paths" for path with spares avalable */
		if(finger)
		{

			up = finger->up;
		
			fingerBw = ack_bw;

			if(finger->bw < ack_bw)
			{
				MoreBwNeeded=TRUE;
				fingerBw = finger->bw;
			}
			clink = (linkdata*) head(&cnode->links);
			while(clink->dest != up)
				clink = (linkdata*)next(&cnode->links);

			if(clink && clink->active && clink->spare)
			{
				char creqStr[6];
				int connect_bw = fingerBw > clink->spare ? clink->spare : fingerBw;

				if(fingerBw > connect_bw)
					MoreBwNeeded=TRUE;

				Take_Spares(cnode, up, NO_NODE, connect_bw);
				ack_bw -= connect_bw;
				if(finger->bw > connect_bw)
				{
					tempfinger = (tablerec*)malloc(sizeof(tablerec));
					Copy_Tablerec(finger, tempfinger);
					tempfinger->bw -=connect_bw;
					tempfinger->time_received = cnode->time;
					Insert_After_Tablerec(cnode, finger, tempfinger);
					finger->bw = connect_bw;
					
				}
				finger->time_connected = cnode->time;
				finger->confirmed.value = ACKED;

			

				tempfinger = startfinger;
				while(tempfinger && tempfinger->path  == path && tempfinger->up != NO_NODE)
					tempfinger = tempfinger->next;
				if(tempfinger)
				{
					if(tempfinger->bw <= connect_bw)
						Remove_Tablerec(cnode, tempfinger);
					else
						tempfinger->bw -= connect_bw;
				}	

			
				finger->confirmed.ack.up = cnode->creq;

				sprintf(creqStr, "%2d", cnode->creq++);
				init_symbol(&ev->msg, "ID", creqStr);

				/* acc bw */
				if(connect_bw != accumulated_bw(cnode, path, up))
				{
					fprintf(stderr,"ERROR TIMER_EXP_Processing\n");
					fprintf(stderr,"connect_bw != accumulated_bw() this should not be possible \n");
					exit(1);
				}



				InitActualBw(ev, finger->bw);
				sprintf(aPacket,"%5d%5d%5d%5d%5d%2d", path, qos, NO_NODE, connect_bw, n, 1);
				Forward_Msg(cnode, n, ev, "ACK", up, aPacket);
				add_up_ack(cnode, path, up);
				if(!ack_sent)
				{
					path_ack_info *new_info = (path_ack_info*)malloc(sizeof(path_ack_info));

					new_info->next = NULL;
					new_info->num_acks = 1;
					new_info->uplist = NULL;
					new_info->downlist = NULL;
					new_info->path = path;

					if(!cnode->acks_sent)
						cnode->acks_sent = new_info;
					else
					{
						ack_sent = cnode->acks_sent;
						while(ack_sent->next)
							ack_sent = ack_sent->next;
						ack_sent->next = new_info;
					}

				}/*end if(!ac..*/
			} /* end if(clink...*/
		}/* enf if(finger...)*/
		Pack_Table(cnode);
	}/*End while(...*/


}








/*  in: erec containing failure detection message received by 'n' 
				internal identification number and pointer to node information
					for same node
		  If failure detection message contains information about the loss of
		  a node, new failure detection messages are constructed for each of
		  the lost links and sent to the remaining active node on the end of
		  each lost link.
		  If the failure detection message contains information about the loss
		  of a link, the lost channels are marked inactive (unusable).  If 'n' 
		  is a MIDDLE node on any of the lost paths, the channels in the 
		  remaining active links on the lost paths are released as spares and 
		  a FAIL message in sent to the affected nodes on the lost paths.  If
		  'n' is a START node on any of the lost paths, 'n' initiates
		  broadcast of a PACK message.
	 */
static void SIM_Processing(erec *ev, int n, node *cnode)
{
	static boolean not_stopped = true;
	char value[256];
	erec *hold;
	int v;
	char aPacket[PACKET_SIZE+1];
	int err; 
	linkdata *clink;
	tablerec *finger;
	char STR1[256];
	int receiver_id,
	i;
	Packet_info *fails = NULL,
	*packs = NULL,
	*last  = NULL,
	*present = NULL,
	*previous = NULL;

	
	
	/* queue event to stop simulation */
	if (not_stopped) 
	{
		hold = (erec *)malloc(sizeof(erec));
		*hold = *ev;
		copy_symbols(&ev->msg, &hold->msg);
		init_symbol(&hold->msg, "TYPE", "STOP");
		init_symbol(&hold->msg, "PATH", " SIMULATION STOP");
		hold->time = STOP_TIME;
		hold->next = n; 
		hold->src = 0;
		hold->dst = n;
		queue_event(hold);
		not_stopped = false;
	} /* end if not_stopped */


	find_symbol_def (value, ev->msg, "ACTION");
	if (!strcmp (value, "BREAK")) 
	{
		err = (sscanf(find_symbol_def(STR1, ev->msg, "ID"), "%ld", &v) == 0);
		clink = (linkdata *) head (&cnode->links);
		while (clink->dest != v)
			clink = (linkdata *) next (&cnode->links);
		clink->active = false;

		/* n is the node that received the SIM msg
					  v is the node at the other end of the break 
					  set msg id to 'nv' or 'vn' */
					  
		/* Chow: %3d%3d cause problem, since there may be space characters
			between first node and second node, and in PACK_Processing code
			sprintf(packfinger->id, "%s", packID); is used to copy the packID */

		if (n > v)
			sprintf (STR1, "%03d%03d", n, v);
		else
			sprintf (STR1, "%03d%03d", v, n);  
		init_symbol (&ev->msg, "ID", STR1);

		finger = cnode->tablehead;
		while (finger) 
		{
			receiver_id = NO_NODE;
			if (finger->whichPart == MIDDLE && (finger->up == v ||
			    finger->down == v))
			{
				if (finger->up == v) 
				{
					if (finger->path == selected)
					{
						selected_path = true;
						if (debug) Print_Table(cnode);
					}
					sprintf(aPacket, "%5d%5d%5d%5d%5d", 
						finger->path, finger->qos, NO_NODE, finger->bw, n);
					if (reuse)
						Give_Spares (cnode, finger->up, finger->down, finger->bw);
					receiver_id = finger->down;
				} /* end if finger->up == v */

				else 
				    {
					if (finger->path == selected)
					{
						selected_path = true;
						if (debug) Print_Table(cnode);
					}
					sprintf(aPacket, "%5d%5d%5d%5d%5d", 
						finger->path, finger->qos, n, finger->bw, NO_NODE);
					if (reuse)
						Give_Spares (cnode, finger->up, finger->down, finger->bw);
					receiver_id = finger->up;
				} /* end else */

				if (!fails)
				{
					fails = (Packet_info *) malloc(sizeof(Packet_info));
					fails->receiver_id = receiver_id;
					strcpy(fails->thePacket, aPacket);
					fails->next = NULL;
				}
				else
				{
					present = fails;
					while (present && present->receiver_id != receiver_id)
					{
						previous = present;
						present = present->next;
					}
					if (present && present->receiver_id == receiver_id)
					{
						if (strlen(present->thePacket) + PACKET_SIZE <=   
						    MAX_PACKET_LEN)
							strcat(present->thePacket, aPacket);
						else
						{
							Forward_Msg(cnode, n, ev, "FAIL", receiver_id,
							present->thePacket);
							strcpy(present->thePacket, aPacket);
						}
					} /* end if fails */
					else
					{
						last = (Packet_info *) malloc (sizeof(Packet_info));
						last->next = NULL;
						previous->next = last;
						last->receiver_id = receiver_id;
						strcpy(last->thePacket, aPacket);
					} /* end not fails */
				} /* end else fails */

			} /* end if whichPart == MIDDLE */
			else if (finger->whichPart == START && finger->down == v)
			{
				linkdata *clink;

				if (reuse)
					Give_Spares (cnode, finger->up, finger->down, finger->bw);
				finger->confirmed.value = 0;
				finger->confirmed.conf_recv = 0;
				finger->confirmed.conf_sent = false;
				finger->down = NO_NODE;
				/* set msg id to 'nv' or 'vn' */
				if (n > v)
					sprintf (STR1, "%03d%03d", n, v);
				else
					sprintf (STR1, "%03d%03d", v, n);  
				init_symbol (&ev->msg, "ID", STR1);
				init_symbol(&ev->msg, "TRAIL", cnode->name);
				init_symbol(&ev->msg, "HOPS", "0");
				sprintf(aPacket, "%5d%5d%5d%5d%5d", 
					finger->path, finger->qos, n, finger->path_bw, NO_NODE);

				clink = (linkdata *) head (&cnode->links);
				while (clink)
				{
					if (clink->active)
					{
						if (!packs)
						{
							packs = (Packet_info *) malloc(sizeof(Packet_info));
							packs->receiver_id = clink->dest;
							strcpy(packs->thePacket, aPacket);
							packs->next = NULL;
						} /* end if not packs */
						else
						{
							present = packs;
							while(present && present->receiver_id != clink->dest)
							{
								previous = present;
								present = present->next;
							}
							if (present && present->receiver_id == clink->dest)
							{
								if (strlen(present->thePacket) + PACKET_SIZE <= 
								    MAX_PACKET_LEN)
									strcat(present->thePacket, aPacket);
								else
								{
									Forward_Msg(cnode, n, ev, "PACK", clink->dest,
									present->thePacket);
									strcpy(present->thePacket, aPacket);
								}
							} /* end if receiver_id == clink->dest */
							else
							{
								last = (Packet_info *) malloc (sizeof(Packet_info));
								last->next = NULL;
								previous->next = last;
								last->receiver_id = clink->dest;
								strcpy(last->thePacket, aPacket);
							} /* end else receiver_id = clink->dest not found */
						} /* end else packs */
					}
					clink = (linkdata *) next (&cnode->links);
				} /* end while clink */
			} /* end else if whichPart == START */
			else if (finger->whichPart == FINISH && finger->up == v)
			{
				if (reuse)
					Give_Spares (cnode, finger->up, finger->down, finger->bw);
				finger->up = NO_NODE;
				finger->confirmed.value = 0;
				finger->confirmed.conf_recv = 0;
				finger->confirmed.conf_sent = false;
			}
			if (finger->whichPart == MIDDLE &&(finger->up == v || finger->down
			    == v))
			{
				tablerec *tempfinger = finger;

				if (finger->next)
					finger = finger->next;
				else
					finger = NULL;
				Remove_Tablerec(cnode, tempfinger);
			}
			else
				finger = finger->next;
		}
		Pack_Table(cnode);
		if (fails)
		{
			present = fails;
			while(present)
			{
				previous = present;
				present = present->next;
				Forward_Msg(cnode, n, ev, "FAIL", previous->receiver_id,
				previous->thePacket);
				free(previous);
			} /* end while fails */
		}
		if (packs)
		{
			present = packs;
			while(present)
			{
				previous = present;
				present = present->next; 
				Forward_Msg(cnode, n, ev, "PACK", previous->receiver_id,
				previous->thePacket);
				free(previous);
			} /* end while packs */
		}
		return;
	}
	else if (!strcmp (value, " DEAD")) 
	{
		/* Simulate node break (DEAD) by multiple link breaks (ACTIONS). */
		for (i = 1; i <= cnode->links.count; i++) 
		{
			clink = (linkdata *) recall (&cnode->links, i);
			clink->active = false;
			hold = (erec *) malloc (sizeof (erec));
			hold->time = ev->time  + clink->delay + noise (); 
	/* chow: add link delay time, not all neighbors detect at the same time */
			hold->src = 0;
			hold->dst = clink->dest;
			hold->next = clink->dest;
			hold->msg = NULL;
			init_symbol (&hold->msg, "ID", int2str_(STR1, n));
			init_symbol (&hold->msg, "ACTION", "BREAK");
			init_symbol (&hold->msg, "TYPE", " SIM");
			init_symbol (&hold->msg, "PATH", " INTERNAL FAULT DETECTION");
			queue_event (hold);
		}
		return;
	}
} /* End SIM_Processing */


/* in:  identification number of the FAIL message
				erec containing FAIL message
				internal node identification number and pointer to node
					information for same node
		 Converts working channels on lost path/paths to spare channels.  
		 If the FAIL message contains multiple packettes, the packettes
		 in the FAIL message will be considered one by one.    
		 The 'fails' and 'packs' data structures keep track of which
		 nodes receive which of the updated packettes in a FAIL or a
		 PACK message.
		 If the node is a MIDDLE node on the lost path in the packette
		 being processed, an addition is made to the 'fails' data
		 structure for the node closer to the end of the lost path--if
		 the packette is from a node closer to the FINISH node, an entry
		 to 'fails' is added for the node closer to the START node on the
		 lost path OR if the packette is from the node closer to the
		 START node, an entry to fails is added for the node closer to the
		 FINISH node.
		 If the node is a START node for the lost path in the packette being
		 processed, an addition is made to the 'packs' data structure for
		 each of the node's immediate neighbors.
		 When all packettes in the received message have been processed,
		 FAIL and PACK messages are sent. 
	 */
static void FAIL_Processing(char *packID, erec *ev, int n,
node *cnode)
{
	char newPacket[PACKET_SIZE+1], 
	aPacket[PACKET_SIZE+1],
	thePacket[MAX_PACKET_LEN + 1];
	int i;
	int path, 
	qos,
	up, 
	bw, 
	down,
	whichPart, 
	path_bw,
	num_packettes,
	receiver_id;  
	tablerec *finger = NULL,
	*tempfinger = NULL;
	Packet_info *fails = NULL,
	*packs = NULL,
	*present = NULL,
	*previous = NULL,
	*new_info = NULL;

	




	find_symbol_def(thePacket, ev->msg, "PACK"); 
	if (debug_node && choosen == n)  {
		fprintf(choosen_out, 
			"at %f, %d received FAIL=%s\n", cnode->time, n, thePacket);
		Print_Node_Table(cnode);
		}
	num_packettes = strlen(thePacket) / PACKET_SIZE;
	for(i = 0; i < num_packettes; i++)
	{
		strncpy(newPacket, &(thePacket[PACKET_SIZE * i]), PACKET_SIZE);
		parse_packet(newPacket, &path, &qos, &up, &bw, &down);
		if (debug_path && path == selected) {
			selected_path = true;
			Print_Table(cnode);
		}

		sprintf(aPacket, "%5d%5d%5d%5d%5d", path, qos, NO_NODE, bw, n); 
		finger = get_start_finger(cnode, path);
		if (finger)
		{
			whichPart = finger->whichPart;
			path_bw = finger->path_bw;
		}
		while (finger && finger->path == path)
		{
			receiver_id = NO_NODE;
			if (finger->confirmed.value)
			{
				Give_Spares(cnode, finger->up, finger->down, finger->bw);
				if (whichPart == MIDDLE)
				{
					if (up == NO_NODE)
					{
						if(num_packettes == 1)
							Forward_Msg(cnode, n, ev, "FAIL", finger->down,
							aPacket);  
						else
							receiver_id = finger->down;
					} /* end if up == NO_NODE */
					else
					{
						sprintf(aPacket, "%5d%5d%5d%5d%5d", 
							path, qos, n, bw, NO_NODE); 
						if(num_packettes == 1)
							Forward_Msg(cnode, n, ev, "FAIL", finger->up,
							aPacket);  
						else
							receiver_id =  finger->up;
					} /* end else down == NO_NODE */
					tempfinger = finger;
					if (finger->prev)
					{
						finger = finger->prev;
						Remove_Tablerec(cnode, tempfinger);
					} 
					else
					{
						Remove_Tablerec(cnode, tempfinger);
						finger = cnode->tablehead;       
					}
					if (num_packettes > 1)
					{
						if (!fails)
						{
							fails = (Packet_info *) malloc(sizeof(Packet_info));
							fails->receiver_id = receiver_id;
							strcpy(fails->thePacket, aPacket);
							fails->next = NULL; 
						} /* end if not fails */
						else
						{
							present = fails;
							while(present && present->receiver_id != receiver_id)
							{
								previous = present;
								present = present->next;
							} /* end while fails */
							if (present && present->receiver_id == receiver_id)
							{
								if(strlen(present->thePacket) + PACKET_SIZE >
								    MAX_PACKET_LEN)
								{
									Forward_Msg(cnode, n, ev, "FAIL", receiver_id,
									present->thePacket);
									strcpy(present->thePacket, aPacket);
								}
								else
									strcat(present->thePacket, aPacket); 
							} /* if receiver_id found */
							else
							{
								new_info = (Packet_info *) malloc(sizeof(Packet_info));
								new_info->receiver_id = receiver_id;
								strcpy(new_info->thePacket, aPacket);
								new_info->next = NULL; 
								previous->next = new_info;
							} /* end else new receiver_id */
						} /* end else fails exists */

					} /* end if num_packettes > 1 */
				} /* end if whichPart == MIDDLE */
				else if (whichPart == START)
				{
					linkdata *clink = (linkdata *)head(&cnode->links);

					finger->confirmed.value = 0;
					finger->confirmed.conf_recv = 0;
					finger->confirmed.conf_sent = false; 
					finger->down = NO_NODE;
					init_symbol(&ev->msg, "TRAIL", cnode->name);
					init_symbol(&ev->msg, "HOPS", "0");
					sprintf(aPacket, "%5d%5d%5d%5d%5d", 
						path, qos, n, path_bw, NO_NODE);

					while (clink) 
					{
						if (clink->active)
						{
							if (num_packettes == 1)
								Forward_Msg(cnode, n, ev, "PACK", clink->dest,
								aPacket);
							else
							{
								if (!packs)
								{
									packs = (Packet_info *) malloc(sizeof(Packet_info));
									packs->receiver_id = clink->dest;
									strcpy(packs->thePacket, aPacket);
									packs->next = NULL;
								} /* end if not packs */
								else
								{
									present = packs;
									while(present && 
									    present->receiver_id != clink->dest)
									{
										previous = present;
										present = present->next;
									}
									if (present && 
									    present->receiver_id == clink->dest)
									{
										if (strlen(present->thePacket) + PACKET_SIZE <= 
										    MAX_PACKET_LEN)
											strcat(present->thePacket, aPacket);
										else
										{
											Forward_Msg(cnode, n, ev, "PACK", clink->dest,
											present->thePacket);
											strcpy(present->thePacket, aPacket);
										}
									} /* end if receiver_id == clink->dest */
									else
									{
										new_info = (Packet_info *) malloc 
										    (sizeof(Packet_info));
										new_info->next = NULL;
										previous->next = new_info;
										new_info->receiver_id = clink->dest;
										strcpy(new_info->thePacket, aPacket);
									} /* end else receiver_id = clink->dest 
																						not found */
								} /* end else packs */
							} /* end else more than one packette */
						} /* end if clink->active */
						clink = (linkdata *)next(&cnode->links);
					} /* end while clink */
				} /* end else if START node */
				/* else FINISH node */
				else
				{
					finger->confirmed.value = 0;
					finger->confirmed.conf_recv = 0;
					finger->confirmed.conf_sent = false; 
					finger->up = NO_NODE;
				} /* end else FINISH node */
			} /* end if connected entry */
			if (finger != NULL) {
			/* Chow: with mci3.net /b=3-2 there is an instance the following
						 code cannot advance finger and therefore cannot exit
						 while loop 
			if (!(finger == cnode->tablehead && (finger->path != path ||
			    finger->path == path && finger->whichPart == MIDDLE)))
				finger = finger->next;
				*/
				finger = finger->next;
			}
		} /* end while finger */
	} /* end for i */
	Pack_Table(cnode);
	if (fails)
	{
		present = fails;
		while(present)
		{
			previous = present;
			present = present->next;
			Forward_Msg(cnode, n, ev, "FAIL", previous->receiver_id,
			previous->thePacket);
			free(previous);
		} /* end while fails */
	}
	if (packs)
	{
		present = packs;
		while(present)
		{
			previous = present;
			present = present->next; 
			Forward_Msg(cnode, n, ev, "PACK", previous->receiver_id,
			previous->thePacket);
			free(previous);
		} /* end while packs */
	}
} /* end FAIL_Processing */


/*  in:  identification number of PACK
				 erec containing received PACK message   
				 internal identification number and pointer to node
					 information for the same node
		  If the lost path has already been maximally restored and the 
			  receipient of the PACK knows that the lost path has been maximally
			  restored, the PACK message is ignored.  
		 
		  Otherwise:

			  Upon receipt of a PACK, a node determines if it has seen
			  a PACK message with the same identification number previously.
				  If a PACK message with the same identification number has
					  been seen previously:
						  if the node is a MIDDLE node for the
						  paths in the packettes, the node checks to see if it has 
						  previously notified its immediate neighbors that it is 
						  a node upstream of them on the paths in the packettes
						  in the PACK. 
							  If the neighbors have already been notified, they
								  will NOT be notified again.
							  If the neighbors have not been notified, PACKs with
								  updated packettes will be forwarded, and the  
								  node will record that having notified 
								  its immediate neighbors except the sender of 
								  the message. 

			 Whether the PACK is forwarded or not, using the information obtained
			 from the packettes in the PACK, the node (whether MIDDLE or FINISH
			 node for a path) will add an entry(s) to its Connections Table, 
			 one entry for each packette in the PACK with information about a
			 path that the node does NOT know to be maximally restored.

			 If the PACK makes it from the START node to the FINISH
			 node, a path must exist from the START node to the FINISH
			 node.

			 If the node is the FINISH node for a path in a packette in the PACK
			 message, the node will determine the sum of the number of bandwidths
			 already restored and the number of spares that are reserved for the
			 restoration of the lost path.  If the sum is less than the bandwidth
			 of the lost path, the FINISH node will determine if it has any spares
			 to the PACK's sender.  If the FINISH node has spares left and 
			 needs to make further restoration efforts for the lost path, it will
			 reserve some or all of its spares to the PACK's sender and reply to
			 the PACK's sender with an ACK.

			 If a node failure occurs and the lost node is the end node for a
			 path that path cannot be restored.  If the lost node is the START
			 node for at path, a PACK message will NOT be initiated for the path.  
			 To ensure that PACK messages do not run around the network forever
			 looking for a dead FINISH node, each PACK message contains a TRAIL
			 field.  The TRAIL field contains a list of all the nodes that have 
			 forwarded this specific copy of the PACK message.  A node may not
			 forward a PACK to a node in the TRAIL of a PACK message.  A node
			 appends its name to the TRAIL before forwarding the PACK.
	 */
static void PACK_Processing(char *packID, erec *ev, int n, 
node *cnode)  
{
	char aPacket[PACKET_SIZE+3],
	newPacket[PACKET_SIZE+1],
	thePacket[MAX_PACKET_LEN + 1],
	trail[256];
	int i, k,
	trail_length;
	int path, 
	qos,
	up, 
	bw, 
	down,
	whichPart, 
	path_bw,
	num_packettes;
	tablerec *finger = NULL,
	*tempfinger = NULL,
	*start_finger = NULL;
	boolean forward = true,
	skip = false,
	*availables;
	linkdata *clink;
	packrec *packfinger = cnode->packlist;
	Packet_info *packs = NULL,
	*last = NULL,
	*present = NULL,
	*previous = NULL;
	int hopcount;





	find_symbol_def(trail, ev->msg, "TRAIL");

	availables = (boolean *) malloc(sizeof(boolean) * (netsize + 1));
	for (i = 0; i <= netsize; i++)
		availables[i] = true;
	trail_length = (strlen(trail) + 1) / NodeNameLength;
	hopcount = trail_length;
	for (i = 0; i < trail_length; i++)
		availables[Get_Node_ID(&(trail[NodeNameLength * i]))] = false;

	Append(trail, cnode->name);
	find_symbol_def(thePacket, ev->msg, "PACK");
	if (debug_node && choosen == n) {
		fprintf(choosen_out,
			"at %f, %d received PACK=%s\n", cnode->time, n, thePacket);
		Print_Node_Table(cnode);
		}
	if (n == 31) {
		printf("Chow: check if PACK got sent to node 9\n");
	}
	num_packettes = strlen(thePacket) / PACKET_SIZE;
	for (i = 0; i < num_packettes; i++)
	{ 
		strncpy(newPacket, &(thePacket[PACKET_SIZE * i]), PACKET_SIZE);
		skip = false;
		parse_packet(newPacket, &path, &qos, &up, &bw, &down);

	



		create_up_ack(cnode, path, up);
		if (path==86 && up==21 && n==2) 
			{
			printf("BUG: Chow: why entry to merge \n");
			}
		if (debug_path && path == selected) 
			{
			selected_path = true;
			Print_Table(cnode);
			}

		/* Upon detection of a failure, the node detecting the failure may
			 issue a FAIL for the paths for which it is a MIDDLE node and a
			 PACK for the paths for which it is a START node.  Both the FAIL
			 and the PACK messages will have the same identification number,
			 but will reference different paths.
			 Plus, upon receipt of a FAIL, a FAIL message may be split into
			 two new messages with the same identification number, but 
			 the new messages will contain packettes for different paths.
		 Thus, it is possible to have PACK messages with the same IDs,
		 but containing packettes for different paths.
		 */
		while(packfinger && forward)
		{
			if (!strcmp(packID, packfinger->id))
			{
				int_node *paths = packfinger->paths;

				while (paths && paths->num != path)
					paths = paths->next;
				while (forward && paths && paths->num == path) 
					{
					if (paths->up == up) 
						{
				/* Chow: found 13-33cut case, node 23 receive from both
					45 and 44 but did not forward pack from 44 to 45 
					resulting missing six bw */

						forward = false;
						} 
					else 
						{
					/* Chow: only paths->up need to be forward with PCK 
						all other links are forwarded with PACK which creates
						the entry pointed by paths */
						/* Chow: big mistake by having for (i=0; here,
							since it changes the outside loop that also use i
							as loop variable */
						for (k = 0; k <= netsize; k++)
							availables[k] = false;
						availables[paths->up] = true;
						}
					paths = paths->next;
					}
			}
			packfinger = packfinger->next;
		} /* end while packfinger */

	


		if (forward)
		{
			int_node *new_path = (int_node *) malloc(sizeof(int_node));

			new_path->num = path;
			new_path->up = up;
			new_path->next = NULL;
			if (!packfinger)
			{
				packfinger = (packrec *) malloc(sizeof(packrec));
				sprintf(packfinger->id, "%s", packID);
				packfinger->paths = new_path;
				packfinger->next = cnode->packlist;
				cnode->packlist = packfinger;
			}
			else
			{
				int_node *paths = packfinger->paths;

				while(paths->next)
					paths = paths->next;
				paths->next = new_path;
			} 
		} /* end if forward */

		whichPart = searchWhichPart(cnode, path);
		if (whichPart == -1) whichPart = MIDDLE;
		path_bw = bw;
		/* Add table entry for up node. */
		start_finger = get_start_finger(cnode, path);
		if (start_finger) 
		{
			


			whichPart = start_finger->whichPart;  
			finger = start_finger;
			while (!skip && finger && finger->path == path)
			{
				if (finger->time_connected > 0.0 && 
				    finger->confirmed.conf_recv == FINAL)
					 /* Chow: 2-3cut case shows that the return decision is wrong,
						the entries generated by node 40's pack msg only partially
						restored, path 59 and 60 are not restored. but by only
						compare path 52 of node 16's pack msg and throw the
						rest away is a bad mistake. should skip this packettee only

					return;     

					 If one path in the PACK has already
							 been processed than this PACK has already
						 been broadcast. */
						 skip = true;

				finger = finger->next;
			}
			if (!skip) {
			if (!finger)
			{
				tempfinger = (tablerec *) malloc(sizeof(tablerec));
				Init_Tablerec(path, qos, up, hopcount, bw, NO_NODE, 0, whichPart,
				cnode->time, tempfinger);
				Insert_After_Tablerec(cnode, cnode->tabletail, tempfinger); 
			}
			else if (finger->path != path)
			{
				tempfinger = (tablerec *) malloc(sizeof(tablerec));
				Init_Tablerec(path, qos, up, hopcount, bw, NO_NODE, 0, whichPart,
				cnode->time, tempfinger);


			

				Insert_Before_Tablerec(cnode, finger, tempfinger);
			}
			}
		} /* end if start_finger */
		else
		{
			/* Chow: here we should check if this is FINISH node */
			
			tempfinger = (tablerec *) malloc(sizeof(tablerec));
			Init_Tablerec(path, qos, up, hopcount, bw, NO_NODE, 0, MIDDLE,
			cnode->time, tempfinger);
			finger = cnode->tablehead;
			while(finger && finger->path < path)
				finger = finger->next;
			if (finger)
				{
				

				Insert_Before_Tablerec(cnode, finger, tempfinger);
				}
			else
			{
				finger = cnode->tabletail;
				Insert_After_Tablerec(cnode, finger, tempfinger);
			}
			start_finger = get_start_finger(cnode, path);
		}







		/* If a PACK arrives at the FINISH node, a route must exist from
						 the START node to the FINISH node.
					 */
		if (whichPart == FINISH && Timer_expired(cnode, path) )
		{
			

			
			int acked_bw = 0,ack_bw, fingerBw, MoreBwNeeded=TRUE;

			if(debug)
			{
			printf("PACK received by finish node %d requesting %d bw for path %d at time %f, with EXPIRED timer\n",n, bw, path, cnode->time);
			printf("thePacket(path,qos,up,bw,down)=(%d,%d,%d,%d,%d)\n\n", path,qos,up,bw,down);
			}


			finger = start_finger;
			while (finger && finger->path == path && finger->confirmed.value)
			{
				acked_bw += finger->bw;
				finger = finger->next;
			} 
			ack_bw = path_bw - acked_bw;
			if(!ack_bw)
				MoreBwNeeded=FALSE;

			while(MoreBwNeeded)
			{

				path_ack_info *ack_sent = cnode->acks_sent;

				MoreBwNeeded = FALSE;
				while(ack_sent && ack_sent->path != path)
					ack_sent = ack_sent->next;

				init_symbol(&ev->msg, "TRAIL", cnode->name);
					
			
				finger = Get_unconnected_entry_with_min_hop_and_spares(cnode, path);

				if(finger)
					{
					up = finger->up;
					fingerBw = finger->bw;
					if(ack_bw > fingerBw)
						MoreBwNeeded=TRUE;
					if(ack_bw <= fingerBw)
						fingerBw = ack_bw;

					}
				else /* FINGER == NULL */
					{
					MoreBwNeeded = 0;
					}


			

				if (finger)
				{
					clink = (linkdata *) head (&cnode->links); 
					while(clink->dest != up)
						clink = (linkdata *) next (&cnode->links); 

					if (clink && clink->active && clink->spare)
					{
						char creqStr[6];
						int connect_bw = fingerBw > clink->spare ? clink->spare : fingerBw; 
						
						if(fingerBw > connect_bw)
							MoreBwNeeded=TRUE;

						Take_Spares(cnode, up, NO_NODE, connect_bw);
						ack_bw -= connect_bw; 
						if (finger->bw > connect_bw)
						{
							tempfinger = (tablerec *)malloc(sizeof(tablerec));

							Copy_Tablerec(finger, tempfinger);
							tempfinger->bw -= connect_bw;
							tempfinger->time_received = cnode->time;
							Insert_After_Tablerec(cnode, finger, tempfinger);
							
						}
						finger->bw = connect_bw;
						/* FINISH node does NOT know the DCS settings,
						 therefore, the FINISH node reserves the spares.
						 No connections are made.
											 */
						finger->time_connected = cnode->time;
						finger->confirmed.value = ACKED;

						/* At this point have not reduced bandwidth of NO_NODE-
									 NO_NODE entry by connect_bw.
											 */
						tempfinger = start_finger;
						while(tempfinger && tempfinger->path == path && tempfinger->up != NO_NODE)
							tempfinger = tempfinger->next;
						if (tempfinger) 
						{
							if (tempfinger->bw <= connect_bw)
								Remove_Tablerec(cnode, tempfinger);  
							else
								tempfinger->bw -= connect_bw;
						}
						finger->confirmed.ack.up = cnode->creq;

						sprintf(creqStr, "%2d", cnode->creq++);
						init_symbol(&ev->msg, "ID", creqStr);  
						
						connect_bw = accumulated_bw(cnode, path, up);


						sprintf(aPacket, "%5d%5d%5d%5d%5d%2d", 
						path, qos, NO_NODE, connect_bw, n, 1);

						InitActualBw(ev, finger->bw);


						Forward_Msg(cnode, n, ev, "ACK  ", up, aPacket);
						add_up_ack(cnode, path, up);
						if (!ack_sent)
						{
							path_ack_info *new_info = (path_ack_info *)
							malloc(sizeof(path_ack_info)); 

							new_info->next = NULL;
							new_info->num_acks = 1; 
							new_info->uplist = NULL;
							new_info->downlist = NULL;
							new_info->path = path;
							if (!cnode->acks_sent)
								cnode->acks_sent = new_info;
							else
							{
								ack_sent = cnode->acks_sent;
								while (ack_sent->next)
									ack_sent = ack_sent->next;
								ack_sent->next = new_info; 
							} /* end else */
						} /* if !ack_sent */
					}
				}
				Pack_Table(cnode);
			} /* end while*/

		


		} /* end if whichPart == FINISH */

		if(whichPart == FINISH && ( Timer_initiated(cnode, path) ) && ( !Timer_expired(cnode, path) ) )
		{
			if(debug)
			{
				printf("PACK received by finish node %d requesting %d bw at time %f\n",n, bw, cnode->time);
				printf("thePacket(path,qos,up,bw,down)=(%d,%d,%d,%d,%d)\n\n", path,qos,up,bw,down);


			
			
				printf("Timer already initiated for path %d at node %d\n time:%f but NOT expired\n", path, cnode->id, ev->time);
			}
			
		}
		/*

		Timer has not been initiated */

		if(whichPart == FINISH && ( !Timer_initiated(cnode, path) ) )
		{

			if(debug)
			{
			printf("PACK received by finish node %d requesting %d bw for path:%d at time %f\n",n, bw, path, cnode->time);
			printf("thePacket(path,qos,up,bw,down)=(%d,%d,%d,%d,%d)\n\n", path,qos,up,bw,down);

			printf("Timer NOT initiated for path %d at node %d\n time:%f\n", path, cnode->id, ev->time);
			printf("Initiating Timer Event\n\n");
			
			}

				if(!Initiate_timer_event(cnode, ev, path, bw, qos))
				{
					printf("ERROR Initiate_timer_event\n");
					exit(1);
				}	

		}


		
		
		
		else if (forward && whichPart == MIDDLE)
		{
		
		if(debug)
		{
		printf("PACK msg received by MIDDLE node %d at time %f\n", cnode->id, cnode->time);
		printf("thePacket(path,qos,up,bw,down)=(%d,%d,%d,%d,%d)\n\n", path,qos,up,bw,down);
		}



			init_symbol(&ev->msg, "TRAIL", trail);

			clink = (linkdata *) head (&cnode->links);
			while(clink)
			{
				/* Chow: in 13-44 find 33 forward PACK to 22 but 33-22 has
							no spare */
				if (clink->active && availables[clink->dest] && clink->spare)
				{
					sprintf(aPacket, "%5d%5d%5d%5d%5d", path, qos, n, bw, NO_NODE);
				
				

					if (num_packettes == 1)
					{
						
						Forward_Msg(cnode, n, ev, "PACK", clink->dest, aPacket);
					}
					else
					{
						if (!packs)
						{
							packs = (Packet_info *) malloc(sizeof(Packet_info));
							packs->receiver_id = clink->dest;
							strcpy(packs->thePacket, aPacket);
							packs->next = NULL;
						} /* end if not packs */
						else
						{
							present = packs;
							while(present && present->receiver_id != clink->dest)
							{
								previous = present;
								present = present->next;
							}
							if (present && present->receiver_id == clink->dest)
							{
								if (strlen(present->thePacket) + PACKET_SIZE <= 
								    MAX_PACKET_LEN)
									strcat(present->thePacket, aPacket);
								else
								{
									init_symbol(&ev->msg, "ID", packID);

								
									Forward_Msg(cnode, n, ev, "PACK", clink->dest,
									present->thePacket);
									strcpy(present->thePacket, aPacket);
								}
							} /* end if receiver_id == clink->dest */
							else
							{
								last = (Packet_info *) malloc (sizeof(Packet_info));
								last->next = NULL;
								previous->next = last;
								last->receiver_id = clink->dest;
								strcpy(last->thePacket, aPacket);
							} /* end else receiver_id = clink->dest not found */
						} /* end else packs */
					} /* end else more than one packette */
				} /* end if clink->active */
				clink = (linkdata *) next (&cnode->links);  
			} /* end while clink */
		} /* end else if whichPart == MIDDLE */
	} /* end for i*/


	if (packs)
	{
		

		init_symbol(&ev->msg, "ID", packID);


	


		present = packs;
		while(present)
		{
			previous = present;
			present = present->next; 

		

			Forward_Msg(cnode, n, ev, "PACK", previous->receiver_id,
			previous->thePacket);
			free(previous);
		} /* end while packs */
	}
	Pack_Table(cnode);


} /* End PACK_Processing */


/* Common initialization for single packette messages (ACK, CONF,
		 or CONFF). 
		 in:  erec containing message received by node
				pointer to node information for same node
		 out: message identification number
				information obtained from packette in message
					internal path identification number
					qos
					bandwidth 
					internal identification number for upstream node 
					internal identification number for downstream node
				position (START, MIDDLE or FINISH) of node on path in packette
				original bandwidth of path in packette
				also return the pointer, whichpart, and bw of 
								first table entry that match the path id 
	 */
void init_one_packette_msg(erec *ev, node *cnode, int *msg_num_ptr,
int *path_ptr, int *qos_ptr, 
int *up_ptr,
int *bw_ptr, 
int *down_ptr, tablerec **start_finger_ptr,
int *whichPart_ptr, int *path_bw_ptr) 
{
	char  msg_id[25];
	tablerec *finger;
	char aPacket[PACKET_SIZE+1];
	int path;
	int qos, up, bw, down;


	/* Get number of message received. */   
	find_symbol_def(msg_id, ev->msg, "ID");
	*msg_num_ptr = atoi(msg_id);

	find_symbol_def(aPacket, ev->msg, "PACK");
	parse_packet(aPacket, &path, &qos, &up, &bw, &down);
	*path_ptr = path;
	if (debug_path && path == selected)
	{
		selected_path = true;
		Print_Table(cnode);
	}
	*qos_ptr = qos;
	*up_ptr = up;
	*bw_ptr = bw;
	*down_ptr = down;

	finger = cnode->tablehead;
	while (finger && finger->path < path)
		finger = finger->next;
	if (finger && finger->path == path)
	{
		*start_finger_ptr = finger;
		*whichPart_ptr = finger->whichPart;
		*path_bw_ptr = finger->path_bw;
	}
	else 
	    *start_finger_ptr = NULL;
} /* end init_one_packette_msg */

/* Common initialization for ACK packette messages 
		 in:  erec containing message received by node
				pointer to node information for same node
		 out: message identification number
				information obtained from packette in message
					internal path identification number
					qos
					bandwidth 
					internal identification number for upstream node 
					internal identification number for downstream node
					ackhop
				position (START, MIDDLE or FINISH) of node on path in packette
				original bandwidth of path in packette
				also return the pointer, whichpart, and bw of 
								first table entry that match the path id 
	 */
void init_ack_packette_msg(erec *ev, node *cnode, int *msg_num_ptr,
int *path_ptr, int *qos_ptr, 
int *up_ptr,
int *bw_ptr, 
int *down_ptr, 
int *ackhop_ptr,
tablerec **start_finger_ptr,
int *whichPart_ptr, int *path_bw_ptr) 
{
	char  msg_id[25];
	tablerec *finger;
	char aPacket[PACKET_SIZE+3]; /* with 2 byte field for ackhop*/
	int path;
	int qos, up, bw, down, ackhop;


	/* Get number of message received. */   
	find_symbol_def(msg_id, ev->msg, "ID");
	*msg_num_ptr = atoi(msg_id);

	find_symbol_def(aPacket, ev->msg, "PACK");
	parse_ack_packet(aPacket, &path, &qos, &up, &bw, &down, &ackhop);
	*path_ptr = path;
	if (debug_path && path == selected)
	{
		selected_path = true;
		Print_Table(cnode);
	}
	*qos_ptr = qos;
	*up_ptr = up;
	*bw_ptr = bw;
	*down_ptr = down;
	*ackhop_ptr = ackhop;

	finger = cnode->tablehead;
	while (finger && finger->path < path)
		finger = finger->next;
	if (finger && finger->path == path)
	{
		*start_finger_ptr = finger;
		*whichPart_ptr = finger->whichPart;
		*path_bw_ptr = finger->path_bw;
	}
	else 
	    *start_finger_ptr = NULL;
} /* end init_ack_packette_msg */

/* in:  pointer to node information for node preparing to forward
					 ACKs 
				pointer to nodelist containing information about which 
					 nodes to forward ACKs to and information for each ACK 
					 to be forwarded
				information for ACK
					receiver
					path
					message number
				sender of the received ACK
		 out: new or revised nodelist
	 */ 
void generate_nodelist(node *cnode, nodelist **ptr_ptr, boolean up,
boolean ack, int path, int root, int msg_num,
int msg_sender, int still_waiting)
{
	tablerec *finger = cnode->tablehead;
	nodelist *start_ptr = NULL,
	*ptr = NULL,
	*prev_ptr = NULL;

	while(finger && finger->path != path)
		finger = finger->next;

	if (up)
	{
		while(finger && finger->path == path && finger->confirmed.value)
		{
			if (finger->up == root)
			{
				ptr = start_ptr;
				while(ptr && ptr->num != finger->down)
				{
					prev_ptr = ptr;
					ptr = ptr->next;
				}
				if (!ptr)
				{
					ptr = (nodelist *) malloc(sizeof(nodelist));
					if (!start_ptr)
					{
						start_ptr = ptr; 
						*ptr_ptr = start_ptr;
					}
					else
						prev_ptr->next = ptr;
					ptr->next = NULL;
					ptr->num = finger->down;
					if (ack)
					{
						ptr->msg_num = finger->confirmed.ack.down;
						if (ptr->msg_num == msg_num)
							ptr->acked = true;
						else 
						    ptr->acked = false;
						ptr->acked_bw = finger->bw;
					} /* end if ACK */
					/* else CONF */
					else
					{
						if (msg_sender == root)
						{
							if (still_waiting || (!still_waiting &&
							    finger->confirmed.ack.down >=
							    finger->confirmed.ack.recv))
								ptr->msg_num = finger->confirmed.conf_recv;
							else
								ptr->msg_num = finger->confirmed.ack.recv;
						}
						else
							ptr->msg_num = finger->confirmed.conf_recv;
						if (finger->confirmed.value == 1)
						{
							ptr->acked = true; 
							ptr->acked_bw = finger->bw;
						} /* end if confirmed bandwidth */

						/* else not confirmed bandwidth */ 
						{
							ptr->acked = false;
							ptr->acked_bw = 0;
						} /* end else not confirmed bandwidth */

					} /* end else CONF */
				} /* end if not ptr */
				/* else already link */
				else
				{
					if (ack)
					{
						ptr->acked_bw += finger->bw;
						if (finger->confirmed.ack.down == msg_num)
							ptr->acked = true;
					} /* end if ack */
					/* else conf */
					else
					{
						if (finger->confirmed.value == 1)
							ptr->acked_bw += finger->bw;
						if (!still_waiting && root == msg_sender && 
						    finger->confirmed.ack.recv > ptr->msg_num)
							ptr->msg_num = finger->confirmed.ack.recv;  
					} /* end else conf */
				} /* end else already link */
			} /* end if finger->up == root */
			finger = finger->next;
		} /* end while finger */
	} /* end if up */

	/* else down */
	else
	{
		while(finger && finger->path == path && finger->confirmed.value)
		{
			if (finger->down == root)
			{
				ptr = start_ptr;
				while(ptr && ptr->num != finger->up)
				{
					prev_ptr = ptr;
					ptr = ptr->next;
				}
				if (!ptr)
				{
					ptr = (nodelist *) malloc(sizeof(nodelist));
					if (!start_ptr)
					{
						start_ptr = ptr; 
						*ptr_ptr = start_ptr;
					}
					else
						prev_ptr->next = ptr;
					ptr->next = NULL;
					ptr->num = finger->up;
					if (ack)
					{
						ptr->msg_num = finger->confirmed.ack.down;
						if (ptr->msg_num == msg_num)
							ptr->acked = true;
						else 
						    ptr->acked = false;
						ptr->acked_bw = finger->bw;
					} /* end if ACK */
					/* else CONF */
					else
					{
						if (ptr->num == msg_sender)
						{
							if (still_waiting || (!still_waiting && 
							    finger->confirmed.ack.down >=
							    finger->confirmed.ack.recv))
								ptr->msg_num = finger->confirmed.conf_recv;
							else
								ptr->msg_num = finger->confirmed.ack.recv; 
						}
						/* else ptr->nu is not message sender */
						else
							ptr->msg_num = finger->confirmed.conf_recv;
						if (finger->confirmed.value == 1)
						{
							ptr->acked = true;
							ptr->acked_bw = finger->bw;
						} /* end if confirmed */
						else
						{
							ptr->acked = false; 
							ptr->acked_bw = 0;
						} /* end else not confirmed */
					} /* end else conf */
				} /* end if not ptr */
				/* else ptr */
				else
				{
					if (ack)
					{
						ptr->acked_bw += finger->bw;
						if (finger->confirmed.ack.down == msg_num)
							ptr->acked = true;
					} /* end if ack */
					/* else conf */
					else
					{
						if (finger->confirmed.value == CONFIRMED)
							ptr->acked_bw += finger->bw;
						if (ptr->num == msg_sender && !still_waiting &&
						    finger->confirmed.ack.recv > ptr->msg_num) 
							ptr->msg_num = finger->confirmed.ack.recv; 
					} /* end else conf */
				} /* end else ptr */
			} /* end if finger->down == root */
			finger = finger->next;
		} /* end while finger */
	} /* end else down */
} /* end generate_nodelist */

/* in: pointers to node and table entry, bw to be left on the current entry
		 a new entry will be generated and insert before the current entry */

void split_entry_bw(node* cnode, tablerec *finger, int bw) {
		tablerec *tempfinger;
     	tempfinger = (tablerec *)malloc(sizeof(tablerec));
     	Copy_Tablerec(finger, tempfinger);
     	tempfinger->bw -= bw;
     	tempfinger->time_received = cnode->time;
     	finger->bw = bw;
     	Insert_Before_Tablerec(cnode, finger, tempfinger);
}


/* in:  pointer to node information
				identification number of path
				identification number of upstream node
		 returned: message number of ACK to be forwarded upstream to 'up' for
						  'path' 
	 */
int get_ack_num(node *cnode, int path, int up)
{
	int msg_num = 0;
	nodelist *downs = NULL,
	*ptr = NULL;
	tablerec *finger = get_start_finger(cnode, path);
	int whichPart = finger->whichPart;

/* the entry just got split and not sorted the while condition need to be
	change to search all entry with path == path 
	while (finger && finger->path == path && finger->confirmed.value)
	*/
	while (finger && finger->path == path)
	{
		if (finger->up == up && finger->confirmed.value)
		{
			ptr = downs;
			if (whichPart == FINISH) 
			{
				msg_num += finger->confirmed.ack.up;
			} 
			else 
			{
				while(ptr && ptr->num != finger->down)
					ptr = ptr->next;
				if (ptr)
				{
					/* chow replace with ack.up 
					if (finger->confirmed.ack.down > ptr->msg_num)
						ptr->msg_num = finger->confirmed.ack.down;
						*/
					if (finger->confirmed.ack.up > ptr->msg_num)
						ptr->msg_num = finger->confirmed.ack.up;
				}
				else
				{
					ptr = (nodelist *) malloc(sizeof(nodelist));
					ptr->num = finger->down;
					/* chow replace with ack.up 
					ptr->msg_num = finger->confirmed.ack.down;
					*/
					ptr->msg_num = finger->confirmed.ack.up;
					if (downs)
						ptr->next = downs;
					else 
						ptr->next = NULL;
					downs = ptr;     
				} /* end else not ptr */
			} /* end else whichPart */
		} /* end if finger->up == up */
		finger = finger->next;
	} /* end while finger */
	if (whichPart == FINISH) 
	{
		msg_num++;
		return msg_num;
	}

	msg_num = 0;
	ptr = downs;
	while (ptr)
	{
		msg_num += ptr->msg_num;
		ptr = ptr->next;
	} /* end while ptr */
	free_nodelist(downs);
	/*
	if (msg_num == 0) msg_num = 1;
	return msg_num;
	*/
	return msg_num+1;
} /* end get_ack_num */


/* in: pointers to node and table entry, node number of downstream node
		 that are being rejected with conf with 0 
		 entry will be updated a status corresponding to being acked,
		 and ack message will be sent out on the upstream node finger->up */

void 
update_entry_ack(node *cnode, erec* ev, tablerec *finger, 
	tablerec *entry, int msg_num)
{

	tablerec *tp;
	char msgStr[16];
	char trail[256];
	char aPacket[PACKET_SIZE+3];
	int acked_bw = 0;
	int ackhop = 1;

	/* Chow: need to find out the total accumulated bw send to the up node
	  not just finger->bw  since the bandwidth request may be fragmented */

	tp = get_start_finger(cnode,finger->path);
	while (tp && tp->path == finger->path) 
	{
		if (tp->up == finger->up && tp->confirmed.value > 0)
			acked_bw += tp->bw;
		tp = tp->next;
	}
	acked_bw += finger->bw;

	/* Chow: this bw calculation must be move up here before 
		finger->confirmed.value is set to ACKED otherwise finger->bw
		will be counted twice !! */

	Take_Spares(cnode, finger->up, NO_NODE, finger->bw);
	finger->confirmed.value = ACKED;
	/* in 3-40cut, layer3.net , found n=9 rcvd conf0 from 30 but msg_num=1
		recv and down field should be set to entry->ack.* value
	finger->confirmed.ack.recv = msg_num;
	finger->confirmed.ack.down = msg_num;
	*/
	finger->confirmed.ack.recv = entry->confirmed.ack.recv;
	finger->confirmed.ack.down = entry->confirmed.ack.down;
	finger->confirmed.ack.up =  get_ack_num(cnode, finger->path, finger->up);
	finger->time_connected = dxc_time(cnode, finger->bw);
	finger->down = entry->down;
	finger->ackhop = entry->ackhop;

	if (finger->whichPart == FINISH)  {
		ackhop = 1;
		init_symbol(&ev->msg, "TRAIL", cnode->name);
	} else {
		find_symbol_def(trail, ev->msg, "TRAIL");
		init_symbol(&ev->msg, "TRAIL", trail);
		ackhop = finger->ackhop;
	}
	sprintf(msgStr, "%2d", finger->confirmed.ack.up);
	init_symbol(&ev->msg, "ID", msgStr);
	sprintf(aPacket,"%5d%5d%5d%5d%5d%2d", 
	/*Chow replace with acked_bw 
		finger->path,finger->qos, NO_NODE, finger->bw, cnode->id, finger->ackhop);
		*/
			finger->path,finger->qos, NO_NODE, acked_bw, cnode->id, ackhop+1);


		
	InitActualBw(ev, finger->bw);
	Forward_Msg (cnode, cnode->id, ev, "ACK  ", finger->up, aPacket);
	add_up_ack(cnode, finger->path, finger->up);
}



/* in:  pointer to node information
				path for which unconfirmed entries are to be eliminated
				internal identification number for upstream node for
					which all entries are to be removed 
		 out: updated node information--all unconfirmed entries for 'path' 
					having the upstream node 'up' have been eliminated
	 */
void zero_up_entries(node *cnode, int path, int up)
{
	tablerec *finger = cnode->tablehead;

	while (finger && (finger->path != path || 
	    finger->confirmed.value == CONFIRMED))
		finger = finger->next;

	while (finger && finger->path == path)
	{
		if (finger->up == up)
		{
			if (finger->confirmed.value)
			{
				Give_Spares(cnode, finger->up, finger->down, finger->bw);
				finger->time_connected = (float)0.0;
				finger->confirmed.value = 0;
			} /* end if connected */
			finger->bw = 0;
		} /* end if finger->up == up */

		finger = finger->next;
	} /* end while finger */

	Pack_Table(cnode);
} /* end zero_up_entries */





/* in:  pointer to node information
		 returned:  true if the referenced node has one or more spares
							  remaining to any of its immediate neighbors
						else
							  false
	 */
boolean spares_remaining(node *cnode)
{
	linkdata *clink = (linkdata *)head(&cnode->links);

	while(clink && (!clink->active || 
	    (clink->active && !clink->spare)))
		clink = (linkdata *)next(&cnode->links);

	if (clink)
		return true;
	else
		return false;
} /* end spares_available */


/* in:  pointer to node information
				identification number for node--node other than node whose 
					information is pointed to by 'cnode'        
		 returned:  the number of spares that node 'cnode' has to node 
							'node_num' if 'node_num' is reachable by an active 
							link, 
						else
							zero 
	 */
int num_spares(node *cnode, int node_num)
{
	linkdata *clink = (linkdata *)head(&cnode->links);

	while (clink && clink->dest != node_num)
		clink = (linkdata *) next(&cnode->links);

	if (clink && clink->active)
		return clink->spare;
	else
		return 0;
} /* end num_spares */



boolean same_ack_down(node *cnode, tablerec *finger) 
{
	tablerec *entry = get_start_finger(cnode, finger->path);
	int path = finger->path;
	int down = finger->down;
	int ackdown = finger->confirmed.ack.down;
	while (entry && entry->path == path) 
	{
		if (entry->confirmed.value == ACKED &&
			 entry->down == down &&
			 entry->confirmed.ack.down == ackdown &&
			 entry != finger)
			 return true;
		else entry = entry->next;
	}
	return false;
}





/* in:  pointer to node information
				identification number of path
				identification number of upstream node
		 returned: bandwidth for packette for ACK to be forwarded upstream 
						  to 'up' for 'path'
	 */
int get_ack_bw(node *cnode, int path, int up)
{
	tablerec *finger = get_start_finger(cnode, path);
	int bw = 0;

	while(finger && finger->path == path && finger->confirmed.value)
	{
		if (finger->up == up)
			bw += finger->bw;
		finger = finger->next;
	} /* end while finger */

	return bw;
} /* end get_ack_bw */



/* in:  erec containing ACK message received
				node identification number
				pointer to node information for node 'n'

		 If receiver of ACK message is a MIDDLE node on the path specified in
		 packette in the ACK message, the node will:
			 1)  determine the minimum of:
					  the bandwidth in the packette of the received ACK minus
							the bandwidth of the already protected connections for
							the path
					  the number of spares to the sender
					  the number of spares to available nodes known to be upstream
							of itself on the path in the ACK packette--an upstream
							node is available if it has not already received two
							requests from 'n' to participate in restoration paths
							for the path specified in the ACK packette
			 2)  make and protect as many additional connections (the minimum
					  determined in 1) from the ACK's sender through itself to
					  nodes upstream of itself on the path specified in the ACK 
					  packette
			 3)  if any additional connections are made, update and forward
					  the ACK to each affected upstream node
				  if no additional connections are made:
					  if all connections for 'path' with the ACK's sender are
						  confirmed connections, return a CONF or CONFF for the
						  confirmed bandwidth
					  else
						  note having received the ACK and wait for a reply 
						  from the ACK previously forwarded for the protected
						  bandwidth 

			Note:  Several possible cases exist, denoted as straight forward, 
					 multiple upstream, multiple downstream and worst case.
					 Straight forward refers to the situation where all protected
						bandwidth for a specific downstream node is connected 
						through 'n' to single upstream node AND all protected
						bandwidth for a specific upstream node is connected
						through 'n' to a single downstream node.  Two co-existing
						straight forward cases.
						
							  down1---n---up1                

							  down2---n---up2

					 Multiple upstream refers to the situation where one
						downstream node connects through 'n' to more than one 
						upstream node, and each upstream node connected
						through 'n' to the downstream node, connects through
						'n' to only the one downstream node.  

											 up1
											/
										  / 
							  down1---n---up2
										  \
											\
											 up3

					 Multiple downstream refers to the case where more that one 
						downstream node connects through 'n' to a single upstream 
						node and each downstream node connected through 'n' to the
						upstream node, connects through 'n' to only the one
						upstream node.

								down1
									  \
										\
							  down2---n---up1 
							  
					 Worst case refers to all other the situations, such as when
						a single downstream nodes connect through 'n' to several
						upstream nodes and a second downstream node connects
						through 'n' to one of the same upstream nodes.

								down1     up1
									  \   /           
										\ / 
							  down2---n---up2
										  \
											\
											 up3

							  down1 connects through 'n' to up1 and up2
							  down2 connects through 'n' to up1, up2 and up3 

					 For the multiple upstream or worst cases, it is possible
					 for the reception of one ACK message to result in the 
					 sending of more than one ACK message.

					 For the multiple downstream and worst cases, the message
					 number of the forwarded ACK is derived by summing the 
					 message numbers of the most recent ACK received from
					 each of the downstream nodes with a connection through
					 'n' to the receipient of the ACK. For example, if
					 previously ACK #3 from down1 resulted in connections from 
					 down1 through 'n' to up1 and now ACK #6 from down 2
					 results in connections from down2 through 'n' to up1,
					 the ACK forwarded to up1 will have a message number of
					 3 + 6 = 9. 

		 If the receiver is the START node for the path in the ACK packette,
		 then a restoration path has been carved out from the FINISH node
		 to the START node.  
		 The START node will make and confirm additional connections up to
		 the minimum of:
			 the bandwidth in the received ACK minus the already confirmed
				  connections with the ACKs sender for the path in the ACK
				  packette and
			 the bandwidth of the connections it can make with the ACK's
				  sender  
		 Whether additional bandwidth is confirmed or not, the START node 
		 will initiate a CONF or CONFF, sending a confirmation message for
		 its confirmed bandwidth with the ACK's sender to the ACK's sender.
		 If:
			 the sum of the confirmed bandwidth of all the START node's 
				  entries for the path == the bandwidth lost by the path,
				  i.e., the path has been fully restored,
		 OR
			 all the connections of the START node for the path in the ACK
			 are confirmed connections and the START node for the path in the 
			 ACK has ZERO spares remaining, i.e., the path cannot be restored
			 further,
		 the START node will initiate a CONFF, otherwise, the START node will
		 initiate a CONF.
		 If the START node still has spares remaining and the START node issues
		 a CONFF, the START node will also issue a 'Path DONE' or PDONE to 
		 to each of it immediate neighbors except the receipient of the CONFF.
		 If the START node has no spares remaining, the START node for the
		 path in the CONFF will if it is an end node for any other path.  If
		 the START node for the path in the CONFF is NOT an end node for any 
		 other paths, the START node for the path in the CONFF will issue
		 PDONEs for the path in the CONFF to each of its immediate neighbors
		 other than the receipient of the CONFF.  
		 If the START node for the path in the CONFF has no spares remaining
		 and the node is an end node for other paths, the START node for the
		 path in the CONFF will issue multiple packette PDONEs to each of its
		 immediate neighbors.  The PDONEs to each immediate neighbor other
		 than the CONFF's receiver, will contain packettes for each maximally
		 restored path.  The PDONE to the CONFF's receiver will contain
		 packettes for each maximally restored path except the path in the 
		 CONFF.
	 */
void ACK_Processing(erec *ev, int n, node *cnode)
{
	char aPacket[PACKET_SIZE+3],
	trail[256],
	/* AHA win32 strange error*/
	/* msgSTR[MSG_BASE_SIZE + 10] from beginning */
	/* but damaged path, so I made it + 20 instead*/
	msgStr[MSG_BASE_SIZE+20];
	int path,
	qos,
	up,
	bw,
	down,
	ackhop,
	whichPart, 
	path_bw,
	msg_num,
	ack_bw,
	conf_bw = 0;
	boolean  multiple_upstream = false,  
	multiple_downstream = false,
	worst_case = false;
	boolean acked = false;
	tablerec *finger = NULL,
	*start_finger = NULL, 
	*tempfinger = NULL; 
	float start_time = cnode->time;
	int msgID;

	





	/* Initialization common to all single packette messages. */
	init_ack_packette_msg(ev, cnode, &msg_num, &path, &qos, &up, &bw, &down,
	&ackhop, &start_finger, &whichPart, &path_bw);




	/* AHA Debug */
	if(debug && whichPart == MIDDLE)
	{
		printf("ACK packette received by MIDDLE node %d at time %f\n",n, cnode->time);
		printf("msg_num:%d path:%d qos:%d up:%d bw:%d down:%d ackhop:%d whichPart:%d\n",msg_num,path,qos,up,bw,down,ackhop,whichPart);
	}

	if(debug && whichPart == START)
	{
		printf("ACK packette received by START node %d at time %f\n",n, cnode->time);
		printf("msg_num:%d path:%d qos:%d up:%d bw:%d down:%d ackhop:%d whichPart:%d\n",msg_num,path,qos,up,bw,down,ackhop,whichPart);
	}


	if (n==43 && down==19 && bw==21 && path==45) {
		printf("test why sort not done properly, with 6 1 and 24 2\n");
	}
   find_symbol_def(trail, ev->msg, "TRAIL");

	


	if (debug_node && choosen == n) {
		fprintf(choosen_out, 
			"at %f, %d received ACK: msg_num=%d, path=%d, qos=%d, up=%d, bw=%d, down=%d, ackhop=%d, whichPart=%d, path_bw=%d\n", cnode->time, cnode->id, 
			msg_num, path, qos, up, bw, down, ackhop, whichPart, path_bw);
		fprintf(choosen_out, "trail=%s\n", trail);
		Print_Node_Table(cnode);
		}
	

	find_symbol_def(msgStr, ev->msg, "ID");
	


	sscanf(msgStr, "%d", &msgID);




   if (In_Trail(n, trail)) 
	{
		Append(trail, cnode->name);
		init_symbol(&ev->msg, "TRAIL", trail);
		
		if(debug)
			printf("IN Trail aPacket path:%5d qos:%5d n:%5d bw:%5d %5d\n", path, qos, n, 0, NO_NODE);
	
		

      sprintf(aPacket, "%5d%5d%5d%5d%5d", path, qos, n, 0, NO_NODE);
      Forward_Msg(cnode, n, ev, "CONF ", down, aPacket);


	


      return;
	}
	Append(trail, cnode->name);
	init_symbol(&ev->msg, "TRAIL", trail);
	if (ackhop > MAX_HOPS) {
			
			if(debug)
				printf("ackhop>MAX_HOPS aPacket path:%5d qos:%5d n:%5d bw:%5d %5d\n", path, qos, n, 0, NO_NODE);
		

			sprintf(aPacket, "%5d%5d%5d%5d%5d", path, qos, n, 0, NO_NODE);
			Forward_Msg(cnode, n, ev, "CONF ", down, aPacket);


			

			return;
		}
	/* Should not happen unless 0 bandwidth CONFF erased all
				  entries for the path and messages crossed.
			  */
	if (!start_finger)
		return;






	/* Messages crossed. */ 
	if (start_finger->confirmed.conf_recv == FINAL)
		return;

	if (whichPart == MIDDLE)
	{
		nodelist *up_ptr = NULL;
		int trail_length, actualbw,
		index,
		acked_bw = 0,
		*unavailables = NULL,
		downlinks = num_spares(cnode, down);
		char trail[256];   /* This limits a path to 64 nodes. */
		int forward_bw = 0;
		int diff_bw = 0; 
		/* keep track the difference btw request bw and that
		 on the downlink, in racing situation this could be nozero 
		 case 3-40cut, n=20, rcv n==31, bw =2 but only 1 bw left on 20-31 link*/

		tablerec *ack_finger = NULL;
		boolean all_confirmed = true;

		

		finger = start_finger; 


	

		while(finger && finger->path == path && finger->confirmed.value)
		{
			if (finger->down == down)
			{
				acked_bw += finger->bw;
				if (finger->confirmed.value == CONFIRMED)
					conf_bw += finger->bw;
				else
					all_confirmed = false;
			}
			finger = finger->next;
		} /* end while finger */

	


	
		ack_bw = bw - acked_bw;

		/* AHA debug */
		if(ack_bw < 0 )
		{
			fprintf(stderr, "ACK_Processing bw acked_bw => ack_bw < 0\n");
			exit(1);
		}

		actualbw = GetActualBw(ev);


		/* to prevent from sending a request bigger than actual request */
		/* sometimes happen when the acc bw has not been updated */
		if(ack_bw > actualbw)
		{
			ack_bw = actualbw;
		}

		forward_bw = ack_bw;
		if (ack_bw == 0) { /* racing situation, send back conf 0 */
					sprintf(aPacket, "%5d%5d%5d%5d%5d", 
						path, qos, n, 0, NO_NODE);
				
					if(debug)
						printf("Racing situation aPacket path:%5d qos:%5d n:%5d bw:%5d %5d\n", path, qos, n, 0, NO_NODE);
					

					Forward_Msg(cnode, n, ev, "CONF ", down, aPacket);


				

					return;
		}
		if (downlinks)
		{ 

		


			/* Get list of nodes that message has already passed through. */
			find_symbol_def(trail, ev->msg, "TRAIL");
			trail_length = (strlen(trail) + 1) / NodeNameLength;
			unavailables = (int *) malloc(sizeof(int) * trail_length);
			for (index = 0; index < trail_length; index++)
				unavailables[index] = Get_Node_ID(&(trail[NodeNameLength * index]));
			/* set unavailables so that no ack sending cycle can be formed */

			/* Make and Ack any additional connections to down. */

			if (downlinks < ack_bw)
			{
				diff_bw = ack_bw - downlinks;
				bw = acked_bw + downlinks; 
				ack_bw = downlinks;
			}
			
		
			finger = Get_suitable_entry_with_min_hop_for_ack(cnode,path,ackhop,unavailables,down,trail_length);
			
			while (finger && finger->path == path && ack_bw)
			{

			

				if (up_ack(cnode, path, finger->up) <= ACK_LIMIT)
				{
			

					if (ackhop + finger->hopcount <= MAX_HOPS) 
					{

					
						if (up_available(finger->up, trail_length, unavailables))
						{

							int uplinks = num_spares(cnode, finger->up);

							if (finger->up != down) 
							{
								if (uplinks)
								{
									int connect_bw = uplinks > ack_bw ? ack_bw : uplinks;
									int acc_bw;
						
									if (finger->bw > connect_bw)
									{
										tablerec *tempfinger = NULL;
										tempfinger = (tablerec *)malloc(sizeof(tablerec));
										Copy_Tablerec(finger, tempfinger);
										tempfinger->bw -= connect_bw;
										tempfinger->time_received = cnode->time; 
										finger->bw = connect_bw;
										Insert_Before_Tablerec(cnode, finger, tempfinger);
									} /* end if finger->bw > connect_bw */
									else 
									{
									finger->bw = connect_bw;
									}
									finger->ackhop = ackhop;
									Take_Spares(cnode, finger->up, down, connect_bw);
									finger->confirmed.ack.recv = msg_num;
									finger->confirmed.ack.down = msg_num;
									finger->confirmed.ack.up = 
									get_ack_num(cnode, path, finger->up);
									finger->confirmed.conf_recv = 0;
									finger->confirmed.value = ACKED;
									finger->time_connected = dxc_time(cnode, connect_bw);
									finger->down = down;
									sprintf(msgStr, "%2d", finger->confirmed.ack.up);
									init_symbol(&ev->msg, "ID", msgStr);
									acc_bw = accumulated_bw(cnode, path, finger->up);

									

									InitActualBw(ev, finger->bw);

									sprintf(aPacket, "%5d%5d%5d%5d%5d%2d", path, qos, NO_NODE, 
									acc_bw, n, ackhop+1);
									Forward_Msg(cnode, n, ev, "ACK  ", finger->up, aPacket);
									add_up_ack(cnode, path, finger->up);
									acked = true;
										ack_bw -= connect_bw;
								} /* end if uplinks */
							} /* end if finger->up != down */

						} /* end if up_available */
					} /* if  (ackhop+finger->hopcount <= MAX_HOPS) */
				} /* if (up_ack() <= ACK_LIMIT) */


			finger = Get_suitable_entry_with_min_hop_for_ack(cnode,path,ackhop,unavailables,down,trail_length);
			
			/*	finger = finger->next;*/
			} /* end while finger */
			/* move inside if downlinks since downlinks == 0
				ack_bw will not be set */
			if (ack_bw != 0) 
			{
				if (ack_bw == forward_bw) 
				{
					sprintf(msgStr, "%2d", msgID);
					init_symbol(&ev->msg, "ID", msgStr);
					sprintf(aPacket, "%5d%5d%5d%5d%5d", 
						path, qos, n, 0, NO_NODE);

					


					Forward_Msg(cnode, n, ev, "CONF ", down, aPacket);

				

					return;
				} 
				else 
				{
					/* should add diff_bw */
					sprintf(msgStr, "%2d", msgID);
					init_symbol(&ev->msg, "ID", msgStr);
					sprintf(aPacket, "%5d%5d%5d%5d%5d", 
						path, qos, n, ack_bw+diff_bw, NO_NODE);


					




					Forward_Msg(cnode, n, ev, "CONFX", down, aPacket);
					Pack_Table(cnode);
				}
			} 
			else 
			{ /* ack_bw == 0 */ 
				if (diff_bw != 0) 
				{
					sprintf(msgStr, "%2d", msgID);
					init_symbol(&ev->msg, "ID", msgStr);
					sprintf(aPacket, "%5d%5d%5d%5d%5d", 
						path, qos, n, diff_bw, NO_NODE);

					



					Forward_Msg(cnode, n, ev, "CONFX", down, aPacket);
					Pack_Table(cnode);
					}
			}
		} 
		else 
		{ /* downlinks == 0, racing situation */
				sprintf(msgStr, "%2d", msgID);
				init_symbol(&ev->msg, "ID", msgStr);
				sprintf(aPacket, "%5d%5d%5d%5d%5d", path, qos, n, 0, NO_NODE);

			


				Forward_Msg(cnode, n, ev, "CONF ", down, aPacket);



			

				return;
		}

		Pack_Table(cnode);
		free(unavailables);

		/* Chow: assume DXC connect server and restoration process are 
			independent 
		if (acked && cnode->time < connection_time)
			cnode->time = connection_time;
		*/
	}  /* end if MIDDLE node */

	/* When the ACK reaches the START node, it has reached its
				  destination, now it is up to the START node to check how much 
				  bandwidth it can contribute to the path via the connections to the  
				  node that sent the ACK and initiate a CONF.
			  */ 
	if (whichPart == START) 
	{  
		int confirmed_bw = 0,actualbw,
		initial_confirmed_bw = 0,
		final_confirmed_bw,
		conf_bw,
		downlinks = num_spares(cnode, down);
		boolean all_confirmed = true,
		new_connection = false,
		max_restoration = false; 

		




		if (path == 413 && n == 13) {
			printf("Chow: check how PDONE is sent\n");
		}
		finger = start_finger;
		while(finger && finger->path == path && finger->confirmed.value ==
		    CONFIRMED)
		{
			initial_confirmed_bw += finger->bw; 
			if (finger->down == down)
				confirmed_bw += finger->bw;
			finger = finger->next;
		}

		if (finger == NULL) return; /* Chow: tempoary fix */
		final_confirmed_bw = initial_confirmed_bw;
		conf_bw = bw - confirmed_bw; 

		
		actualbw = GetActualBw(ev);
		if(conf_bw > actualbw)
			conf_bw = actualbw;



		if (downlinks)
		{  
			int new_bw = downlinks > conf_bw ? conf_bw : downlinks;

			new_connection = true;
			final_confirmed_bw += new_bw; 
			finger = get_unconnect_finger(cnode, path);
			if (finger == NULL) return;
			Take_Spares(cnode, down, NO_NODE, new_bw);
			if (finger->bw <= new_bw)
			{  
				finger->bw = new_bw;
				finger->confirmed.value = CONFIRMED;
				finger->time_connected = dxc_time(cnode, finger->bw);
				finger->down = down;
			} /* end if finger->bw <= new_bw */
			else
			{
				tempfinger = (tablerec *) malloc(sizeof(tablerec));
				Copy_Tablerec(finger, tempfinger);
				finger->bw -= new_bw;
				tempfinger->bw = new_bw;
				tempfinger->confirmed.value = CONFIRMED;
				tempfinger->time_connected = dxc_time(cnode, new_bw);
				tempfinger->down = down; 
				Insert_After_Tablerec(cnode, finger, tempfinger);
			} /* end else finger->bw > new_bw */
			Pack_Table(cnode);
			start_finger = get_start_finger(cnode, path);
			conf_bw -= new_bw;
		} /* end if downlinks */


		if (conf_bw)
			bw -= conf_bw;

		init_symbol(&ev->msg, "TRAIL", "");

		/* Set all_confirmed. */
		all_confirmed = !spares_remaining(cnode); 
		finger = cnode->tablehead;
		while(all_confirmed && finger)  
		{
			if (finger->confirmed.value == ACKED)
				all_confirmed = false;   
			finger = finger->next;
		}

		/* Send CONF */
		sprintf(aPacket, "%5d%5d%5d%5d%5d", path, qos, n, bw, NO_NODE);
		init_symbol(&ev->msg, "TRAIL", cnode->name);
		if (final_confirmed_bw == path_bw || all_confirmed)
		{
			
			

			Forward_Msg(cnode, n, ev, "CONFF", down, aPacket);
			max_restoration = true;
		}
		else
		{	
			
			
			Forward_Msg(cnode, n, ev, "CONF ", down, aPacket);
		}
		/* DXC and restroation processes are independent 
		if (new_connection
		    && cnode->time < connection_time)
			cnode->time = connection_time;
		*/
		bwCount += (final_confirmed_bw - initial_confirmed_bw);

	

		return;
	} /* end else if whichPart is START */


	


} /* end ACK_Processing */


/* in:  erec containing the CONF or CONFF message received
				identification number of node that received the CONF
				pointer to node information for node 'n'
				type of CONF--'N' for CONF
								  'F' for CONFF

		 When a node receives a CONF or CONFF, the node confirms any
		 additional bandwidth needed to bring its confirmed bandwidth for the
		 path up to the bandwidth in the CONF or CONFF.  

		 If the node has received a CONF and the node is waiting for an 
		 additional CONF from the sender of the CONF, it continues to protect 
		 any acked but unconfirmed connections.

		 If the node has received the last CONF expected from the CONF's
		 sender, the node will release any acked but unconfirmed
		 connections with the CONF's sender.  If the node has only confirmed
		 connections and it has no spares left, the node will check to see if
		 it is the end node for any paths that have not been fully restored 
		 and are not already known to be maximally restored.  If the node is
		 an end node for paths not already known to be maximally restored,
		 it will issue PDONEs with a packette for each of the paths it has
		 just discovered to be maximally restored.  The PDONEs will be sent 
		 each of the node's immediate neighbors. 

		 If the node received a CONFF, the node will release any acked but
		 unconfirmed connections with the CONFF's sender.  The node will
		 mark the path as known to be maximally restored.  If the node has
		 any unconfirmed connections or any spares remaining, the node will
		 issue a PDONE for the path in the CONFF to all its immediate
		 neighbors except those to which it will forward the CONFF and those
		 which are in the TRAIL in the CONFF.  If the node received a CONFF
		 that confirmed its last unconfirmed connection and the node has no
		 spares remaining, the node will determine if it is an end more for
		 any other path not already known to be maximally restored.  If so,
		 the node will issue multipackette PDONEs to all its immediate
		 neighbors.  All immediate neighbors that would have received
		 notification that the path in the CONFF was maximally restored will
		 receive PDONEs with packettes for the path in the CONFF and all the
		 paths just discovered to be maximally restored.  The other immediate
		 neighbors will receive PDONEs with packettes for all the paths
		 just discovered to be maximally restored. 
		 
		 Depending on the position of the node on the path in the message,
		 whether the message is a CONF or a CONFF and on the configuration--
		 straight forward, multiple downstream, multiple upstream or worst 
		 case--none, one or many CONFs or CONFFs may be forwarded or ACKs
		 initiated upon receipt of a CONF/CONFF.  

		 If a CONF was received and spares were released, the node will see 
		 if it can use the released spares to initiate restoration of any path
		 for which it is the FINISH node.

		 If a CONFF was received and spares were released, the node will
		 see if it can use the released spares to initiate new restoration
		 attempts for paths other than the path in the CONFF received.

		 If a node is a MIDDLE node on the path in the CONF or CONFF, the
		 situation will determine whether CONFs or CONFFs are forwarded.
		 In the straight forward case, receipt of a CONF/CONFF results
		 in the CONF/CONFF being forwarded to the downstream node of the 
		 connection with the CONF/CONFFs sender.  

		 In the multiple upstream case, a MIDDLE node received one ACK and
		 forwarded it to several upstream nodes.  The MIDDLE node may not
		 forward a CONF/CONFF downstream until it has received a CONF or
		 CONFF from each of those upstream nodes.  If a CONFF is received
		 from one of those upstream nodes, a CONFF is forwarded, otherwise
		 a CONF is forwarded.

		 In the multiple downstream case, each ACK received by the MIDDLE
		 node resulted in one ACK being forwarded upstream.  Prior to the
		 existence of a multiple downstream case, one downstream node sent
		 an ACK and it was forwarded to the upstream node.  When a second
		 ACK was received from a different node and forwarded to the same
		 upstream node, the case became multiple downstream and the number
		 of the forwarded ACK was set equal to the number of the ACK received
		 from the first downstream node plus the number of the ACK received
		 from the second downstream node.  If the CONF in reply to the
		 earlier ACK is for the entire requested bandwidth, the CONF may be
		 forwarded immediately.  If the CONF in reply to the earlier ACK is 
		 NOT for the entire requested bandwidth, it is possible that the
		 CONF/CONFF for the later request will still be for the full bandwidth
		 of the later request, so the CONFs/CONFFs for the earlier and later
		 requests will both be sent when the replies to both requests have
		 been received.

		 And of course, the situation for a MIDDLE node upon receipt of a 
		 CONF/CONFF in the worst case combines elements of both the multiple 
		 upstream and multiple downstream cases, i.e., its name worst case. 
	 */
void CONF_Processing(erec *ev, int n, node *cnode, char type)
{
	char aPacket[PACKET_SIZE+3],
	msgStr[MSG_BASE_SIZE],
	trail[256];
	int path,
	qos,
	up,
	bw,
	down,
	whichPart, 
	path_bw,
	msg_num; 
	boolean still_waiting = false,     /* if true, waiting for a higher
										numbered CONF from the same 
										upstream node that sent the CONF
															 */ 

	conf_outstanding = false;  /* if true, this node is still
								waiting for an additional CONF
								for this path */

	tablerec *finger = NULL,
	*entry = NULL,
	*start_finger = NULL,
	*tempfinger = NULL,  
	*ack_finger = NULL,
	*nonodefinger = NULL; 
	boolean final = (type == 'F') ? true : false,
	spares_released = false,
	ackable = (final == true) ? false : true;
	Packet_info *pdones = NULL,
	*last = NULL,
	*present = NULL,
	*previous = NULL;
	int i; /* looping counter */
	int ackhop;

	


	/* Initialization common to all single packette messages. */
	init_one_packette_msg(ev, cnode, &msg_num, &path, &qos, &up, &bw, &down,
	&start_finger, &whichPart, &path_bw);
	/* Chow: implement adaptive ack limit  
		for positive bw conf msg decrease ack count down by 1 until 
		found 22-29 looping or not terminating with this adaptive ack 
		need further study 
	*/





	/* AHA if(bw!=0)*/
	if (bw != 0 && type != 'X') decrease_up_ack(cnode, path, up);
	find_symbol_def(trail, ev->msg, "TRAIL");

	/* find entry that trigger the confirm msg and set ackhop */
	finger = start_finger;
	while (finger && finger->up == up) 
	{
		if (finger->confirmed.ack.down == msg_num && finger->confirmed.value==2) 
		{
			ackhop = finger->ackhop;
			finger = NULL;
		} 
		else 
		{
			finger = finger->next;
		}
	}
	if (debug_node && choosen == n) 
	{
		fprintf(choosen_out, 
			"at %f, %d received CONF%c: msg_num=%d, path=%d, qos=%d, up=%d, bw=%d, down=%d, whichPart=%d, path_bw=%d\n", 
			cnode->time, cnode->id, type,
			msg_num, path, qos, up, bw, down, whichPart, path_bw);
		fprintf(choosen_out, "trail=%s\n", trail);
		Print_Node_Table(cnode);
	}

	/* Remove from TRAIL */
	if (bw == 0 || type == 'X') 
	{
		i = strlen(trail);
		trail[i-NodeNameLength] = '\0'; 
		init_symbol(&ev->msg, "TRAIL", trail);
	}
	

	if (path == 40 && bw == 9 && n == 18 && up == 19) 
	{
		printf("chow: test 13-44cut.l for why conff stop not reach 29 and or not confirmed. \n");
	}


	if (whichPart == MIDDLE) 
	{ 
		int conf_bw = bw, 
		conf_num = msg_num, 
		down = NO_NODE,  
		prior_msg_num;
		nodelist *downs = NULL,
		*ptr,
		*prev_ptr;
		boolean *notified = NULL,
		found,
		up_entry;
		int entry_bw, entry_down, entry_ackhop, ack_bw;
		tablerec *entry;
		boolean search;

		if (!bw && !start_finger)
			return;


		/* Rejecting whole or part of ack request */
		if (!bw || type == 'X')  
		{


			/* AHA debug */

		if(debug && !bw)
		{
			printf("MIDDLEnode %d received CONF#%d(bw=%d) at time %f\n", cnode->id,msg_num,bw,cnode->time);
		printf("( msg_num=%d path=%d qos=%d up=%d bw=%d down=%d whichPart=%d path_bw=%d \n", msg_num, path, qos, up, bw, down, whichPart, path_bw);
		}
		if(debug && type == 'X')
		{
			printf("MIDDLEnode %d received CONFX#%d(bw=%d) at time %f\n",cnode->id,msg_num,bw,cnode->time);
		printf("( msg_num=%d path=%d qos=%d up=%d bw=%d down=%d whichPart=%d path_bw=%d \n",  msg_num, path, qos, up, bw, down, whichPart, path_bw);
		}



			finger = start_finger;
			/* Chow: in 6-8 layer3m node 4 recevived conf 0 from 24,
			  the code below did not find 24 2 8 19
				it stops at first entry 6 1 21 19 and update the wrong entry
			while (finger && finger->up != up && finger->path == path
				&& finger->down == NO_NODE) 
				finger = finger->next;
			*/
			search = true;

			/* find entry that corresponds to msg */
			while (search && finger && finger->path == path) 
			{
				if (finger->up == up && finger->down != NO_NODE &&
					 finger->confirmed.value == 2 && 
					 /* chow the above this added to avoid modify CONFIRMED entry */
					 finger->confirmed.ack.up == msg_num) search =false;
				else 
					finger = finger->next;
			}

			if (finger == NULL) 
			{
				printf("cannot find the entry with path=%d, up=%d\n", path, up);
				/* Chow: 28-29 cut with 20 packetslenth path 818 conf with 0 bw
					 arrive but can not find 818 entry with up =27 (was removed?)
					 only up =22 left */
					 /* Chow: try to see replace exit() and return for difference 
				exit(1);
				*/
				return;
			}
			if (finger->path != path) 
			{
				printf("cannot find the entry with path=%d, up=%d\n", path, up);
				/* 
				exit(1);
				*/
				return;
			}
			/* find path and up entry */
			if (type == 'X') 
			{
				if (finger->bw < bw) 
				{
					/* racing situation */
					bw = finger->bw;
				} 
				else if (finger->bw != bw) 
					split_entry_bw(cnode, finger, bw);
			} 
			/* set conf_recv */
			/* Chow try to set conf_recv to 0 and time_connected = 0.0 

			finger->confirmed.conf_recv = 0; 
			finger->time_connected = 0.0;

			But unlike the finish node case, this change worsens 
			several cases, including the 3-40 case
			from 113 to 240 sec.  Worth future study 
				*/

			/* release rejected bw */
			Give_Spares(cnode, finger->up, NO_NODE, finger->bw);
			finger->confirmed.conf_recv = msg_num; 
			finger->confirmed.value = 0; 
			/* need to set 0 otherwise will be
				considered as conf outstanding */
			entry_down = finger->down;
			entry_bw = finger->bw;
			ack_bw = entry_bw;
			entry_ackhop = finger->ackhop;
			entry = finger;

			/* search for the entries with the same path but different up node 
				and the conf_recv is not set yet.
				retry by sending ack to those upstream node one at time
				until there are no more of those */

			/* cannot use get_unconnect_finger here since we change 
				entry->confirmed.value = 0, the search will stop there
				not at the first unconnect entry */
			/*finger = start_finger;*/
			finger = Get_entry_min_hop_and_spares_suitable_for_retry(cnode, up, entry_ackhop, path, entry_down, trail);
			search = true;
			while (search && finger != NULL && finger->path==path) 
			{
				/* find suitable up node */
				if (finger->confirmed.conf_recv == 0 && 
					finger->confirmed.value == 0 &&
			/*		finger->down == NO_NODE && Chow: add this addtional cond */
					finger->up != entry_down &&
					finger->up != NO_NODE &&
					!In_Trail(finger->up, trail)) 
				{
					int link_spare;

					/* find actual spare left on the link */
					link_spare = num_spares(cnode, finger->up);
					/* add the additional hopcount selection criteria make sure
						it can reach start node within repeat limit*/

					if (up_ack(cnode, path, finger->up) <= ACK_LIMIT) 
					{
						/* Chow add fair search condition */
						if ((finger->up==up && Only_available_entry(cnode, finger))||
							finger->up != up) 
						{
							/* check hopcount limit */
							if (link_spare != 0 && finger->hopcount+entry->ackhop<=MAX_HOPS)
							{

								/*there exists enough spares upnode(finger->up) */
								if (link_spare >= finger->bw) 
								{
									/* The available bw to upnode does not cover */
									/*the whole rejected bw*/ 
									if (entry_bw > finger->bw) 
									{	
										/* send ACK to finger->up with ack_bw=finger->bw */
										update_entry_ack(cnode,ev, finger, entry, msg_num);
										entry_bw -= finger->bw;
									} 
									/* The available bw to upnode is equal */
									/*the whole rejected bw*/ 
									else if (entry_bw == finger->bw) 
									{
										update_entry_ack(cnode,ev, finger, entry, msg_num);
										entry_bw = 0;
										search = false; /* found enough entries for retry */
									} 
									else 
									/* There is more bw available to upnode than needed */ 
									{ /* entry_bw < finger->bw  */
										split_entry_bw(cnode, finger, entry_bw);
										/* send ACK to finger->up with ack_bw=rejected bw*/
										update_entry_ack(cnode,ev, finger, entry, msg_num);
										entry_bw = 0;
										search = false; /* found enough entries for retry */
									}
								} 
								else 
								/* the number of available spares to upnode is smaller than */
								/* the number of bw according to entry */
								{/* link_spare < finger->bw */


									/* The rejected bw is bigger than there are available spares */
									if (entry_bw > link_spare) 
									{
										split_entry_bw(cnode, finger, link_spare);
										/* send ACK to finger->up with ack_bw=finger->bw */
										update_entry_ack(cnode, ev, finger, entry, msg_num);
										entry_bw -= link_spare;
									} 
									/* The rejected bw is equal to the number of available spares */
									else if (entry_bw == link_spare) 
									{
										split_entry_bw(cnode, finger, link_spare);
										/* send ACK to finger->up with ack_bw=rejected bw*/
										update_entry_ack(cnode,ev, finger, entry, msg_num);
										entry_bw = 0;
										search = false; /* found enough entries for retry */
									} 
									else 

									/*The rejected bw is covered by the available spares */
									{ /* entry_bw < link_spare  */
										split_entry_bw(cnode, finger, entry_bw);
										/* send ACK to finger->up with ack_bw=rejected bw */
										update_entry_ack(cnode, ev, finger, entry,msg_num);
										entry_bw = 0;
										search = false; /* found enough entries for retry */
									}
								}
							} /* end if link_spare != NULL */
						} /* Chow: check fair conditiontion */
					} /* within ACK_LIMIT */
				}/* if(finger..*/


				finger = Get_entry_min_hop_and_spares_suitable_for_retry(cnode, up, entry_ackhop, path, entry_down, trail);
				/*finger = finger->next;*/
			}/* end while */
			if (type == 'X') /* CONFX( ) message */
			{/* some bw got rejected */

				/* entry_bw contais the bw that could not be handled by the node after */
				/* search of available upnodes => rejected bw => send CONFX to down node */
				/* with the bw that could not be handled */ 
				if (entry_bw) 
				{
					sprintf(msgStr, "%2d", entry->confirmed.ack.down);
					init_symbol(&ev->msg, "ID", msgStr);
			        sprintf(aPacket, "%5d%5d%5d%5d%5d", 
								path, qos, n, entry_bw, NO_NODE);
			
					
					Forward_Msg(cnode, n, ev, "CONFX", entry->down, aPacket);
					Give_Spares(cnode, NO_NODE, entry->down, entry_bw);
				}
				Pack_Table(cnode);

				

				return;
			}

			/* All rejected bw was covered with other upnodes */
			if (entry_bw == 0) 
			{
				Pack_Table(cnode);

			



				return; /* since all bw are retried */
			}
			
			/* the whole rejected bw in the CONF(bw=0) could not be covered */
			else if (entry_bw != ack_bw)
			{
				sprintf(msgStr, "%2d", entry->confirmed.ack.down);
				init_symbol(&ev->msg, "ID", msgStr);
			    sprintf(aPacket, "%5d%5d%5d%5d%5d", 
								path, qos, n, entry_bw, NO_NODE);




				Forward_Msg(cnode, n, ev, "CONFX", entry->down, aPacket);
				Give_Spares(cnode, NO_NODE, entry->down, entry_bw);
				/* chow: should not send duplicate msg conf0
				add the following return; */
				Pack_Table(cnode);

			


				return;
			} 
			else /* (entry_bw == ack_bw) nothing of the rejected bw could be covered send CONF(bw=0) to down node */
			{ 
				/* forward conf 0 */
				/* need to check if ack with same down and ack.down exist*/
				/* should not send conf 0 if exist such entry */
				sprintf(msgStr, "%2d", entry->confirmed.ack.down);
				init_symbol(&ev->msg, "ID", msgStr);
				if (!same_ack_down(cnode, entry)) 
				{
					sprintf(aPacket, "%5d%5d%5d%5d%5d", 
									path, qos, n, 0, NO_NODE);

					
					Forward_Msg(cnode,n, ev, "CONF ", entry->down, aPacket);
					/* chow: should not send duplicate msg conf0
					add the following return; */
				} 
				else 
				{

					sprintf(aPacket, "%5d%5d%5d%5d%5d", 
									path, qos, n, entry->bw, NO_NODE);

				
					Forward_Msg(cnode,n, ev, "CONFX", entry->down, aPacket);
				}
				Give_Spares(cnode, NO_NODE, entry->down, entry_bw);
				Pack_Table(cnode);

			
				return;
			}/*end else */


			/* AHA do not think that code below will ever be executed */
			if (entry_bw != ack_bw)  
			{ 
				Pack_Table(cnode);
				return;
			} 
			else 
			{ /* does not found entry to retry */
				entry->confirmed.conf_recv = 0; /* reset */
				entry->confirmed.value = 2;
			}
			/* end unexecuted code */


		}/*end if(!bw...)  */
		
	/* ************END CONFX & CONF(0) Processing***************/


	/********** START CONF(X) & CONFF Processing *********************/
		
		/* CONFF ( final=(type=F)true?false ) */
		
		if (final)
		{
			if(debug)
			{
			printf("MIDDLEnode %d received CONFF#%d(bw=%d) at time %f\n",cnode->id,msg_num,bw,cnode->time);
			printf("( msg_num=%d path=%d qos=%d up=%d bw=%d down=%d whichPart=%d path_bw=%d \n",  msg_num, path, qos, up, bw, down, whichPart, path_bw);
			}

			/* check which nodes this msg has visited and set them to "notified" */ 
			notified = (boolean *)malloc(sizeof(boolean) * (netsize + 1));
			for(i = 1; i <= netsize; i++)
				notified[i] = false;
			find_symbol_def(trail, ev->msg, "TRAIL");
			for(i = 0; i < (int)((strlen(trail) + 1) / NodeNameLength); i ++)
				notified[Get_Node_ID(&(trail[NodeNameLength * i]))] = true;
			Append(trail, cnode->name);
			init_symbol(&ev->msg, "TRAIL", trail);
		} /* end if final */
		if(type == 'N')
		{
			if(debug)
			{
			printf("MIDDLEnode %d received CONF#%d(bw=%d) at time %f\n",  cnode->id,msg_num,bw,cnode->time); 
			printf("( msg_num=%d path=%d qos=%d up=%d bw=%d down=%d whichPart=%d path_bw=%d \n", msg_num, path, qos, up, bw, down, whichPart, path_bw);
			}
		}



		/* init int conf_bw = bw*/ 
		if (conf_bw)
		{
			/* check how much bw that has been confirmed for path ( to upnode)*/
			/* Confirm bandwidth. */
			finger = start_finger;
			while(finger && finger->path == path &&
			    finger->confirmed.value == CONFIRMED)
			{ 
				if (finger->up == up)
				{
					/* If MULTIPLE DOWN STREAM CASE */
					/* Need to check if CONF is in reply to all bw */

					/* confirmed.conf_recv contains FINAL after path has been
					completly restored, 0 if no CONF received for this replacement
					connection and  >0 message number of last CONF received */

					/* update confirmed */
					if (finger->confirmed.conf_recv < msg_num)
						finger->confirmed.conf_recv = msg_num;

					conf_bw -= finger->bw;
				}
				finger = finger->next;
			} /* end while finger */
		} /* end if conf_bw */

		
		if (!conf_bw)
		{
			tablerec *ack_finger = get_ack_finger(cnode, path);

			finger = ack_finger;
			while(down == NO_NODE && finger && finger->path == path &&
			    finger->confirmed.value)
			{
				/* finger->confirmed.ack.up is the id number of last ACK
				forwarded to the upstream node */

				/* find entry with corresponding ACK message sent */
				if (finger->up == up && finger->confirmed.ack.up ==
				    msg_num && down == NO_NODE)
				{

					down = finger->down;
					if (n == 34) 
						printf("Chow in 8-22cut, forward to node 0\n");
					ptr = (nodelist *) malloc(sizeof(nodelist));
					ptr->num = down;
					ptr->acked = true;
					ptr->acked_bw = 0;
					ptr->msg_num = finger->confirmed.ack.down;/* id# of last ACK from downstream node
															  for which an ACK was forwarded */

					/* set entry to point at ACK entry that corresponds to msg */
					entry = finger;
					ptr->next = NULL;

					/* init nodelist *downs = NULL */
					
					if (!downs)
						downs = ptr;
					else
					{
						prev_ptr = downs;
						while (prev_ptr->next)
							prev_ptr = prev_ptr->next;
						prev_ptr->next = ptr;     
					}
					if (finger->confirmed.conf_recv < msg_num)
						finger->confirmed.conf_recv = msg_num;         
				} /* end if finger->up == up */
				finger = finger->next;
			} /* end while(down ... */

			if (down != NO_NODE)
			{ 
				finger = ack_finger;
				while(finger && finger->path == path &&
				    finger->confirmed.value)
				{

					/* confirmed.ack.up contains id number of last ACK forwarded to the
					upstream node */

					if (finger->up == up && finger->confirmed.ack.up <= 
					    msg_num && finger->down != down)
					{
						ptr = downs;
						while (ptr && ptr->num != finger->down)
						{
							prev_ptr = ptr;
							ptr = ptr->next;
						} /* end while ptr */
						if (!ptr)
						{
							ptr = (nodelist *) malloc(sizeof(nodelist));
							if (n==34) 
								printf("Chow in 8-22cut, forward to node 0\n");
							ptr->num = finger->down;
							ptr->acked = true;
							ptr->acked_bw = 0;
							ptr->msg_num = finger->confirmed.ack.down;
							ptr->next = NULL;
							prev_ptr->next = ptr;
						} /* end if not ptr */
						if (finger->confirmed.conf_recv > 0 && 
						    finger->confirmed.conf_recv < msg_num)
							finger->confirmed.conf_recv = msg_num;         
					} /* end if finger->up == up */
					finger = finger->next;
				} /* end while finger */
			} /* end if down != NO_NODE */
		} /* end if(!conf... no additional bandwidth was confirmed */

		i=0;


		/* not all was confirmed */
		while (conf_bw)
		{
			i++;
			if (i==1000) return;
			down = NO_NODE;
			finger = get_ack_finger(cnode, path);  
			if (conf_bw && !finger)
			{
				/*
				fprintf(stderr, "path %d, node %d, bw %d, up  %d\n",
				path, n, bw, up);
				fprintf(stderr, "msg_num %d\n", msg_num);
				*/
			/* Chow: on layer3m.net /b=1-9 there is an instance	exit(1); */
			/*       temporary fix */
				return;
			} /* end if conf_bw */
			while(conf_bw && finger && finger->path == path &&
			    finger->confirmed.value)
			{ 
				/* Chow: add ack.up == msg_num to find the right entry,
					based on n=19, 6-8cut case 
				if (finger->up == up && finger->confirmed.ack.up == msg_num)
				*/
				if (finger->up == up)
				{
					if (down == NO_NODE)
					{
						down = finger->down;     
						if (n==34) 
							printf("Chow: in 8-22 cut, finger->down==0\n");
						ptr = (nodelist *) malloc(sizeof(nodelist));
						ptr->num = down;
						ptr->acked = true;
						ptr->acked_bw = 0;
						ptr->msg_num = finger->confirmed.ack.down;
						entry = finger;
						ptr->next = NULL;
						if (!downs)
							downs = ptr;
						else
						{
							prev_ptr = downs;
							while (prev_ptr->next)
								prev_ptr = prev_ptr->next;
							prev_ptr->next = ptr;
						}
					} /* end if down == NO_NODE */
					if (finger->down == down)
					{ 
						if (finger->bw <= conf_bw)
						{
							conf_bw -= finger->bw;
						} /* end if finger->bw <= conf_bw */
						/* else finger->bw >conf_bw */
						else
						{
							tempfinger = (tablerec *) malloc
							    (sizeof(tablerec));
							Copy_Tablerec(finger, tempfinger);
							tempfinger->bw = finger->bw - conf_bw;
							tempfinger->confirmed.conf_recv = msg_num;
							tempfinger->time_received = cnode->time;
							finger->bw = conf_bw;
							conf_bw = 0;
							Insert_After_Tablerec(cnode, finger, tempfinger);
							still_waiting = true;
							conf_outstanding = true;
						} /* end else finger->bw > conf_bw */
						finger->confirmed.value = CONFIRMED;
						finger->time_connected = dxc_time(cnode, finger->bw);
						finger->confirmed.conf_recv = msg_num;          
						if (finger->confirmed.ack.down > ptr->msg_num)
							ptr->msg_num = finger->confirmed.ack.down;  
					} /* end if finger->down == down */
				} /* end if finger->up == up */
				finger = finger->next;
			} /* end while(conf_bw && finger..  */
			Pack_Table(cnode);
			start_finger = get_start_finger(cnode, path);
		} /* end while conf_bw */

		/* Set cleanup booleans. */
		ack_finger = get_ack_finger(cnode, path);
		finger = ack_finger;
		while(finger && finger->path == path && finger->confirmed.value)
		{
			if (finger->up != up)
			{
				conf_outstanding = true;
			}
			/* else finger->up == up */
			/* Chow: it is possible finger has lower ack.up but did not
				receive conf yet, should allow to stand, if not setting
				the still_waiting and conf_outstanding, this entry will be
				removed. Based on 6-8cut n=19 4 8 1 2 1/2/1 but msg_num is 3
				XXXX
			*/
			/*
			else if (!final)
			*/
			/*
			else if (finger->confirmed.ack.up > msg_num && !final &&
			    finger->confirmed.ack.up >
			    finger->confirmed.conf_recv) 
			*/
			else if (finger != entry && finger->confirmed.value == ACKED)
			{
				still_waiting = true;
				conf_outstanding = true;   
			}
			finger = finger->next;
		} /* end while finger */

		/* if still_waiting = TRUE, waiting for higher numbered CONF from same upstrem node
		that the CONF came from ( multiple downstream case ? )*/

		/* Unack, disconnect and set up for removal unneeded 'up' entries. */
		if (!still_waiting)
		{
			finger = start_finger;
			while (finger && finger->path == path &&
			    finger->confirmed.value == CONFIRMED)
				finger = finger->next;
			while (finger && finger->path == path) 
			{
				if (finger->up == up)
				{

					/* Release "protected" spares to "upnode" */
					if (finger->confirmed.value)
					{
						finger->confirmed.value = 0;
						Give_Spares(cnode, finger->up, finger->down,
						finger->bw);
						spares_released = true;
						finger->down = NO_NODE;
						finger->time_connected = (float)0.0;
						finger->time_received = cnode->time;
					} /* end if finger->connected */
				} /* end if finger->up == up */
				
				/*ackable = (final == true) ? false : true*/
				if (!ackable && !finger->confirmed.value)
				{
					tempfinger = finger;
					finger = finger->next;
					Remove_Tablerec(cnode, tempfinger);
				}
				else               
				    finger = finger->next;
			} /* end while finger */

			Pack_Table(cnode);
			start_finger = get_start_finger(cnode, path);
		} /* end if not still waiting */

		
		  /* If conf_outstanding is TRUE, this node is still waiting for an additional CONF
			for this path */


		/* Disconnect and set up for removal other unneeded entries.
					 */
		if (!conf_outstanding)
		{
			if (final)
			{
				finger = get_end_finger(cnode, path);
				while (finger && finger->path == path && 
				    !finger->confirmed.value)
				{
					tempfinger = finger;
					if (finger->prev)
					{
						finger = finger->prev;
						Remove_Tablerec(cnode, tempfinger);
					}
					else 
					{
						Remove_Tablerec(cnode, tempfinger);
						finger = NULL;
					} /* end else not finger->prev */

				} /* end while finger */
			} /* end if final */

			finger = get_ack_finger(cnode, path);
			while (finger && finger->path == path &&
			    finger->confirmed.value) 
			{
				finger->confirmed.value = 0;
				Give_Spares(cnode, finger->up, finger->down,
				finger->bw);
				spares_released = true;
				finger->time_connected = (float)0.0;
				finger->time_received = cnode->time;
				finger->down = NO_NODE;
				if (!finger->confirmed.value)
				{
					tempfinger = finger;
					finger = finger->next;
					Remove_Tablerec(cnode, tempfinger);
				}
				else               
				    finger = finger->next;
			} /* end while finger */

			Pack_Table(cnode);
			start_finger = get_start_finger(cnode, path);
		} /* end if(!conf_outstanding */



		/* init nodelist *downs = NULL */
		
		
		/* Update down information. */
		ptr = downs;
		while (ptr) 
		{
			finger = start_finger;
			while (finger && finger->path == path &&
			    finger->confirmed.value)
			{
				if (finger->confirmed.value == CONFIRMED)
				{
					if (finger->down == ptr->num)
					{
						ptr->acked_bw += finger->bw;
						if (finger->up == up )
						{
							/* Chow: XXXX use the original finger->confirmed.ack.down
							if (finger->confirmed.ack.down >
							    ptr->msg_num)
								ptr->msg_num = finger->confirmed.ack.down;
							if (!still_waiting && finger->confirmed.ack.recv
							    > ptr->msg_num)
								ptr->msg_num = finger->confirmed.ack.recv;
							 */
						}/* end if finger->up == up */
					}
					if (finger->confirmed.conf_recv == FINAL && !final) 
					{
						int i;
						final = true;
						notified = (boolean *)malloc(sizeof(boolean) * (netsize + 1));
						for(i = 1; i <= netsize; i++)
							notified[i] = false;
						find_symbol_def(trail, ev->msg, "TRAIL");
						for(i = 0; i < (int)((strlen(trail) + 1) / NodeNameLength); i ++)
							notified[Get_Node_ID(&(trail[NodeNameLength * i]))] = true;
						Append(trail, cnode->name);
						init_symbol(&ev->msg, "TRAIL", trail);
					}
				}
				/* Do NOT want to forward CONFs if:
										  
				 a)  multiple upstream nodes resulted from one
					  ACK message and this is not the
					  reply from the last upstream node for this
					  message--condition, finger->up != up and
					  the down ack numbers are the same and not
					  all entries have been confirmed (up ack
					  numbers may be different if one of the upstream
					  nodes has an older connection through this node
					  to a different downstream node) 
									 */
				/* Chow: the above consideration need to be reexamined.
					in 8-22 cut, layer3m.net , this prevents node 43 from
					forward bw=21 to 22 
				else if (finger->down == ptr->num && finger->up != up &&
				    finger->confirmed.ack.down == ptr->msg_num)
					ptr->acked = false; 
				*/
				finger = finger->next;
			} /* end while finger */

			ptr = ptr->next;
		} /* end while ptr */

		/* Set conf_recv field. */
		finger = start_finger;
		while (finger && finger->path == path)
		{
			if (final
			    && (finger->confirmed.value == CONFIRMED || 
			    !finger->confirmed.value
			    || (finger->up == up && !still_waiting)))
				finger->confirmed.conf_recv = FINAL;
			finger = finger->next;
		} /* end while finger */

		/* Forward CONFs/CONFFs. */
		prior_msg_num = msg_num;
		ptr = downs;
		while (ptr)
		{
			if (ptr->acked)
			{
				finger = start_finger;
				/* Don't send CONF to already notified nodes.  Update
										 notified nodes.
									 */
				up_entry = false;
				found = false;
				ptr->acked = false;
				while(finger && finger->path == path && 
				    finger->confirmed.value == CONFIRMED)
				{
					if (finger->down == ptr->num)
					{
						found = true;
						if (!finger->confirmed.conf_sent)
						{
							finger->confirmed.conf_sent = true;
							if (!ptr->acked)
								ptr->acked = true;
						}
						if (finger->up == up)
							up_entry = true; 
					}
					finger = finger->next;
				} /* end while finger */
				if (!up_entry)
					found = false;
				if (!found)
					ptr->acked = true;
				if (ptr->acked)
				{
					if (ptr->msg_num != prior_msg_num)
					{
						sprintf(msgStr, "%2d", ptr->msg_num);
						init_symbol(&ev->msg, "ID", msgStr);
					}
					sprintf(aPacket, "%5d%5d%5d%5d%5d", 
						path, qos, n, ptr->acked_bw, NO_NODE); 
					if (final)
					{
						Forward_Msg(cnode, n, ev, "CONFF", ptr->num, aPacket);
						notified[ptr->num] = true;
					}
					else
						Forward_Msg(cnode, n, ev, "CONF ", ptr->num, aPacket);
					prior_msg_num = ptr->msg_num; 
				} /* end inner if ptr->acked */
			} /* end if ptr->acked */
			ptr = ptr->next;
		} /* end while ptr */


		/* If all of this node's spares have been used and all of this
						 node's connections are confirmed, then for any path that this
						 node is an end node, restoration has been completed and this
						 node needs to send a PDONE for the path.
					 */
		if (!conf_outstanding)
		{
			boolean all_confirmed = true;

			if (!spares_remaining(cnode))
			{
				finger = cnode->tablehead; 
				while(all_confirmed && finger)
				{
					if (finger->confirmed.value && 
					    finger->confirmed.value != CONFIRMED)
						all_confirmed = false;
					finger = finger->next;
				} /* end while all_confirmed */

				if (all_confirmed)
				{
					int this_path = 0;
					linkdata *clink;

					finger = cnode->tablehead;
					while (finger)
					{
						if (finger->whichPart == MIDDLE)
						{
							this_path = finger->path;
							while(finger && finger->path == this_path &&
							    finger->confirmed.value == CONFIRMED)
								finger = finger->next;
							while (finger && finger->path == this_path)
							{
								tempfinger = finger;
								if (finger->prev)
								{
									finger = finger->prev;
									Remove_Tablerec(cnode, tempfinger);
									finger = finger->next;
								}
								else
								{
									Remove_Tablerec(cnode, tempfinger);
									finger = cnode->tablehead;
								} /* end else not finger->prev */
							} /* end while finger */
						} /* end if whichPart == MIDDLE */

						/* else if finger->whichpart != MIDDLE */
						else if (finger->confirmed.conf_recv != FINAL) 
						{
							int confirmed_path_bw = 0;

							whichPart = finger->whichPart;
							this_path = finger->path;
							while(finger && finger->path == this_path)
							{
								if (finger->confirmed.value == CONFIRMED)
								{
									confirmed_path_bw += finger->bw;
									finger->confirmed.conf_recv = FINAL;
								}
								else
								{
									if (finger->up != NO_NODE ||
									    finger->down != NO_NODE)
										finger->bw = 0;
									if (finger->up == NO_NODE &&
									    finger->down == NO_NODE)
										finger->bw = finger->path_bw - 
										    confirmed_path_bw; 
								} /* end else not connected */
								finger = finger->next;
							} /* end inner while finger */


							init_symbol(&ev->msg, "TRAIL", cnode->name);
							start_finger = get_start_finger(cnode, this_path);
							clink = (linkdata *) head(&cnode->links);
							while (clink)
							{
								if (clink->active)
								{
									tempfinger = start_finger;
									conf_bw = 0;
									while(tempfinger && tempfinger->path ==
									    this_path && tempfinger->confirmed.value)
									{
										if (tempfinger->up == clink->dest ||
										    tempfinger->down == clink->dest)
											conf_bw += tempfinger->bw; 
										tempfinger = tempfinger->next;
									} /* end while tempfinger */

									if (whichPart == START)
										sprintf(aPacket, "%5d%5d%5d%5d%5d",
										this_path, qos, n, conf_bw,  NO_NODE);
									else
										sprintf(aPacket, "%5d%5d%5d%5d%5d",
										this_path, qos, NO_NODE, conf_bw, n);

									if (!pdones)
									{
										pdones = (Packet_info *) 
											malloc(sizeof(Packet_info));
										pdones->receiver_id = clink->dest;
										strcpy(pdones->thePacket, aPacket);
										pdones->next = NULL;
									} /* end if not pdones */
									else
									{
										present = pdones;
										while(present && 
										    present->receiver_id != clink->dest)
										{
											previous = present;
											present = present->next;
										}
										if (present && 
										    present->receiver_id == clink->dest)
										{
											if (strlen(present->thePacket) + 
											    PACKET_SIZE <= MAX_PACKET_LEN)
												strcat(present->thePacket, aPacket);
											else
											{
												char idStr[6];

												sprintf(idStr, "%d", ++(cnode->creq));
												init_symbol(&ev->msg, "TRAIL",
												cnode->name);
												Forward_Msg(cnode, n, ev, "PDONE", 
												clink->dest,
												present->thePacket);
												strcpy(present->thePacket, aPacket);
											}
										} /* end if receiver_id == clink->dest */
										else
										{
											last = (Packet_info *) 
												malloc (sizeof(Packet_info));
											last->next = NULL;
											previous->next = last;
											last->receiver_id = clink->dest;
											strcpy(last->thePacket, aPacket);
										} /* end else receiver_id = 
																							  clink->dest not found */
									} /* end else pdones */

								} /* end if clink->active */

								clink = (linkdata *) next (&cnode->links);
							} /* end while clink */

						} /* end else if end node*/

						else
							finger = finger->next;
					} /* end while finger */
				} /* end if all connections have been confirmed */
			} /* end if not clink */

		} /* end if node is not waiting for another CONF */


		free_nodelist(downs);

		if (final)
		{
			linkdata *clink = (linkdata *) head(&cnode->links);
			int pdone_bw;

			while (clink)
			{
				if (clink->active && !notified[clink->dest])
				{
					pdone_bw = 0;
					finger = get_start_finger(cnode, path);
					while(finger && finger->path == path &&
					    finger->confirmed.value)
					{
						if (finger->down == clink->dest)
							pdone_bw += finger->bw;
						finger = finger->next;
					} 
					sprintf(aPacket, "%5d%5d%5d%5d%5d", 
						path, qos, n, pdone_bw, NO_NODE);  
					init_symbol(&ev->msg, "TRAIL", cnode->name);
					if (!pdones)
					{
						char idStr[6];

						sprintf(idStr, "%d", ++(cnode->creq));
						init_symbol(&ev->msg, "ID", idStr);
						/* Chow: in 13-44cut, we observe 33 after receiving CONFF
							send out PDONE to 22 and that got forwarded faster 
							to downstream than CONFF which forwarded along the 
							longer path to 28. 
						try to see the effect of not to initiate it 
						Forward_Msg(cnode, n, ev, "PDONE", clink->dest, aPacket);
						*/
					}
					else
					{
						present = pdones;
						while(present && present->receiver_id != clink->dest)
						{
							previous = present;
							present = present->next;
						}
						if (present && 
						    present->receiver_id == clink->dest)
						{
							if (strlen(present->thePacket) + 
							    PACKET_SIZE <= MAX_PACKET_LEN)
								strcat(present->thePacket, aPacket);
							else
							{
								char idStr[6];

								sprintf(idStr, "%d", ++(cnode->creq));
								init_symbol(&ev->msg, "ID", idStr);
								Forward_Msg(cnode, n, ev, "PDONE", 
								clink->dest,
								present->thePacket);
								strcpy(present->thePacket, aPacket);
							}
						} /* end if receiver_id == clink->dest */
						else
						{
							last = (Packet_info *) 
								malloc (sizeof(Packet_info));
							last->next = NULL;
							previous->next = last;
							last->receiver_id = clink->dest;
							strcpy(last->thePacket, aPacket);
						} /* end else receiver_id = 
															  clink->dest not found */
					} /* end else pdones */
				} /* end if clink->active */

				clink = (linkdata *) next(&cnode->links);
			} /* end while clink */
			free(notified);
		} /* end if final */
		if (pdones)
		{
			init_symbol(&ev->msg, "TRAIL", cnode->name);
			conf_num = ++(cnode->creq);
			sprintf(msgStr, "%2d", conf_num);
			init_symbol(&ev->msg, "ID", msgStr);
			present = pdones;
			while(present)
			{
				previous = present;
				present = present->next; 
				Forward_Msg(cnode, n, ev, "PDONE", previous->receiver_id,
				previous->thePacket);
				free(previous);
			} /* end while packs */
		}
		Clean_Table(cnode);
		Sort_Table(cnode);
		if (!spares_released)
		{

		

			return; 
		}
	} /* end if whichPart == MIDDLE */


/******************* FINISH NODE **************************/
	else if (whichPart == FINISH) 
	{ 
		int conf_bw = bw,
		acked_path_bw = 0,
		nonode_bw = 0;
		boolean connection_made = false,
		all_confirmed = true;
		tablerec *prevfinger = NULL;
		linkdata *clink = NULL;
		path_ack_info *ack_sent = cnode->acks_sent;
		int entry_bw, entry_down,entry_ackhop, ack_bw;
		tablerec *entry;
		boolean search;

		if (!bw || type == 'X') 
		{



				/* AHA debug */

		if(debug && !bw)
		{
			printf("FINISHnode %d received CONF#%d(bw=%d)\n",cnode->id,msg_num,bw);
			printf("( msg_num=%d path=%d qos=%d up=%d bw=%d down=%d whichPart=%d path_bw=%d \n",  msg_num, path, qos, up, bw, down, whichPart, path_bw);
		}
		if(debug && type == 'X')
		{
			printf("FINISHnode %d received CONFX#%d(bw=%d)\n",cnode->id,msg_num,bw);
			printf("( msg_num=%d path=%d qos=%d up=%d bw=%d down=%d whichPart=%d path_bw=%d \n",  msg_num, path, qos, up, bw, down, whichPart, path_bw);
		}



			finger = start_finger;
			search = true;
			while (search && finger && finger->path == path) 
			{
				if (finger->up == up && 
					 finger->confirmed.value == 2 && 
					 /* Chow: add above to skip those CONFIRMED entry */
					 finger->confirmed.ack.up == msg_num) search =false;
				else 
					finger = finger->next;
			}
			if (finger == NULL) 
			{
				printf("cannot find the entry with path=%d, up=%d\n", path, up);
				/* Chow: 28-29 cut with 20 packetslenth path 818 conf with 0 bw
					 arrive but can not find 818 entry with up =27 (was removed?)
					 only up =22 left */
					 /* Chow: try to see replace exit() and return for difference 
				exit(1);
				*/
				return;
			}
			if (finger->path != path) 
			{
				printf("cannot find the entry with path=%d, up=%d\n", path, up);
				/* 
				exit(1);
				*/
				return;
			}
			/* find path and up entry */
			if (type == 'X') 
			{
				split_entry_bw(cnode, finger, bw);
			} 
			/* set conf_recv */
			/* Chow 3-40 case by setting msg_num it rules out the possibility to
				retry node 2 at node 40  Try to reset to 0
			finger->confirmed.conf_recv = msg_num; 
			*/
			finger->confirmed.conf_recv = 0; 
			finger->confirmed.value = 0; 
			/* Add the following two lines, make it possible for 
				short hopcount to compete, in 3-40case, this allows 2 to be 
				ahead of 14 */
			Give_Spares(cnode, finger->up, NO_NODE, finger->bw);
			finger->time_connected = (float)0.0;
			/* need to set 0 otherwise will be
				considered as conf outstanding */
			entry_down = finger->down;
			entry_bw = finger->bw;
			ack_bw = entry_bw;
			entry_ackhop = finger->ackhop;
			entry = finger;

			/* search for the entries with the same path but different up node 
				and the conf_recv is not set yet.
				retry by sending ack to those upstream node one at time
				until there are no more of those */

			/* cannot use get_unconnect_finger here since we change 
				entry->confirmed.value = 0, the search will stop there
				not at the first unconnect entry */
		/*	finger = start_finger;*/
			finger = Get_entry_min_hop_and_spares_suitable_for_retry(cnode, up, entry_ackhop, path, entry_down, trail);
			search = true;
			while (search && finger != NULL && finger->path==path) 
			{
				if (finger->confirmed.conf_recv == 0 && 
					finger->confirmed.value == 0 &&
			/*		finger->down == NO_NODE && Chow: add this addtional cond */
					finger->up != entry_down &&
					finger->up != NO_NODE &&
					!In_Trail(finger->up, trail)) 
				{
					int link_spare;

					/* find actual spare left on the link */
					link_spare = num_spares(cnode, finger->up);
					/* add the additional hopcount selection criteria make sure
						it can reach start node within repeat limit
					*/

					if (up_ack(cnode, path, finger->up) <= ACK_LIMIT) 
					{
						/* Chow add fair search condition */
						if ((finger->up==up && Only_available_entry(cnode, finger))||
							 finger->up != up) 
						{
							if (link_spare != 0 && finger->hopcount+entry->ackhop<=MAX_HOPS)
							{
								if (link_spare >= finger->bw) 
								{
									if (entry_bw > finger->bw) 
									{
										/* send ack to finger->up with ack_bw=finger->bw */
										update_entry_ack(cnode,ev, finger, entry, msg_num);
										entry_bw -= finger->bw;
									} 
									else if (entry_bw == finger->bw) 
									{
										update_entry_ack(cnode,ev, finger, entry, msg_num);
										entry_bw = 0;
										search = false; /* found enough entries for retry */
									} 
									else 
									{ /* entry_bw < finger->bw  */
										split_entry_bw(cnode, finger, entry_bw);
										update_entry_ack(cnode,ev, finger, entry, msg_num);
										entry_bw = 0;
										search = false; /* found enough entries for retry */
									}
								} 
								else 
								{ /* link_spare < finger->bw */
									if (entry_bw > link_spare) 
									{
										split_entry_bw(cnode, finger, link_spare);
										update_entry_ack(cnode, ev, finger, entry, msg_num);
										entry_bw -= link_spare;
									} 
									else if (entry_bw == link_spare) 
									{
										split_entry_bw(cnode, finger, link_spare);
										update_entry_ack(cnode,ev, finger, entry, msg_num);
										entry_bw = 0;
										search = false; /* found enough entries for retry */
									} 
									else
									{ /* entry_bw < link_spare  */
										split_entry_bw(cnode, finger, entry_bw);
										update_entry_ack(cnode, ev, finger, entry,msg_num);
										entry_bw = 0;
										search = false; /* found enough entries for retry */
									}
								}
							} /* end if link_spare != NULL */
						} /* Chow: check fair conditiontion */
					} /* within ACK_LIMIT */
				}/* end if(finger.confirmed...*/


				
				finger = Get_entry_min_hop_and_spares_suitable_for_retry(cnode, up, entry_ackhop, path, entry_down, trail);
				/*finger = finger->next;*/
			}/* end while(....*/
			/* confx do retry same as conf 0 
					*/
			if (type == 'X') 
			{
				if (entry_bw) 
				{
				/*	printf("lost bandwith \n");*/
				}
				Pack_Table(cnode);


		

				return;
			}
			if (entry_bw != ack_bw)  
			{ /* found entries to retry */
				if (entry_bw != 0) 
				{
					/* chow: retry the entry->up with the remaining bw 
					if (up_ack(cnode, path, up) <= ACK_LIMIT) {
					split_entry_bw(cnode, entry, entry_bw);
                    update_entry_ack(cnode, ev, entry, entry, msg_num);
							}
							*/
				}
				Pack_Table(cnode);


				

				return;
			} 
			else 
			{ /* does not found entry to retry */
			/* chow: retry the entry->up with the remaining bw 
				if (up_ack(cnode, path, up) <= ACK_LIMIT) {
               update_entry_ack(cnode, ev, entry, entry, msg_num);
						}
				*/
				/* Chow: found 6-8cut, 8 modifies those confirm_recv entries
				entry->confirmed.conf_recv = 0; 
				entry->confirmed.value = 2;
				*/


			

				return;
			}
		}/* end if(!bw...*/
		while(ack_sent && ack_sent->path != path)
			ack_sent = ack_sent->next;
		/* Chow; change to up ack limit per path 
		if (ack_sent && ack_sent->path == path && ack_sent >= ACK_LIMIT)
		{
		*/
		if (ack_sent && ack_sent->path == path) 
		{
			if (all_up_ack_exceed_limit(cnode, path) == 1) 
			{
				ackable = false;
			/* Chow in 22-29 layer3m.net, 29 recev confn from 27 with bw=2
				it should not final, since there are 21 yet to be received 
			final = true;
			*/
			}
		}

		if (final)
		{
			finger = start_finger;
			while(finger && finger->path == path && 
			    finger->confirmed.value)
			{
				finger->confirmed.conf_recv = FINAL;

				/* All paths may not be confirmed if the START node sends
					a CONFF because it ran out of spare links. */
				if (finger->up == up)
				{
					if (finger->confirmed.value == CONFIRMED)
						conf_bw -= finger->bw;
					else
					{
						/* chow: forget to check if conf_bw==0 */
						if (conf_bw == 0) 
						{
							Give_Spares(cnode, finger->up, NO_NODE, finger->bw);
							finger->bw = conf_bw;
							spares_released = true;
						} 
						else 
						{
							if (finger->bw <= conf_bw)
								conf_bw -= finger->bw;
							else
							{
								Give_Spares(cnode, finger->up, NO_NODE,
								finger->bw - conf_bw);
								finger->bw = conf_bw;
								conf_bw = 0;
								spares_released = true;
							}
							if (finger->bw == 0) finger->bw = conf_bw;
							finger->confirmed.value = CONFIRMED;
							/* CONF/CONFF messages contain DCS information, so
										 FINISH node makes connections upon receipt of
										 CONF or CONFF.
														 */
							finger->time_connected = dxc_time(cnode, finger->bw);
							connection_made = true;
							finger->confirmed.ack.down = 0;
							finger->confirmed.ack.recv = 0;
						} 
					} /* end else finger->confirmed.value > 1 */

				 } /* end if finger->up == up */
				finger = finger->next;
			} /* end while finger */

			while (finger && finger->path == path)
			{
				prevfinger = finger;
				finger = finger->next;
				Remove_Tablerec(cnode, prevfinger);
			} /* end while finger */
			Pack_Table(cnode);

			if (!spares_remaining(cnode))
			{
				finger = cnode->tablehead;
				while (all_confirmed && finger)
				{
					if (finger->confirmed.value && 
					    finger->confirmed.value != CONFIRMED)
						all_confirmed = false;
					finger = finger->next;
				} /* end while all_confirmed */

				if (all_confirmed)
				{
					int this_path = 0;
					linkdata *clink;
					Packet_info *pdones = NULL,
					*present = NULL,
					*previous = NULL,
					*lost = NULL;

					init_symbol(&ev->msg, "TRAIL", cnode->name);
					finger = cnode->tablehead;
					while (finger)
					{
						while (!this_path && finger)
						{
							if (finger->confirmed.value != CONFIRMED &&
							    (finger->whichPart == MIDDLE 
							    || finger->up != NO_NODE || finger->down !=
							    NO_NODE))
							{
								tempfinger = finger;
								if (finger->prev)
								{
									finger = finger->prev;
									Remove_Tablerec(cnode, tempfinger);
									finger = finger->next;
								}
								else
								{
									Remove_Tablerec(cnode, tempfinger);
									finger = cnode->tablehead;
								} /* end else not finger->prev */
							} /* end if whichPart is MIDDLE */

							else if (finger->confirmed.conf_recv != FINAL)
								this_path = finger->path;
							else
								finger = finger->next;
						} /* end while finger */

						if (this_path)
						{
							clink = (linkdata *) head(&cnode->links);  
							while (clink)
							{
								if (clink->active)
								{
									tempfinger = get_start_finger(cnode,
									this_path);
									conf_bw = 0;
									while(tempfinger && tempfinger->path ==
									    this_path &&
									    tempfinger->confirmed.value == CONFIRMED)
									{
										if (tempfinger->up == clink->dest ||
										    tempfinger->down == clink->dest)
											conf_bw += tempfinger->bw; 
										tempfinger = tempfinger->next;
									} /* end if tempfinger */

									sprintf(msgStr, "%2d", ++(cnode->creq));
									init_symbol(&ev->msg, "ID", msgStr);
									if (whichPart == START)
										sprintf(aPacket, "%5d%5d%5d%5d%5d",
										this_path, qos, n, conf_bw,  NO_NODE);
									else
										sprintf(aPacket, "%5d%5d%5d%5d%5d",
										this_path, qos, NO_NODE, conf_bw, n);
									if (!pdones)
									{
										pdones = (Packet_info *) 
											malloc(sizeof(Packet_info));
										pdones->receiver_id = clink->dest;
										strcpy(pdones->thePacket, aPacket);
										pdones->next = NULL;
									} /* end if not pdones */
									else
									{
										present = pdones;
										while(present && 
										    present->receiver_id != clink->dest)
										{
											previous = present;
											present = present->next;
										}
										if (present && 
										    present->receiver_id == clink->dest)
										{
											if (strlen(present->thePacket) + 
											    PACKET_SIZE <= MAX_PACKET_LEN)
												strcat(present->thePacket, aPacket);
											else
											{
												char idStr[6];

												sprintf(idStr, "%d",
												++(cnode->creq));
												init_symbol(&ev->msg, "ID", idStr);
												Forward_Msg(cnode, n, ev, "PDONE", 
												clink->dest,
												present->thePacket);
												strcpy(present->thePacket, aPacket);
											}
										} /* end if receiver_id == clink->dest */
										else
										{
											last = (Packet_info *) malloc 
											    (sizeof(Packet_info));
											last->next = NULL;
											previous->next = last;
											last->receiver_id = clink->dest;
											strcpy(last->thePacket, aPacket);
										} /* end else receiver_id = clink->dest 
																							 not found */
									} /* end else pdones */
								} /* end if clink->active */
								clink = (linkdata *) next(&cnode->links);
							} /* end while clink */
							while(finger && finger->path == this_path)
								finger = finger->next;   
							this_path = 0;
						} /* end if this_path */
					} /* end while finger */

					if (pdones)
					{
						char idStr[6];  

						present = pdones;
						while(present)
						{
							previous = present;
							present = present->next; 
							sprintf(idStr, "%d", ++(cnode->creq));
							init_symbol(&ev->msg, "ID", idStr);
							Forward_Msg(cnode, n, ev, "PDONE", previous->receiver_id,
							previous->thePacket);
							free(previous);
						} /* end while pdones */
					}



					return;
				} /* end if all_confirmed */
			} /* end if no spares left */
		} /* end if final */


		/* else NOT final */
		else
		{
			/* Set clean up booleans and confirm bandwidth. */
			finger = start_finger;
			while(finger && finger->path == path && finger->confirmed.value)
			{
				if (finger->up != up && finger->confirmed.value > 1)
					conf_outstanding = true;
				else if (finger->up == up)
				{
					finger->confirmed.conf_recv = msg_num;
					if (finger->confirmed.ack.up > msg_num)
					{
						still_waiting = true;
						conf_outstanding = true;
					} 
					if (conf_bw)
					{
						if (finger->confirmed.value == CONFIRMED)
							conf_bw -= finger->bw;
						/* else acked but not yet confirmed */
						else
						{
							boolean added_rec = false;

							if (conf_bw >= finger->bw)
							{
								conf_bw -= finger->bw;
							}
							/* else conf_bw < finger->bw */
							else
							{
								added_rec = true;
								tempfinger = 
								    (tablerec *)malloc(sizeof(tablerec));
								Copy_Tablerec(finger, tempfinger); 
								tempfinger->bw = finger->bw - conf_bw;
								tempfinger->time_received = cnode->time;
								Insert_After_Tablerec(cnode, finger, 
								tempfinger);
								finger->bw = conf_bw;
								conf_bw = 0;
								still_waiting = true;
								conf_outstanding = true;
							} /* end else conf_bw < finger->bw */
							finger->confirmed.value = CONFIRMED;
							finger->time_connected = dxc_time(cnode, finger->bw);
							connection_made = true;
							if (added_rec)
								finger = finger->next;
						} /* end else acked but yet unconfirmed */
					} /* end if conf_bw */
				} /* end else if finger->up == up */
				finger = finger->next; 
			} /* end while finger */


			Pack_Table(cnode);

			start_finger = get_start_finger(cnode, path);

			/* Unack unconfirmed entries and remove the up entries if this is the 
							last CONF expected from up. */
			if (!still_waiting)
			{
				finger = start_finger;
				while(finger && finger->path == path && 
				    finger->confirmed.value == CONFIRMED)
					finger = finger->next;
				while(finger && finger->path == path && finger->confirmed.value)
				{
					if (finger->up == up)
					{
						finger->confirmed.value = 0;  
						Give_Spares(cnode, finger->up, NO_NODE, finger->bw);
						spares_released = true;
						finger->time_connected = (float)0.0;
						/* Chow we should keep the entry for retry 
						finger->up = NO_NODE;
						*/
						add_up_ack(cnode, path, up);
						finger->time_received = cnode->time;
					}  
					finger = finger->next;
				} /* end while finger */
				Pack_Table(cnode);

				if (!ackable)
				{
					zero_up_entries(cnode, path, up);
					Clean_Table(cnode);
				}
			} /* end if not still waiting */

			if (!conf_outstanding && !spares_remaining(cnode))
			{
				finger = cnode->tablehead;
				while(all_confirmed && finger)
				{
					if (finger->confirmed.value == ACKED)
						all_confirmed = false;
					finger = finger->next;
				} /* end while finger */

				if (all_confirmed)
				{
					int other_path = 0,
					other_bw = 0;
					linkdata *clink;

					final = true; 
					finger = cnode->tablehead;
					while(finger)
					{
						if (finger->confirmed.value != CONFIRMED && 
						    (finger->whichPart ==
						    MIDDLE || finger->up != NO_NODE || finger->down !=
						    NO_NODE))
						{
							tempfinger = finger;
							if (finger->prev)
							{
								finger = finger->prev;
								Remove_Tablerec(cnode, tempfinger);
								finger = finger->next;
							}
							else
							{
								finger = finger->next;
								Remove_Tablerec(cnode, tempfinger);
							} /* end else not finger->prev */
						}
						else
							finger = finger->next;
					} /* end while finger */

					/* Send PDONE's for all paths for which node is an end
										node, including the just confirmed path. */
					init_symbol(&ev->msg, "TRAIL", cnode->name);
					finger = cnode->tablehead;
					while (finger)
					{
						while(finger && !other_path)
						{
							if (finger->whichPart != MIDDLE && 
							    finger->confirmed.conf_recv != FINAL)
							{
								other_path = finger->path;
								start_finger = finger;
							} /* end if whichPart != MIDDLE */

							finger = finger->next;
						} /* end while finger */

						if (other_path)
						{
							int otherPart = start_finger->whichPart;
							tablerec *other_finger = NULL;

							clink = (linkdata *)head(&cnode->links);
							while(clink)
							{
								if (clink->active)
								{
									other_finger = start_finger;
									other_bw = 0;
									while(other_finger && other_finger->path ==
									    other_path)
									{
										if ((other_finger->whichPart == START && 
										    other_finger->down == clink->dest) ||
										    (other_finger->whichPart == FINISH &&
										    other_finger->up == clink->dest))
											other_bw += other_finger->bw; 
										other_finger = other_finger->next;
									} 

									if (otherPart == START)
										sprintf(aPacket, "%5d%5d%5d%5d%5d", 
											other_path, qos, n, other_bw, NO_NODE);
									else
										sprintf(aPacket, "%5d%5d%5d%5d%5d", 
											other_path, qos, NO_NODE, other_bw, n);
									if (!pdones)
									{
										pdones = (Packet_info *) malloc(sizeof(Packet_info));
										pdones->receiver_id = clink->dest;
										strcpy(pdones->thePacket, aPacket);
										pdones->next = NULL;
									} /* end if not pdones */
									else
									{
										present = pdones;
										while(present && present->receiver_id != clink->dest)
										{
											previous = present;
											present = present->next;
										}
										if (present && present->receiver_id == clink->dest)
										{
											if (strlen(present->thePacket) + PACKET_SIZE <= 
											    MAX_PACKET_LEN)
												strcat(present->thePacket, aPacket);
											else
											{
												char idStr[6];

												sprintf(idStr, "%d", ++(cnode->creq));
												init_symbol(&ev->msg, "ID", idStr);
												Forward_Msg(cnode, n, ev, "PDONE", clink->dest,
												present->thePacket);
												strcpy(present->thePacket, aPacket);
											}
										} /* end if receiver_id == clink->dest */
										else
										{
											last = (Packet_info *) malloc (sizeof(Packet_info));
											last->next = NULL;
											previous->next = last;
											last->receiver_id = clink->dest;
											strcpy(last->thePacket, aPacket);
										} /* end else receiver_id = clink->dest not found */
									} /* end else pdones */
								} /* end if clink->active */

								clink = (linkdata *)next(&cnode->links);
							} /* while clink */
						} /* end if other_path */
						while(finger && finger->path == other_path)
							finger = finger->next;
						other_path = 0;
						other_bw = 0;
					} /* end while finger */


					if (pdones)
					{
						char idStr[6];

						present = pdones;
						while(present)
						{
							previous = present;
							present = present->next; 
							sprintf(idStr, "%d", ++(cnode->creq));
							init_symbol(&ev->msg, "ID", idStr); 
							Forward_Msg(cnode, n, ev, "PDONE", previous->receiver_id,
							previous->thePacket);
							free(previous);
						} /* end while pdones */
					}


				

					return;
				} /* end if (all_confirmed) */
			} /* end if not conf_outstanding */
		} /* end else not final */

		if (!final)
		{
			int ack_bw;

			start_finger = get_start_finger(cnode, path);

			acked_path_bw = 0;  
			finger = start_finger;
			while(finger && finger->path == path)
			{
				if (finger->confirmed.value)
					acked_path_bw += finger->bw;
				if (finger->up == NO_NODE)
				{
					nonodefinger = finger;
					nonode_bw += finger->bw;
				}
				finger = finger->next;
			} /* end while finger */

			/* Enable initiation of new restoration attempt. */
			if (acked_path_bw < path_bw)
			{
				if (ackable)
				{
					boolean new_ack = false;

					init_symbol(&ev->msg, "TRAIL", cnode->name);
					if (acked_path_bw + nonode_bw < path_bw) 
					{
						if (nonodefinger)
							nonodefinger->bw += path_bw - acked_path_bw - nonode_bw;
					}

					bw = path_bw - acked_path_bw; 


					finger = Get_unconnected_entry_with_min_hop_and_spares(cnode, path);


					while (finger && finger->path == path && bw)
					{
						/* chow: Fair scheme. try to let other entry a chance first */
						/* this allow 3-40, 28-20 cuts to reach 100%
							but 26-41cut reduce from 100% to 33% !!!
							need further study on why this happen */
							/* chow: try Fair scheme again */
						/* chow: compare with unfair scheme 
						if (finger->up == up) {
							tempfinger = finger->next;
							if (tempfinger != NULL) {
								if (tempfinger->path == path && tempfinger->up != up)
									finger = tempfinger;
							}
						}
						*/
						clink = (linkdata *) head(&cnode->links);
						while (clink && clink->dest != finger->up)
							clink = (linkdata *)next(&cnode->links);

						if (clink && clink->active && clink->spare)
						{
							int new_bw = clink->spare > bw ? bw : clink->spare;
							bw -= new_bw;
							Take_Spares(cnode, clink->dest, NO_NODE, new_bw);
							if (finger->bw > new_bw)
							{
								tempfinger = (tablerec *) malloc(sizeof(tablerec));
								Copy_Tablerec(finger, tempfinger);
								tempfinger->bw -= new_bw;
								Insert_Before_Tablerec(cnode, finger, tempfinger);
								tempfinger->time_received = cnode->time;
							}
							finger->bw = new_bw;
							finger->confirmed.value = ACKED;
							finger->time_connected = dxc_time(cnode, finger->bw);
							msg_num = cnode->creq++;
							finger->confirmed.ack.up = msg_num;
							connection_made = true;
							if (nonodefinger) 
							{
								nonodefinger->bw -= new_bw;
								if (!nonodefinger->bw)
									Remove_Tablerec(cnode, nonodefinger);
							}
							ack_bw = 0; 
							tempfinger = start_finger;
							/* Calculate acc bw */
							while(tempfinger && tempfinger->path == path)
							{
								if (tempfinger->up == clink->dest &&
								    tempfinger->confirmed.value)
									ack_bw += tempfinger->bw;
								tempfinger = tempfinger->next;
							} /* end while tempfinger */

							sprintf(aPacket, "%5d%5d%5d%5d%5d%2d", 
								path, qos, NO_NODE, ack_bw, n, 1);
							/* ackhop is 1 since this a finish node */

							InitActualBw(ev, finger->bw);
							sprintf(msgStr, "%2d", msg_num);
							init_symbol(&ev->msg, "ID", msgStr);
							Forward_Msg(cnode, n, ev, "ACK", clink->dest, aPacket); 
							add_up_ack(cnode, path, clink->dest);
							new_ack = true;
						}  /* end if clink->active */



						finger = Get_unconnected_entry_with_min_hop_and_spares(cnode, path);
						/*finger = finger->next;*/
					} /* end while finger */
					if (new_ack)
						ack_sent->num_acks++; 
				} /* end if ackable */

				if (connection_made)
					Pack_Table(cnode);
			} /* end if acked_path_bw < path_bw */

		} /* end if not final */

		Pack_Table(cnode);

		if (!spares_released)
		{
			/* restoration and DXC are independent 
			if (connection_made && cnode->time <
			    connection_time)
				cnode->time = connection_time;
			*/


		

			return;
		} /* end if not spares_released */
	} /* end if whichPart == FINISH */


	if (spares_released)
	{
		linkdata *clink = NULL;
		tablerec *otherfinger;  
		int acked_path_bw,
		ack_bw,
		original_path = path;
		boolean connection_made = false,
		init_trail = false,
		new_ack = false;

		finger = cnode->tablehead;
		while(finger)
		{
			otherfinger = finger;
			path = finger->path;
			if (finger->whichPart == FINISH && finger->confirmed.conf_recv !=
			    FINAL && finger->path != original_path)
			{
				nonodefinger = NULL;
				path_bw = otherfinger->path_bw;
				acked_path_bw = 0;
				while(otherfinger && otherfinger->path == path &&
				    otherfinger->confirmed.value)
				{
					acked_path_bw += otherfinger->bw;
					otherfinger = otherfinger->next;
				}

				if (acked_path_bw < path_bw)
				{
					path_ack_info *ack_sent = cnode->acks_sent;
					boolean ackable = true;

					while(ack_sent && ack_sent->path != path)
						ack_sent = ack_sent->next;
					if (ack_sent && ack_sent->path == path &&
					    ack_sent->num_acks >= ACK_LIMIT)
						ackable = false;

					if (ackable)
					{
						bw = path_bw - acked_path_bw;
						tempfinger = otherfinger;
						while(tempfinger && tempfinger->path == path &&
						    !nonodefinger)
						{
							if (tempfinger->up == NO_NODE)
								nonodefinger = tempfinger;
							tempfinger = tempfinger->next;
						}
						if (nonodefinger) nonodefinger->bw = bw;
						while (otherfinger && otherfinger->path == path && bw)
						{
							clink = (linkdata *) head(&cnode->links);
							while (clink && clink->dest != otherfinger->up)
								clink = (linkdata *) next(&cnode->links);
							if (clink && clink->active && clink->spare)
							{
								int new_bw = clink->spare > bw ? bw :
								clink->spare;
								bw -= new_bw;
								Take_Spares(cnode, clink->dest, NO_NODE, new_bw);
								if (otherfinger->bw > new_bw)
								{
									tempfinger = (tablerec *) malloc(sizeof(tablerec));
									Copy_Tablerec(otherfinger, tempfinger);
									tempfinger->bw -= new_bw;
									Insert_Before_Tablerec(cnode, otherfinger, 
									tempfinger);
									tempfinger->time_received = cnode->time;
								}
								otherfinger->bw = new_bw;
								otherfinger->confirmed.value = ACKED;
								otherfinger->time_connected=dxc_time(cnode, new_bw);
								msg_num = cnode->creq++;
								otherfinger->confirmed.ack.up = msg_num;
								otherfinger->ackhop = ackhop;
								connection_made = true;

								ack_bw = 0; 
								tempfinger = finger;
								while(tempfinger && tempfinger->path == path)
								{
									if (tempfinger->up == clink->dest &&
									    tempfinger->confirmed.value)
										ack_bw += tempfinger->bw;
									tempfinger = tempfinger->next;
								} /* end while tempfinger */

								sprintf(aPacket, "%5d%5d%5d%5d%5d%2d", 
									path, qos, NO_NODE, ack_bw, n, 1);
								sprintf(msgStr, "%2d", msg_num);
								init_symbol(&ev->msg, "ID", msgStr);
								if (!init_trail)
								{
									init_trail = true;
									init_symbol(&ev->msg, "TRAIL", cnode->name);
								}

								InitActualBw(ev, otherfinger->bw);
								Forward_Msg(cnode, n, ev, "ACK", clink->dest, aPacket); 
								add_up_ack(cnode, path, clink->dest);
								new_ack = true;
							} /* end if clink->active */
							otherfinger = otherfinger->next;
						} /* end while otherfinger */

					} /* end if ackable */
				} /* end if acked_bw < path_bw */

			} /* if FINISH node for path */

			if (new_ack)
			{
				path_ack_info *ack_sent = cnode->acks_sent,
				*new_info =  (path_ack_info *)
					malloc(sizeof(path_ack_info)); 

				new_ack = false;
				while(ack_sent && ack_sent->path != path)
					ack_sent = ack_sent->next;
				if (ack_sent && ack_sent->path == path)
					ack_sent->num_acks++;
				else
				{
					new_info->next = NULL;
					new_info->num_acks = 1; 
					new_info->uplist = NULL;
					new_info->downlist = NULL;
					new_info->path = path;
					ack_sent = cnode->acks_sent;
					if (ack_sent == NULL) cnode->acks_sent=new_info;
					else {
						while(ack_sent->next)
							ack_sent = ack_sent->next;
						ack_sent->next = new_info;
					}
				} /* else ack_sent->path != path */
			} /* end if new_ack */
			if (otherfinger && otherfinger->path == path)
				while(otherfinger && otherfinger->path == path)
					otherfinger = otherfinger->next;  
			finger = otherfinger;
		} /* end while finger */

		if (connection_made)
		{
			Pack_Table(cnode);
			/* DXC and restoration process are independent
			if (cnode->time < connection_time)
				cnode->time = connection_time;
			*/
		}
	} /* end if spares_released */


} /* end CONF_Processing */


/* in:  erec containing the recevied PDONE
				node identification number
				pointer to node information for node 'n'
		 
		 When a node receives a PDONE, it marks all paths in the packettes
		 of the PDONE as maximally restored and forwards the PDONE to all
		 its immediate neighbors not found in the TRAIL in the PDONE.  The
		 forwarded PDONE will contain a packette for each path that 'n' has
		 just discovered to be maximally restored.
	 */
static void PDONE_Processing(erec *ev, int n, node *cnode)
{
	char aPacket[PACKET_SIZE+1],
	newPacket[PACKET_SIZE+1],
	thePacket[MAX_PACKET_LEN + 1],
	trail[256];
	int i,  trail_length;
	int path, 
	qos,
	up, 
	down, 
	pdone_bw, 
	bw, 
	num_packettes;  
	tablerec *finger = NULL,
	*start_finger = NULL,
	*prevfinger = NULL;
	boolean  *availables;
	linkdata *clink;
	Packet_info *pdones = NULL,
	*last = NULL,
	*present = NULL,
	*previous = NULL;

	if (path == 413) {
		printf("chow: test 13-44cut.l for why conf 0 resulting in pdone and cancel of the retry that are possible downstream.\n");
	}
	availables = (boolean *) malloc(sizeof(boolean) * (netsize + 1));
	for (i = 0; i <= netsize; i++)
		availables[i] = true;
	find_symbol_def(trail, ev->msg, "TRAIL");
	trail_length = (strlen(trail) + 1) / NodeNameLength;
	for (i = 0; i < trail_length; i++)
		availables[Get_Node_ID(&(trail[NodeNameLength * i]))] = false;
	Append(trail, cnode->name);
	init_symbol(&ev->msg, "TRAIL", trail);

	find_symbol_def(thePacket, ev->msg, "PACK");
	if (debug_node && choosen == n) {
		fprintf(choosen_out, 
			"at %f, %d received PDONE=%s\n", cnode->time, n, thePacket);
		Print_Table(cnode);
		}
	num_packettes = strlen(thePacket) / PACKET_SIZE;
	for(i = 0; i < num_packettes; i ++)
	{
		strncpy(newPacket, &(thePacket[PACKET_SIZE * i]), PACKET_SIZE);
		parse_packet(newPacket, &path, &qos, &up, &bw, &down);
		if (debug_path && path == selected) {
			selected_path = true;
			Print_Table(cnode);
		}

		start_finger = get_start_finger(cnode, path);
		if (!start_finger)
			return;
		finger = start_finger;
		while(finger && finger->path == path && finger->confirmed.value)
		{
			if (finger->confirmed.conf_recv == FINAL)
				return; 
			finger->confirmed.conf_recv = FINAL;
			finger = finger->next;
		}
		while(finger && finger->path == path)
		{
			prevfinger = finger;
			finger = finger->next;
			Remove_Tablerec(cnode, prevfinger);
		} /* end while finger */


		clink = (linkdata *) head (&cnode->links);  
		while (clink)
		{
			if (clink->active && availables[clink->dest])
			{
				pdone_bw = 0;
				finger = start_finger;
				while(finger && finger->path == path &&
				    finger->confirmed.value)
				{
					if (finger->up == clink->dest || 
					    finger->down == clink->dest)
						pdone_bw += finger->bw;
					finger = finger->next;
				} /* end while finger */
				if (up == NO_NODE)
					sprintf(aPacket, "%5d%5d%5d%5d%5d", 
						path, qos, NO_NODE, pdone_bw, n);
				else
					sprintf(aPacket, "%5d%5d%5d%5d%5d", 
						path, qos, n, pdone_bw, NO_NODE);
				if (num_packettes == 1)
					Forward_Msg(cnode, n, ev, "PDONE", clink->dest, aPacket);
				else
				{
					if (!pdones)
					{
						pdones = (Packet_info *) malloc(sizeof(Packet_info));
						pdones->receiver_id = clink->dest;
						strcpy(pdones->thePacket, aPacket);
						pdones->next = NULL;
					} /* end if not pdones */
					else
					{
						present = pdones;
						while(present && present->receiver_id != clink->dest)
						{
							previous = present;
							present = present->next;
						}
						if (present && present->receiver_id == clink->dest)
						{
							if (strlen(present->thePacket) + PACKET_SIZE <= 
							    MAX_PACKET_LEN)
								strcat(present->thePacket, aPacket);
							else
							{
								Forward_Msg(cnode, n, ev, "PDONE", clink->dest,
								present->thePacket);
								strcpy(present->thePacket, aPacket);
							}
						} /* end if receiver_id == clink->dest */
						else
						{
							last = (Packet_info *) malloc (sizeof(Packet_info));
							last->next = NULL;
							previous->next = last;
							last->receiver_id = clink->dest;
							strcpy(last->thePacket, aPacket);
						} /* end else receiver_id = clink->dest not found */
					} /* end else pdones */
				} /* end else more than one packette */
			}
			clink = (linkdata *) next(&cnode->links);
		} /* end while clink */
	} /* end for i */

	if (pdones)
	{
		present = pdones;
		while(present)
		{
			previous = present;
			present = present->next; 
			Forward_Msg(cnode, n, ev, "PDONE", previous->receiver_id,
			previous->thePacket);
			free(previous);
		} /* end while pdones */
	}
} /* end PDONE_Processing */


/* in:  erec containing next message to be processed
				identification number of node that received the message
				pointer to node information for node that received the message

		 If simulation time has exceeded its limit sets variable to causes 
		 simulation to terminate.

		 Otherwise, determines type of message received and calls the 
		 appropriate message processing function.
	 */
void Process(erec *ev, int n, node *cnode)
{

	char       mtype[21];
	char       packID[7];


	/* AHA */
	if(CheckTableForError(cnode))
		{
		printf("When entering Process() \n");
		exit(1);
		}

	/* check for simulation end--event simulator always returns at STOP_TIME,
				therefore o.k. */
	if (ev->time == STOP_TIME) 
	{
		stop = true;
		return;
	}

	/* check for update of node local time */
	/* AHA note !!! The	PPTOVERRIDE + MPTOVERRIDE adding procedure have been*/
	/* moved to Forward_Msg(), else there will be a adding of PPTOVERRIDE + MPTOVERRIDE */
	/* to the time when the timer expire. This we do not want since PPTOVERRIDE + MPTOVERRIDE */
	/* is the delay of the rx side, and the timer is just a timer "inside" the DCS and we should not */
	/* add PPTOVERRIDE + MPTOVERRIDE to the expire time. The PPTOVERRIDE + MPTOVERRIDE should only */
	/* be added to messages.								*/
	if (ev->time >= cnode->time)
		cnode->time = ev->time;
	else 
	{
		fprintf(stderr,"ERROR Process()\n");
		fprintf(stderr,"ev->time < cnode->time\n");
		fprintf(stderr, " Should not be possible \n");
		exit(1);
	}

	if (cnode->dxc_time < cnode->time) cnode->dxc_time = cnode->time;
	/* determine msg type */
	find_symbol_def(mtype, ev->msg, "TYPE");


	/* START SIM CONTROL PROCESSING */
	if (!strcmp(mtype, " SIM")) 
	{
		SIM_Processing(ev, n, cnode);
		
		
		/* AHA */
		if(CheckTableForError(cnode))
			{
			printf("After SIM_Processing\n");
			exit(1);
			}

		return;
	}
	/* END SIM CONTROL PROCESSING */


	/* AHA */
	/* START TIMER EXPIRED PROCESSING */

	if(!strncmp(mtype, "TIEX",4))
	{
		TIMER_EXP_Processing(ev, n, cnode);

	
		if(CheckTableForError(cnode))
			{
			printf("After TIMER_EXP_Processing\n");
			exit(1);
			}



	}




	if (n == failureNodeID) 
		printf("failure node should not participate in message processing.\n");
	
	/* START FAIL PROCESSING */
	if (!strncmp(mtype, "FAIL", 4)) 
	{
		find_symbol_def(packID, ev->msg, "ID");
		
		



		FAIL_Processing(packID, ev, n, cnode);


		/* AHA */
		if(CheckTableForError(cnode))
			{
			printf("After FAIL_Processing\n");
			exit(1);
			}
		return;
	}
	/* END FAIL_PROCESSING */


	/* START PACKET PROCESSING */
	if (!strncmp(mtype, "PACK", 4)) 
	{
		find_symbol_def(packID, ev->msg, "ID");


		

		PACK_Processing(packID, ev, n, cnode);

		
		/* AHA */
		if(CheckTableForError(cnode))
			{
			printf("After PACK_Processing\n");
			exit(1);
			}
		return;
	}
	/* END PACKET PROCESSING */

	if (!strncmp(mtype, "ACK", 3))
		{
		
	

		ACK_Processing(ev, n, cnode);
		
		/* AHA */
		if(CheckTableForError(cnode))
			{
			printf("After ACK_Processing\n");
			exit(1);
			}


		}
	/*  else if (!strncmp(mtype, "NACK", 4))
			  NACK_Processing(ev, n, cnode);
		 */  else if (!strcmp(mtype, "CONF ") || 
						  !strcmp(mtype, "CONFF") ||
						  !strcmp(mtype, "CONFX"))
	{
		if (!strcmp(mtype, "CONFF"))
			{
		
			
			
			CONF_Processing(ev, n, cnode, 'F');

			/* AHA */
			if(CheckTableForError(cnode))
				{
				
				printf("After CONFF_Processing\n");
				exit(1);
				}

			}
		else if (!strcmp(mtype, "CONFX"))
			{


			

			 CONF_Processing(ev, n, cnode, 'X');


			 /* AHA */
			if(CheckTableForError(cnode))
				{
				printf("After CONFX_Processing\n");
				exit(1);
				}

			}
		else
			{


		    CONF_Processing(ev, n, cnode, 'N');

			/* AHA */
			if(CheckTableForError(cnode))
				{
				printf("After CONFN_Processing\n");
				exit(1);
				}

			}
	}
	else if (!strcmp(mtype, "PDONE"))
		{
		
		PDONE_Processing(ev, n, cnode);


		/* AHA */
		if(CheckTableForError(cnode))
			{
			printf("After PDONE_Processing\n");
			exit(1);
			}

		}

}  /* end Process */


/* in:  internal identification number for path, 'path_id'
				bandwidth of path 'path_id'
				internal identification number of node upstream of 'thisNode'
					on path 'path_id'--'prevNode' 
				internal identification number of node whose Connection Table
					will be updated to include an entry about path 'path_id'
				internal identification number of node downstream of 'thisNode'
					on path 'path_id'--'nextNode' 
				position of 'thisNode' on path 'path_id' 

		 Part of network initialization process, adds an entry to the 
		 Connection Table of 'thisNode' for path 'path_id'.
	 */
void Do_Path_Connection (int path_id, int qos, int hopcount, int bw, int pathlength, int prevNode, int thisNode, int nextNode,int whichPart)  
{
	node *this_ptr = NULL;


	this_ptr = (node *) recall (&net, thisNode);
	/* Chow: pathlength is the number of nodes in the path, therefore need
				to be subtracted by 1 for ackhop */
	Add_To_Table (this_ptr, path_id, qos, prevNode, hopcount, bw, nextNode, 
	pathlength-1, whichPart, CONFIRMED, false, bw);
}  /* end Do_Path_Connection */

/* Input: address to a pointer, ap,  pointed to a node list string, where
				  node are separated by '-'
	Outupt: address of the input pointer is updated to pointed to the next node
					in the string,
					the first node in the string is put in pap->name[i],
					return 1 if normal
							 0 if p is already pointed to '\0' end of string  */
int FindNodeName(pathArray *pap,  int i, char **ap) {
	char *q;
	char *p;
	p = *ap;
	if (*p  == '\0') return 0;
	q = pap->name[i];
	while (*p != '-' && *p != '\0') {
		*q = *p;
		p++; 
		q++;
	}
	/* check '*' the non-glass through mark */
	q--;
	if (*q == '*') {
		pap->glassThrough[i] = false;
		*q = '\0';
	} else {
		q++;
		*q = '\0';
		pap->glassThrough[i] = true;
	}

	fillUnderline(pap->name[i]);
	if (*p == '-') p++;  /* skip '-' */
	*ap = p;
	return 1;
}

/* in:  internal identification number for path--'path_id'
				qos of path of 'path_id'
				bandwidth of path of 'path_id'
				pathArray pointer

		 Part of network initialization process.
		 For each node on the path 'path_id', calls Do_Path_Connection to 
		 add 1 hop upstream and/or 1 hop downstream worth of information
		 to the node's Connection Table. 
	 */
void Make_Path (int path_id, int qos, int bw, pathArray *pap)
{
	int prevNodeID, thisNodeID, nextNodeID;
	int i;
	int pl = pap->length;
	int  which_part;

	prevNodeID = NO_NODE;
	i=0;
	if ((thisNodeID=Get_Node_ID(pap->name[i])) <=  0) {
		printf("searchNodeID: fail to find start node! nodename=%s\n", 
			pap->name[i]);
		exit(1);
	} 
	i++;
	if ((nextNodeID=Get_Node_ID(pap->name[i])) <= 0) {
		printf("searchNodeID: fail to find second node! p=%s\n", pap->name[i]);
		exit(1);
	}
	i++;
	Do_Path_Connection (path_id, qos, i-1, bw, pl,
	prevNodeID,
	thisNodeID,
	nextNodeID,
	START);
	do {
		prevNodeID = thisNodeID;
		thisNodeID = nextNodeID;
		if (i==pap->length) {
			nextNodeID = NO_NODE;
			which_part = FINISH;
		} else {
			if ((nextNodeID=Get_Node_ID(pap->name[i])) <= 0) {
				printf("searchNodeID: fail! p=%s\n", pap->name[i]);
				exit(1);
			}
			which_part = MIDDLE; i++;
		}
		Do_Path_Connection (path_id, qos, i-1, bw, pl,
		prevNodeID,
		thisNodeID,
		nextNodeID,
		which_part);
	} 
	while (which_part!=FINISH);
} /* end Make_Path */

/* in:  internal identification number for path--'path_id'
				qos of path of 'path_id'
				bandwidth of path of 'path_id'
				pathArray pointer

		 Issue Fail or PACK msgs.
		 Will skip glass through nodes.
		 For each node on the path 'path_id', calls Do_Path_Connection to 
		 add 1 hop upstream and/or 1 hop downstream worth of information
		 to the node's Connection Table. 
	 */
void Make_Path_Glass_Through (int path_id, int qos, int bw, pathArray *pap)
{
	int prevNodeID, thisNodeID, nextNodeID;
	int i;
	int pl = pap->length;
	int  which_part;
	boolean search;
	int hopcount = 1;
	int actual_pl = 1;

	/* the two end nodes are considered not glass-through */
	for (i=1; i<pl-1; i++) {
		if (!pap->glassThrough[i]) actual_pl++;
	}
	prevNodeID = NO_NODE;
	if ((thisNodeID=Get_Node_ID(pap->name[0])) <=  0) {
		printf("searchNodeID: fail to find start node! nodename=%s\n", 
			pap->name[0]);
		exit(1);
	} 
	i=1;
	search = true;
	while ( search && i < pl) {
		if (!pap->glassThrough[i]) {
			if ((nextNodeID=Get_Node_ID(pap->name[i])) <= 0) {
				printf("searchNodeID: can't find second node p=%s\n", pap->name[i]);
				exit(1);
			} else search = false;
		}
		i++;
	}
	if (search) nextNodeID=Get_Node_ID(pap->name[pl-1]);
	Do_Path_Connection (path_id, qos, hopcount, bw, actual_pl,
		prevNodeID, thisNodeID, nextNodeID, START);
	do {
		prevNodeID = thisNodeID;
		thisNodeID = nextNodeID;
		if (i==pl) {
			nextNodeID = NO_NODE;
			which_part = FINISH;
		} else {
			search = true;
			while ( search && i < pl) {
				if (!pap->glassThrough[i]) {
					if ((nextNodeID=Get_Node_ID(pap->name[i])) <= 0) {
						printf("searchNodeID: can't find next node p=%s\n", 
								pap->name[i]);
						exit(1);
					} else search = false;
				}
				i++;
			}
			if (search) nextNodeID=Get_Node_ID(pap->name[pl-1]);
			which_part = MIDDLE; 
		}
		hopcount++;
		Do_Path_Connection (path_id, qos, hopcount, bw, actual_pl,
			prevNodeID, thisNodeID, nextNodeID, which_part);
	} 
	while (which_part!=FINISH);
} /* end Make_Path_Glass_Through */

/* Input: pointer to a pathArray structure, and a path string 
	Output: an array with the node names in the path string
					with pathLength set */

void  ParsePathStr(pathArray *pap, char *pathStr) {
	int i;
	i = 0;
	strcpy(pap->path, pathStr);
	while (FindNodeName(pap, i, &pathStr)) {
		i++; 
	}
	pap->length = i;
}
	
void notify_send_packmsg() {
	int i;
	node *n;
	char STR1[256];
	erec *ev;
	linkdata *clink;

	if (failureScenario == SINGLE_LINK_BREAK) {
		sprintf (STR1, "%3s%3s", break1str, break2str);
	} else {
		sprintf (STR1, "%3s", failureNode);
	}
	for (i = 1; i <= netsize; i++) {
		n = (node *) recall(&net, i);
		if (strlen(n->packbuf) != 0) {
			ev = (erec *)malloc(sizeof(erec));
			ev->msg = NULL;
			init_symbol (&ev->msg, "ID", STR1);
			init_symbol(&ev->msg, "TRAIL", n->name);
			init_symbol(&ev->msg, "HOPS", "0");
			clink = (linkdata *) head (&n->links);
			while (clink) {
				if (clink->active) {
					Forward_Msg(n, i, ev, "PACK", clink->dest, n->packbuf);
					}
				clink = (linkdata *) next (&n->links);
			} /* end while clink */
			/* Forward_Msg will create its own erec */
			release_all_symbols(&ev->msg);
			Free(ev);
		}
	}
}

void notify_end_nodes(int path, int qos, int bw, pathArray *pap) 
{
	linkdata *clink;
	int n, v;
	node *cnode;
	char STR1[256];
	char aPacket[PACKET_SIZE+1];
	erec *ev;

	


	n = Get_Node_ID(pap->name[0]);
	v = Get_Node_ID(pap->name[pap->length-1]);





	cnode = (node *) recall(&net, n);
	



	Add_To_Table (cnode, path, qos, NO_NODE, 0, bw, NO_NODE,
		0, START, 0, false, bw);
	



	/* set msg id to 'nv' or 'vn' */
	if (failureScenario == SINGLE_LINK_BREAK) 
	{
		sprintf (STR1, "%3s%3s", break1str, break2str);
		
	} else 
	{
		sprintf (STR1, "%3s", failureNode);
	}




	sprintf(aPacket, "%5d%5d%5d%5d%5d", path, qos, n, bw, NO_NODE);

	


	if (strlen(cnode->packbuf) + PACKET_SIZE <= MAX_PACKET_LEN)
	{


		
		
		strcat(cnode->packbuf, aPacket);
	}
	else 
	{
	
	
		ev = (erec *)malloc(sizeof(erec));
		

		
		ev->msg = NULL;


		init_symbol (&ev->msg, "ID", STR1);

	

		init_symbol(&ev->msg, "TRAIL", cnode->name);
		init_symbol(&ev->msg, "HOPS", "0");
		clink = (linkdata *) head (&cnode->links);
		while (clink) {
			if (clink->active) {
				
				Forward_Msg(cnode, n, ev, "PACK", clink->dest, cnode->packbuf);
				}
			clink = (linkdata *) next (&cnode->links);
		} /* end while clink */
		/* Forward_Msg will create its own erec */
		release_all_symbols(&ev->msg);

		Free(ev);
		strcpy(cnode->packbuf, aPacket);
	}
	cnode = (node *) recall(&net, v);
	Add_To_Table (cnode, path, qos, NO_NODE, 0, bw, NO_NODE,
		0, FINISH, 0, false, bw);
}

void create_disrupted_rec(int pathID, boolean restorable, int qos, int bw, 
	pathArray *pap) {
			disruptrec *nextrec;

			nextrec = (disruptrec *)malloc(sizeof(disruptrec));
			nextrec->path_num = pathID;
			++pathAffected;
			if (restorable) ++pathRestorable;
			nextrec->bw = bw;
			nextrec->next = NULL;
			nextrec->restorable = restorable;
			nextrec->restoration_time = (float)0.0;
			nextrec->disrupt_time = (float)0.0;
			nextrec->restored_bw = 0;
			nextrec->links = 0;
			strcpy(nextrec->path, pap->path);
			if (disrupted == NULL) disrupted = nextrec;
			else {
				nextrec->next = disrupted;
				disrupted = nextrec;
				}
			/* Chow: modify  the code to let the start node 
					send PACK msg, skip fail msg reporting.
					To calculate accurately the propagation delay from 
					the cut point to the start node, require addtional info 
			if (restorable)
				if (GlassThrough)
					Make_Path_Glass_Through(pathID, qos, bw, pap); 
				else
					Make_Path(pathID, qos, bw, pap); 
				*/
				/* Chow: only keep those affected */
}

/* in:  file pointer, pointing to first path descriptor in 
				network description file (with file open for reading)  

		  Part of network initialization process.
		  Reads in a path description (integer bandwidth space hyphenated 
			  list of nodes the path passes through) from network description
			  file.  
		  Calls Make_Path to add the path information to node information
			  of each node on the path.
		  Repeats last two steps until all path descriptions have been
			  read in--until a nonpositive number is read in for the 
			  bandwidth.
		  Determines an upper limit for the restorable bandwidth--based
		  on the failure and the paths in the network.  Initializes
		  data structure used only for keeping statistics.
	 */
void Setup_Paths (FILE *path_data) 
{
	char  pathstr[MaxPathStringLength];
	pathArray pa;
	int pathCount=0;
	int  pathID; 
	int  qos;
	int   bw;
	int   i;
	boolean  restorable = false;
	disruptrec *now = NULL;
	int src_add_rbw = 0,
	dest_add_rbw = 0;   
	int noOfPaths;

	/* If link A-C is out, can restore the min(
		sum of remaining spares to A, sum of remaining 
		spares to C) + possibly more.  For example, A has 
		13 remaining spares, C has 6 remaining spares.  
		Path ACD had 3 working paths.  When ACD is
		torn down, 3 additional links from C-D become 
		available.  If the path is rebuilt without
		the use of the links from C to D then the C-D 
		links can be used in the restoration of
		another path.  Thus,  dest_add_rbw is at 
		least 2 * 3 and the upper limit on restorable
	
	
		Chow: not clear when it is double, should it
				be just 3 instead of 2*3?
	
		bandwidth is at least the min(13,6 + 2 * 3).
		Of course, do not want to restore more
		bandwidth than was lost, so if the lost 
		bandwidth were less than 12, the
		lost bandwidth would be the upper limit on 
		the restorable bandwidth.
											*/
	

	fscanf(path_data, "%d", &noOfPaths);




	while (fscanf(path_data, "%d %d %d %s", &pathID, &qos, &bw, pathstr)!= EOF)
	{
		
		/* AHA */
		if(pathID < 0 || qos < 0 || bw < 0 )
			{
			printf("ERROR Setup_Paths\n");
			printf("pathID=%d qos=%d bw=%d\n", pathID, qos, bw);
			exit(1);
			}

	



		pathCount++;
		ParsePathStr(&pa, pathstr);



		if (pathID == 18) 
			printf("why it is not restored\n");
		restorable = false;
		if (failureScenario == SINGLE_NODE_FAILURE) 
		{
			if (strcmp(pa.name[0], failureNode) == 0 || 
			    strcmp(pa.name[pa.length-1], failureNode) == 0) 
			{
				 /* failure node is the first node or the last node of the path */
				 restorable = false;
				 create_disrupted_rec(pathID, restorable, qos, bw, &pa);
			} else 
			{
				for (i=1; i<pa.length-1; i++) 
				{
					if (strcmp(pa.name[i], failureNode) == 0) 
					{
						restorable = true;
				 		create_disrupted_rec(pathID, restorable, qos, bw, &pa);
						notify_end_nodes(pathID, qos, bw, &pa);
						if (i > 1) src_add_rbw += bw;
						if (i < pa.length-2) dest_add_rbw += bw;
					}
				}
			}
		} else if (failureScenario == SINGLE_LINK_BREAK) 
		{
		
		
				if( pa.length == 0)
				exit(1);

			for(i = 0; i < pa.length; i++) 
			{
				if (strcmp(pa.name[i], break1str) == 0) 
				{
			

					if (i+1 < pa.length)  
					{
						if (strcmp(pa.name[i+1], break2str) == 0) 
						{
							restorable = true;
							

				 			create_disrupted_rec(pathID, restorable, qos, bw, &pa);

							

							notify_end_nodes(pathID, qos, bw, &pa);

						

							if (i > 0) src_add_rbw += bw; 
							/* the link after src node will be released for reuse */
							if (i+1 < pa.length-1) dest_add_rbw += bw;
							/* the link before dest node will be release for reuse */
						}
					}
				} else if (strcmp(pa.name[i], break2str) == 0) 
				{

				

					if (i+1 < pa.length)  
					{
						if (strcmp(pa.name[i+1], break1str) == 0) 
						{
							restorable = true;
				 			create_disrupted_rec(pathID, restorable, qos, bw, &pa);

						
							notify_end_nodes(pathID, qos, bw, &pa);


							if (i > 0) src_add_rbw += bw; 
							/* the link after src node will be released for reuse */
							if (i+1 < pa.length-1) dest_add_rbw += bw;
							/* the link before dest node will be release for reuse */
						}
					}
				}

			}
		
		}
			
	}
	

	if (pathCount != noOfPaths)  {
		printf("noOfpaths=%d != actual count=%d\n", noOfPaths, pathCount);
		exit(1);
	} 
	printf("No of paths affected by the failure =%d\n", pathAffected);
	max_path = noOfPaths;
	if (failureScenario == SINGLE_LINK_BREAK) {
		src_rbw += src_add_rbw;    
		dest_rbw += dest_add_rbw;
		if (src_rbw > dest_rbw)
		    uplim_rbw = dest_rbw;
		else
			uplim_rbw = src_rbw;
		if (lost < uplim_rbw)
		    uplim_rbw = lost;
	}


} /* End Setup_Paths */

void tear_down_links(int break1, int break2) {
	node *snode;
	node *dnode;
	linkdata *clink;
	boolean search;

	snode = (node *)recall(&net, break1);
	dnode = (node *)recall(&net, break2);
   clink = (linkdata *) head (&snode->links);
	search = true;
   while (search && clink) {
      if (clink->dest == break2) {
			clink->active = false;
			search = false;
      } else clink = (linkdata *) next (&snode->links);
   } /* end while clink */
   clink = (linkdata *) head (&dnode->links);
	search = true;
   while (search && clink) {
      if (clink->dest == break1) {
			clink->active = false;
			search = false;
      } else clink = (linkdata *) next (&dnode->links);
   } /* end while clink */
}

void tear_down_node_links(int failNodeID) {
	node *snode;
	node *dnode;
	linkdata *slink;
	linkdata *dlink;
	boolean search;

	snode = (node *)recall(&net, failNodeID);
   slink = (linkdata *) head (&snode->links);
   while (slink) {
		slink->active = false;
		dnode = (node *)recall(&net, slink->dest);
   	dlink = (linkdata *) head (&dnode->links);
		search = true;
		while (search && dlink) {
			if (dlink->dest == failNodeID) {
				dlink->active = false;
				search = false;
      	} else dlink = (linkdata *) next (&dnode->links);
   	} /* end while dlink */
      slink = (linkdata *) next (&snode->links);
	}
}

/*  in:  name of network description file  

    Part of network initialization process. 
    Opens network description file, 'network.net'.  Reads in the
    initial section of the file giving the network topology, message
    processing delays,...  Gives nodes internal identification numbers,
    begins initialization of node information structures, calls
    Setup_Paths to complete initialization of node information.
*/
static void init_nodes(char *s)
{
	FILE *netdef;
	char line[256], 
	tline[256];
	int idx, 
	jdx, 
	err, 
	linkcount;
	float distance;
	node *cnode, 
	*idxnode, 
	*jdxnode;
	linkdata *thelink, 
	*thetail;
	char STR1[256];
	char STR2[256];
	node *src_node, 
	*dest_node;
	int src;
	char *TEMP;
	float linkdistance, TEMP1, 
	TEMP2;
	char type[40];
    int zero;  /* used to read in value of unknown significance between delay 
	                and latitude values */



	/* get net file ready for read */
	if ((netdef = fopen( to_lower(STR1,s), "r")) == NULL)
	    {
		fprintf(stderr, "Cannot open input file:  %s\n", STR1);
		exit(1);
	} 

	/* define SIM node, mainly for logging purposes as source */
	cnode = (node *)malloc(sizeof(node));
	strcpy(cnode->name, "SIM");
	init_symbol(&nodename, int2str_(STR1, 0L), cnode->name);
	add(&net, (Anyptr)cnode, 0L);

	fgets(line, 256, netdef);
	sscanf(line, "%ld %s", &netsize, type);
	if (strcmp(type, "V_H3") == 0) {
		V_Hcoord = V_H3;
	} 
	else if (strcmp(type, "V_H3wSpanMiles") == 0) {
		OVERWRITE_LINK_DISTANCE = 1;
		V_Hcoord = V_H3wSpanMiles;
		printf("Overwrite Propagation delay using the span miles specified in each link.\n");
	} 
	else if (strcmp(type, "L_L3wSpanKms") == 0) {
		OVERWRITE_LINK_DISTANCE = 1;
		V_Hcoord = L_L3wSpanKms;
		printf("Overwrite Propagation delay using the span kms specified in each link.\n");
	} 
	else if (strcmp(type, "L_L3") == 0) {
		V_Hcoord = L_L3;
	} else if (strcmp(type, "V_H3wSpanMilesGlassThrough")== 0) {
		OVERWRITE_LINK_DISTANCE = 1;
		GlassThrough = true;
		V_Hcoord = V_H3wSpanMiles;
		printf("Overwrite Propagation delay using the span miles specified in each link.\n");

	} else {
		printf("cannot read old version file format, need to be version 3\n");
	}

	if (netsize > MAX_NODES) {
		printf(" Number of specified nodes exceeds program limits!\n");
		printf("   ONEPRONG ABOORTING!!! \n");
		exit(1);
	}

	/* for all nodes ... */

	
	for (idx = 1; idx <= netsize; idx++) 
	{
		
		cnode = (node *)malloc(sizeof(node));
		cnode->id = idx;

		/* AHA */
		cnode->packbuf[0] = '\0';
		cnode->RecordHead = NULL;


		init_list(&cnode->links);
		cnode->tablehead = NULL;
		cnode->tabletail = NULL;
		cnode->packlist = NULL;
		cnode->time = (float)0.0;
		cnode->dxc_time = (float)0.0;
		cnode->creq = 1;
		cnode->acks_sent = NULL;

		fgets(line, 256, netdef);
		TEMP = strchr(line, '\n');
		if (TEMP != NULL)
		    *TEMP = 0;
		tokenize(tline, line);

		/*
		chow: make up 3 characters, follow get_token(cnode->name, tline); 
				with fillUnderline();
		*/
		get_token(cnode->name, tline);
		fillUnderline(cnode->name);
		init_symbol(&nodename, int2str_(STR1, idx), cnode->name);
		printf("cnode->name=%s, strlen=%d\n", cnode->name, strlen(cnode->name));
		strcpy (ID_Table[idx], cnode->name);
		if (strcmp(cnode->name, break1str) == 0) src_node = cnode;
		if (strcmp(cnode->name, break2str) == 0) dest_node = cnode;

		err = (sscanf(get_token(STR1, tline), "%d", &linkcount) == 0);
		err = (sscanf(get_token(STR1, tline), "%g", 
		&cnode->msg_process_delay) == 0);
		err = (sscanf(get_token(STR1, tline), "%d", &zero) == 0);
		err = (sscanf(get_token(STR1, tline), "%g", &cnode->lat) == 0);
		err = (sscanf(get_token(STR1, tline), "%g", &cnode->long_) == 0);

		/* for all links... */
	

		for (jdx = 1; jdx <= linkcount; jdx++) 
		{
			
			thelink = (linkdata *)malloc(sizeof(linkdata));

		

			fgets(line, 256, netdef);
			

			TEMP = strchr(line, '\n');
			if (TEMP != NULL)
			    *TEMP = 0;
			tokenize(tline, line);



	
			err = (sscanf(get_token(STR1, tline), "%d", &thelink->dest) == 0);
			thelink->active = (strcmp(get_token(STR1, tline), "1") == 0);
			thelink->delay = (float)0.0;
			err = (sscanf(get_token(STR1, tline), "%d", &thelink->work) == 0);
			err = (sscanf(get_token(STR1, tline), "%d", &thelink->spare) == 0);
			thelink->delay = (float)0.0;
			if (OVERWRITE_LINK_DISTANCE == 1)  
			{
			
				err = (sscanf(get_token(STR2, tline), "%g", &linkdistance) == 0);
				if (err) 
				{
					printf("Error: tnet file does not have link distance field\n");
					exit(1);
				} 
				else 
				{
					/* linkdistance field in miles, 1 mile = 1.6093km */
					if (V_Hcoord == V_H3wSpanMiles) 
					    thelink->delay = (float)(linkdistance*1.6093 / (LIGHTSPEED / REFRACINDEX));
					if (V_Hcoord == L_L3wSpanKms)
						thelink->delay = (float)(linkdistance/ (LIGHTSPEED / REFRACINDEX));
				}
			} 
			else 
			{
					thelink->delay = (float)0.0;
			}


			add(&cnode->links, (Anyptr)thelink, jdx);
		
			
		}

			
		
	

		add(&net, (Anyptr)cnode, idx);
			
		
	}

	

	/* calculate link delays based on relative distances */
	for (idx = 1; idx <= netsize; idx++) 
	{
	
		idxnode = (node *)recall(&net, idx);



		for (jdx = 1; jdx <= idxnode->links.count; jdx++) 
		{
			
			thelink = (linkdata *)recall(&idxnode->links, jdx);
			
			if (thelink->active) 
			{

		

				jdxnode = (node *)recall(&net, thelink->dest);

			


				thelink->last_trans = (float)0.0;
				TEMP1 = idxnode->lat - jdxnode->lat;
				TEMP2 = idxnode->long_ - jdxnode->long_;
				if (V_Hcoord == V_H3) 
				{ /* use V_H coord */
					distance = (float)(sqrt((TEMP1*TEMP1+TEMP2*TEMP2)/10)*1.6);
					/* from Kershenbaum's Telecom. design text instrution manual */
					thelink->delay = (float)(distance / (LIGHTSPEED / REFRACINDEX));
				} 
				else if (V_Hcoord == L_L3) 
				{
					distance = (float)sqrt(TEMP1 * TEMP1 + TEMP2 * TEMP2);
					thelink->delay = (float)(distance * KMPERDEG / (LIGHTSPEED / REFRACINDEX));
				} 
			}

			

		}
	}
	

	if (failureScenario == SINGLE_LINK_BREAK) 
	{
		

			break1 = Get_Node_ID(break1str);
			break2 = Get_Node_ID(break2str);

			if (break1 == NO_NODE || break2 == NO_NODE) 
			{
				
				fprintf(stderr, 
				"Link scenario with broken link '%s' not in specified net '%s'\n."
				    , scenario, to_lower(STR1, s));
				exit(1); 
			} /* end if node in scenario not found in net */

			if (break1 == break2) 
			{
				fprintf(stderr,
				"No break occured.  For a link scenario both the source and\n");
				fprintf(stderr, "destination nodes were identified as %s.\n\n",
				break2str);
				exit(1);
			}
			

			
			tear_down_links(break1, break2);

	

			src_rbw = 0;
			dest_rbw = 0;
			thetail = (linkdata *)tail(&src_node->links);
			thelink = (linkdata *)head(&src_node->links);
			if (thelink->dest != break2)
			    src_rbw = thelink->spare;
			else
				lost = thelink->work;
			while (thelink != thetail) 
			{
				thelink = (linkdata *)next(&src_node->links);
				if (thelink->dest != break2)
				    src_rbw += thelink->spare;
				else
					lost += thelink->work; 
			/* Chow: should be += instead of = for multiple linke between nodes */
			}
			thetail = (linkdata *)tail(&dest_node->links);
			thelink = (linkdata *)head(&dest_node->links);
			if (thelink->dest != src)
			    dest_rbw = thelink->spare;

			while (thelink != thetail) 
			{
				thelink = (linkdata *)next(&dest_node->links);
				if (thelink->dest != src)
				    dest_rbw += thelink->spare;
			}

		} 
	else if (failureScenario == SINGLE_NODE_FAILURE) 
		{
			tear_down_node_links(Get_Node_ID(failureNode));
		}

	

		Setup_Paths (netdef);

	

		notify_send_packmsg();
		fclose(netdef);
		netdef = NULL;



}   /* end init_nodes */


	/* in:  internal node identification number 'node_id'
	        internal path identification number 'path_id', for the initial
	          call to this recursive function node 'node_id' should be
	          the START node for path 'path_id'
	        bandwidth of the path
	        hyphenated list of nodes on the path
	        file pointer pointing to location in file to put 
	          information about path 'path_id'
	        time node 'node_id''s connection on path 'path_id' was 
	          confirmed 
	        internal identification number for node upstream of node
	          'node_id' on path 'path_id'--in the case where node 'node_id'
	          is the START node, 'last_node' will be NO_NODE 
	
	   Initially called with node 'node_id', the START node for path
	   'path_id'.  Updates path information at each node.  Works its way
	   downstream along path 'path_id' by calling itself with the next node
	   on path 'path_id' until reaching the FINISH node.
	   At FINISH node prints information about path 'path_id' at location 
	   of file pointer.   
	*/
	void Follow_Path (int     node_id,
	int     path_id,
	int     path_bw,
	char    thePath[],
	FILE   *path_data,
	float   time,
	int     last_node) 
	    {
		node      *this_node;
		tablerec  *finger;
		char       newPath[160];

		this_node = (node *) recall (&net, node_id);
		Append (thePath, this_node->name);
		finger = get_start_finger(this_node, path_id); 
		while (finger && finger->path == path_id && finger->confirmed.value ==
		    CONFIRMED) 
		    {
			if (finger->up == last_node) {
				if (finger->time_connected > time) 
				    time = finger->time_connected;
				if (finger->bw < path_bw) 
				    path_bw = finger->bw;
				if (finger->whichPart == FINISH) {
					fprintf (path_data, "Path %2d (%2d bw) (%f msec):  %s\n", 
					path_id, finger->bw, time * 1000.0, thePath);
				}
				else {
					sprintf (newPath, "%s", thePath);
					Follow_Path (finger->down, path_id, finger->bw, newPath, 
						path_data, time, node_id);
				}
			}
			finger = finger->next;
		}
	}  /* end Follow_Path */


	/* in:  name of file to write pre-break paths into
	 
	   Generates header for file followed by a list of the paths in the 
	   network prior to the break. 
	*/
	void Trace_Paths (char filename[])
	    {
		FILE      *path_data;
		int        i;
		node      *start_node;
		tablerec  *finger;
		char       thePath[160];
		struct tm * local;
		time_t ltime;

		if ((path_data = fopen (filename, "w")) == NULL) 
		    {
			fprintf(stderr, "Cannot open output file:  %s", filename);
			exit(1);
		}


		gettime( &ltime );                /* Get time as long integer. */
        local = localtime( &ltime ); /* Convert to local time. */
		/* Must add 1 to the month since local returns 0 - 11 for month */
		fprintf(path_data, "Run Date:  %d/%d/%d\nRun Time:  %d:%02d\n", 
		local->tm_mon +1, local->tm_mday, local->tm_year, local->tm_hour, 
		local->tm_min);
		fprintf(path_data, "Network:  %s\nBreak:  %s\n\n", infilename, scenario);  
		fprintf(path_data, "Paths Prior to the Break\n------------------------\n");

		for (i = 1; i <= netsize; i++) 
		{
			start_node = (node *)recall(&net, i);
			sprintf (thePath, "%s", start_node->name);
			finger = start_node->tablehead;
			while (finger)
			    {
				if (finger->whichPart == START && finger->confirmed.value ==
				    CONFIRMED) 
				    Follow_Path (finger->down, finger->path, finger->bw, thePath, 
				path_data, finger->time_connected, i);
				/* Chow: there can be multiple paths with the same node pair
					we need to initialized thePath otherwise it will be
					accumulated and overflowed */
				sprintf (thePath, "%s", start_node->name);
				finger = finger->next;
			}    
		}
		fclose (path_data);
	} /* end Trace_Paths */


	/* in:  name of file to write disrupted paths into
	 
	   Generates a list of the paths in the network disrupted by the break.
	   This is a temporary file used in creating file 'endpaths.txt'.
	*/
	void Trace_Lost_Paths (char filename[])
	    {
		FILE *path_data;
		int i;
		node *start_node;
		tablerec *finger;
		char thePath[160];
		disruptrec *thisrec;


		if ((path_data = fopen (filename, "w")) == NULL) 
		    {
			fprintf(stderr, "Cannot open output file:  %s", filename);
			exit(1);
		}

		for (i = 1; i <= netsize; i++)
		{
			start_node = (node *) recall(&net, i);
			finger = start_node->tablehead;
			while (finger)
			    {
				if (finger->whichPart == START && finger->confirmed.value)
				    {
					thisrec = disrupted;
					while (thisrec && thisrec->path_num != finger->path)
					    thisrec = thisrec->next;  
					if (thisrec && thisrec->path_num == finger->path)
					    {
						sprintf (thePath, "%s", start_node->name);
						Follow_Path (finger->down, finger->path, 
						finger->bw, thePath, path_data, 
						finger->time_connected, i);
					} /* end if thisrec */
				} /* end if whichPart == START */

				finger = finger->next;
			} /* end while finger */
		} /* end for i */
		fprintf(path_data, "done");
		fclose(path_data);
	} /* end Trace_Lost_Paths */

	/* in:  name of file to write disrupted paths into
	 
	   Generates a list of the paths in the network disrupted by the break.
	   This is a temporary file used in creating file 'endpaths.txt'.
	*/
	void trace_lost_paths (char filename[])
	    {
		FILE *path_data;
		disruptrec *p;


		if ((path_data = fopen (filename, "w")) == NULL) 
		    {
			fprintf(stderr, "Cannot open output file:  %s", filename);
			exit(1);
		}

		p = disrupted;
		while (p) {
			fprintf (path_data, "Path %2d (%2d bw) (%f msec):  %s\n",
								p->path_num, p->bw, p->disrupt_time, p->path);
		    p = p->next;  
		}
		fprintf(path_data, "done");
		fclose(path_data);
	} /* end trace_lost_paths */


	/* in:  internal node identification number 'node_id'
	        internal path identification number 'path_id', for the initial
	          call to this recursive function node 'node_id' should be
	          the START node for path 'path_id'
	        bandwidth of the path
	        hyphenated list of nodes on the path
	        file pointer pointing to location in file to put 
	          information about path 'path_id'
	        internal identification number for node upstream of node
	          'node_id' on path 'path_id'--in the case where node 'node_id'
	          is the START node, 'last_node' will be NO_NODE 
	
	   Initially called with node 'node_id', the START node for path
	   'path_id'.  Updates path information at each node.  Works its way
	   downstream along path 'path_id' by calling itself with the next node
	   on path 'path_id' until reaching the FINISH node.
	   At FINISH node prints information about path 'path_id' at location 
	   of file pointer.   
	   Updates statistics for restoration effort.
	*/
	void Follow_Restored_Path (int     node_id,
	int     path_id,
	int     path_bw,
	char    thePath[],
	FILE   *path_data,
	int     last_node)  
	    {

		node      *this_node;
		tablerec  *finger;
		char       newPath[256];
		int        newPath_bw;
		disruptrec * thisrec = disrupted;
		disruptrec * startrec = disrupted;
		int path_length;

		this_node = (node *) recall (&net, node_id);
		Append (thePath, this_node->name);
		finger = get_start_finger(this_node, path_id);
		if (path_id == 413) 
			printf("why seg fault \n");

		while (finger && finger->path == path_id && 
		    finger->confirmed.value == CONFIRMED
		    && path_bw > 0)
		    {
			if (finger->up == last_node &&
			    finger->bw > 0)  
			    {
				if (path_bw >= finger->bw)
				    {
					newPath_bw = finger->bw;
					path_bw -= finger->bw;
					finger->bw = 0;
				}
				/* finger->bw > path_bw */
				else
				{
					newPath_bw = path_bw;
					finger->bw -= path_bw;
					path_bw = 0;
				}
				if (finger->whichPart == FINISH)
				    {
					char temp[256];

					path_length = strlen(thePath);
					if (strlen(thePath) > 44)
					    {
						short int i;

						strncpy(temp, thePath, 44);
						temp[44] = '\n';
						for(i = 1; i < 34; i++)
							temp[44 + i] = ' ';
						temp[78] = '\0';
						strcat(temp, &(thePath[44]));
						strcpy(thePath, temp);
					}
					removeUnderline(thePath);
					fprintf (path_data, " %2d     %2d       %6.2f    %4d   %s\n", 
					path_id, newPath_bw, 
					(float) newPath_bw * 100.0 / (float) finger->path_bw, 
					(int) (finger->time_connected * 1000), 
					thePath);
					if (finger->time_connected>lastRestoreTime)
						lastRestoreTime=finger->time_connected;

					while(thisrec && thisrec->path_num != path_id)
					    thisrec = thisrec->next;
					if (thisrec && thisrec->path_num == path_id && 
					    thisrec->restorable == true)
					    {
						thisrec->restored_bw += newPath_bw;
						thisrec->links += newPath_bw * (path_length - 3) / 4;
						if(thisrec->restoration_time < finger->time_connected)
							thisrec->restoration_time = finger->time_connected;
					}
				}
				else 
				    {
					sprintf (newPath, "%s", thePath);
					Follow_Restored_Path (finger->down, path_id, 
					newPath_bw, newPath, path_data, node_id);
				}
			}
			finger = finger->next;
		}
	}  /* End Follow_Restored_Path */


	/* in:  name of file to write restoration results into
	        time that last message was sent
	 
	   Generates header for file followed by a lists of the paths in the 
	   network prior to the break, the paths disrupted by the
	   break and the restored paths at the end of the restoration effort. 
	
	   In addition, restoration statistics such as the number of channels
	   released when the break occured, the spare usage, and the restoration
	   level are included. 
	 
	   If any of the debugging features are turned on, additional information
	   is included in file 'filename'.
	*/
	void Trace_Restored_Paths(char filename[], float endtime)
	    {
		FILE       *path_data, *fp, *lost_paths;
		int        i,
		path_num,
		path_bw,
		released_spares= 0;
		node      *start_node;
		tablerec  *finger;
		char thePath[160], STR1[256], STR2[256];
		char theline[160],initial_string[10];
		float      time;
		time_t ltime;
		struct tm *local;
		disruptrec *thisrec = disrupted;
		int num_restored = 0;
		int links_used = 0;
		int totalDisruptedBw = 0;
		int max_restorable = 0; 
		double wamean = 0.0, gemean = 1.0, hamean = 0.0;
		int within2PathNo = 0;
		int within2Bw = 0;
		statistics s;

		if ((path_data = fopen (filename, "w")) == NULL) 
		    {
			fprintf(stderr, "Cannot open output file:  %s", filename);
			exit(1);
		}
	

		gettime( &ltime );                /* Get time as long integer. */
        local = localtime( &ltime ); /* Convert to local time. */
		/* Must add 1 to the month since local returns 0 - 11 for month */
		fprintf(path_data, "Run Date:  %d/%d/%d\nRun Time:  %d:%02d\n", 
		local->tm_mon +1, local->tm_mday, local->tm_year, local->tm_hour, 
		local->tm_min);
		fprintf(path_data,"Network:  %s\nBreak:  %s\nSimulation Length:  %f sec\n"
		    , infilename, scenario, STOP_TIME);  
		if (PPTOVERRIDE >= 0.0)
		    fprintf(path_data,"Message Process Delay:  %f seconds\n",PPTOVERRIDE);
		fprintf(path_data, "DCS Cross Connect Time:  %f seconds\n", 
			CROSS_CONNECT_TIME);
		if (PARALLEL_DCS==1) 
			fprintf(path_data, "DCS operates in sequential mode\n"); 
		else if (PARALLEL_DCS==0) 
			fprintf(path_data, "DCS operates in batch mode, all bw in a request connected in one connect time.\n");
		else fprintf(path_data, "DCS operate in parallel mode with k servers.\n");
		if(!RANDOMDELAY)
			fprintf(path_data, "Constant Timer Delay = %f\n", DELAY);
		else
			{
			fprintf(path_data, "Random Timer Delay, Up_limit = %f, Down_limit = %f\n",RAND_DEL_LIMIT, RAND_DEL_DOWN_LIMIT);
			}
		fprintf(path_data, "Transmission Speed:  %f kbps\n", TX_SPEED);
		fprintf(path_data, "ACK Limit:  %d\n\n", ACK_LIMIT);

		fprintf(path_data, 
		"\nLost Paths\n-------------------------\n");
		if ((lost_paths  = fopen ("lostpaths.out", "r")) == NULL) 
		    {
			fprintf(stderr, "Cannot open file:  %s for read", "lostpaths.out");
			exit(1);
		}
		fscanf(lost_paths, "%s %d (%d bw) (%f msec):  %s\n", initial_string,
		&path_num, &path_bw, &time, theline);
		while(!strncmp(initial_string, "Path", 4)) 
		    {
			 if (reuse) {
			if (failureScenario == SINGLE_LINK_BREAK) 
				released_spares += path_bw * (strlen(theline) - 7) / 4;
			if (failureScenario == SINGLE_NODE_FAILURE) {
				int length_theline = strlen(theline);

				if (!strncmp(failureNode, theline, 3) || !strncmp(failureNode,
				&theline[length_theline - 3], 3))
				    released_spares += path_bw * (length_theline - 7) / 4;
				else
					released_spares += path_bw * (length_theline - 11) / 4;   
			} /* node failure */
			}
			removeUnderline(theline);
			fprintf(path_data, "Path %2d  (%2d bw)  %s\n", path_num, path_bw, theline);
			fscanf(lost_paths, "%s %d (%d bw) (%f msec):  %s\n", initial_string,
			&path_num, &path_bw, &time, theline);
		} /* end while theline != done */
		fclose(lost_paths);
		system("rm lostpaths.out");

		/* Print restoration statistics */
		if (failureScenario == SINGLE_NODE_FAILURE) {
			fprintf(path_data, "\n\nUnrestorable Paths\n(Lost Node is Source or ");
			fprintf(path_data,"Destination Node for Path)\nPath #\tLost Bandwidth\n");
			fprintf(path_data,"----------------------\n");
			thisrec = disrupted;
			while (thisrec != NULL)
			    {
				if (thisrec->restorable == false && thisrec->bw > 0)
				    fprintf(path_data, "%4d\t%4d\n", thisrec->path_num, thisrec->bw);
				thisrec = thisrec->next;
			}
		}

		fprintf(path_data, "\n\nRestoration Paths\n");
		fprintf(path_data, "Path     BW        %%       Time   Restoration\n");
		fprintf(path_data, " #    Restored  Restored  (msec)     Path\n");
		fprintf(path_data, "--------------------------------------------\n");
		for (i = 1; i <= netsize; i++) 
		{
			start_node = (node *)recall(&net, i);
			finger = start_node->tablehead;
			while (finger) 
			    {
				if (finger->whichPart == START && 
				    finger->confirmed.value == CONFIRMED &&
				    finger->bw > 0) 
				    {
					thisrec = disrupted;
					while(thisrec && thisrec->path_num != finger->path)
					    thisrec = thisrec->next;
					if (thisrec && thisrec->path_num == finger->path 
						&& thisrec->restorable)
					    {
						sprintf (thePath, "%s", start_node->name);
						if (finger->down == failureNodeID) {
							printf("found an entry  with finger->down=%d\n",
								failureNodeID);
						} else {
						Follow_Restored_Path (finger->down, finger->path, 
						finger->bw, thePath, path_data, i);
						}
					}
				}
				finger = finger->next;
			}    
		}

		collect_statistics(&s);

		fprintf(path_data,
		"\n\nUpper Limit on Total Number of Restorable Paths:  %d\n",
		uplim_rbw);
		fprintf(path_data, "Total Path Bandwidths Restored:  %d\n",
		s.totalRestoredBw);
		if (uplim_rbw > 0)
		    fprintf(path_data, "Aggregate Restoration Level:  %f Percent\n", 
			 s.totalRestorableBw == 0 ? 0.0 : 
		(float) s.totalRestoredBw  * 100.0 / s.totalRestorableBw); 
		fprintf(path_data, "\nNumber of Channels Released:  %d\n",
		released_spares);
		fprintf(path_data, "Number of Channels Used in Restoration Paths:  %d\n", 
		links_used);
		fprintf(path_data, "Reduction in Number of Spares In Network:  %d\n",
		links_used - released_spares); 
		fprintf(path_data, "\nTime of Last Message:  %f", endtime); 
		fprintf(path_data, "\nMessage Volume:  %d\n\n", msgCount);

		/*fclose (path_data);*/
	if (failureScenario == SINGLE_LINK_BREAK) 
     sprintf(STR2, "%s-%s", break1str, break2str);
	if (failureScenario == SINGLE_NODE_FAILURE) 
     sprintf(STR2, "%s", failureNode);
   sprintf(STR1, "%s path=%d/%d/%d/%d/%d=%4.2f%%/%4.2f%%/%4.2f%% bw=%d/%d/%d=%4.2f%% in2p=%d/%d/%d=%4.2f%%/%4.2f%%/%4.2f%% in2bw=%d/%d=%4.2f%% lrt=%f wam=%f ham=%f msgs=%d spares=%d/%d/%d\n",
	STR2,
	s.totalDisruptedPath,
	s.totalRestorablePath,
	s.fullyRestoredPath,
	s.partiallyRestoredPath,
	s.notRestoredPath,
	s.totalRestorablePath == 0 ? 0.0 :
		s.fullyRestoredPath*100.0/(double) s.totalRestorablePath,
	s.totalRestorablePath == 0 ? 0.0 :
		s.partiallyRestoredPath*100.0/(double) s.totalRestorablePath,
	s.totalRestorablePath == 0 ? 0.0 :
		s.notRestoredPath*100.0/(double) s.totalRestorablePath,
	s.totalDisruptedBw - s.totalRestorableBw, /* not restorable bw */
   s.totalRestoredBw,
   s.totalRestorableBw,
   s.totalRestorableBw == 0 ? 0.0 : 
		s.totalRestoredBw*100.0/(double) s.totalRestorableBw,
	s.in2FullyRestoredPath,
	s.in2PartiallyRestoredPath,
	s.in2NotRestoredPath,
   s.totalRestorablePath == 0 ? 0.0 : 
		s.in2FullyRestoredPath *100.0/(double) s.totalRestorablePath,
   s.totalRestorablePath == 0 ? 0.0 : 
		s.in2PartiallyRestoredPath*100.0/(double) s.totalRestorablePath,
   s.totalRestorablePath == 0 ? 0.0 : 
		s.in2NotRestoredPath*100.0/(double) s.totalRestorablePath,
	s.in2RestoredBw,
	s.totalRestorableBw,
   s.totalRestorableBw == 0 ? 0.0 : 
		s.in2RestoredBw*100.0/(double) s.totalRestorableBw,
   s.lastRestoreTime,
	s.totalRestoredBw==0 ? 0.0 : s.wamean/(double) s.totalRestoredBw,
	s.totalRestoredBw==0 ? 0.0 : (double) s.totalRestoredBw/s.hamean,
	/* should check hamean== 0.0 */
	msgCount,
	released_spares,
	s.links_used,
   s.links_used - released_spares);
  removeUnderline(STR1);
  printf("%s", STR1);
  fprintf(stderr, "%s", STR1);

  /* AHA */
  fprintf(path_data,"%s",STR1);
  fclose (path_data);

  fp = fopen("netres.dfop.res", "a");
  if (fp == NULL) {
   fprintf(stderr, "can not open netres.dfop.res\n");
  }
  fprintf(fp, "%s", STR1);
  fclose(fp);
	} /* end Trace_Restored_Paths */




	/* Generates header for file "OUT"--set to 'NETSIM.OUT' in  main. 
	*/
static void log_network()
	    {

		char STR1[256], STR2[256];
		char STR8[94];
		char STR9[256];
		time_t ltime;
		struct tm *local;
		char date[9], time[6];
		


		/* dump initial network configuration */
		/* AHA:
		gettimeofday(&t,0); replaced by own func "gettime()"
		local = localtime((time_t *) &t.tv_sec);*/
		gettime( &ltime );                /* Get time as long integer. */
        local = localtime( &ltime ); /* Convert to local time. */

		if(local->tm_mon != 9)
		    {
			date[0] = (local->tm_mon) / 10 + 48; /* localtime returns month as 0 - 11,
			                                          so need to add one more */
			date[1] = (local->tm_mon) % 10 + 49;
		}
		else
		{
			date[0] = '0';
			date[10] = '1';
		}
		date[2] = '/';
		date[3] = (local->tm_mday) / 10 + 48;
		date[4] = (local->tm_mday) % 10 + 48;
		date[5] = '/';
		date[6] = (local->tm_year) / 10 + 48;
		date[7] = (local->tm_year) % 10 + 48;
		date[8] = '\0';
		time[0] = (local->tm_hour) / 10 + 48;
		time[1] = (local->tm_hour) % 10 + 48;
		time[2] = ':';
		time[3] = (local->tm_min) / 10 + 48;
		time[4] = (local->tm_min) % 10 + 48;
		time[5] = '\0';

		fprintf(log_, "Run Date:  %s\n", date);
		fprintf(log_, "Run Time:  %s\n\n", time);
		fprintf(log_, "NETWORK DESCRIPTION for %s\n\n\n", infilename);

		/* dump simulation parameters */
		if (PPTOVERRIDE >= 0.0)
		    fprintf(log_, "       Port-CPU PROCESS DELAY = %f seconds\n", 
		PPTOVERRIDE);
		fprintf(log_, "       DCS CROSS CONNECT TIME = %f seconds\n",
		CROSS_CONNECT_TIME); 
		if (PARALLEL_DCS==1) 
			fprintf(log_, "DCS operates in sequential mode\n"); 
		else if (PARALLEL_DCS==0) 
			fprintf(log_, "DCS operates in batch mode, all bw in a request connected in one connect time.\n");
		else fprintf(log_, "DCS operate in parallel mode with k servers.\n");
		fprintf(log_, "       TRANSMISSION SPEED = %f kbps\n", TX_SPEED);
		fprintf(log_, "       ACK LIMIT = %d\n", ACK_LIMIT);
		if(!RANDOMDELAY)
			fprintf(log_, "Constant Timer Delay = %f\n", DELAY);
		else
			{
			fprintf(log_, "Random Timer Delay, Up_limit = %f, Down_limit = %f\n",RAND_DEL_LIMIT, RAND_DEL_DOWN_LIMIT);
			}
		sprintf(STR1, "       STOPTIME = %s", simtime2str(STR2, STOP_TIME));
		simlog(STR1);
		sprintf(STR1, "       LCS = %s", simtime2str(STR2, LINK_CHECK_SCALEFACTOR));
		simlog(STR1);
		sprintf(STR1, "       INDEX = %s", simtime2str(STR2, REFRACINDEX));
		simlog(STR1);
		sprintf(STR1, "       NOISE = %s", simtime2str(STR2, NOISE_SIZE));
		simlog(STR1);
		sprintf(STR8, "       OUT = %s", outfilename);
		simlog(STR8);
		simlog(" ");
		simlog(" ");
		sprintf(STR9, "       BREAK:  %s",scenario);
		simlog(STR9);
		simlog(" ");
		simlog(" ");
		simlog(" ");
		simlog("START OF SIMULATION RUN:  ");
		simlog(" ");
		simlog(" EV-TIME    Q-WAIT");
		simlog(" ");
	} /* end log_network */

	  /* AHA: Pascal Main (p2clib.c)*/
void PASCAL_MAIN(int num_arg, char **arg)
{
	P_argc = num_arg;
	P_argv = arg;
}

void main(int argc, char *argv[])
{
		char STR1[256], STR2[256];
		char STR4[256];
		char STR5[256], STR6[256];
		char STR7[256];
		char STR8[256], STR9[256];
		char STR12[256], STR13[256];
		char temp[259], 
		tempstr[81], 
		debug_info[41],
		path_num[10];
		time_t ltime;
		char string[8], str[128];
		struct tm *local;
		int i, j, q;
		float endtime = (float)0; /* For statistical use only:  final value is
		                        the time that the last message was sent in the
		                        simulation, always < STOP_TIME of the
		                        simulation.  */


		/* AHA stuct used to get systemtime*/
		struct _timeb timebuffer;

		/* AHA _ftime() used to get systemtime into timebuffer */
	/* timebuffer.millitm contains the 1/1000 sec from system time */
	/* used to set the random seed, so if you run the program several */
	/* times the random value returned by random_delay is not the same */

		_ftime( &timebuffer );
		srand(timebuffer.millitm );


		_netsim_init();

		/* PASCAL_MAIN is required--the cli function which reads in the
		     command line uses P_argc and P_argv which are globally available. */
		PASCAL_MAIN(argc, argv);
		log_ = NULL;

		/* The command line must contain at least 2 arguments, the first
		      cannot be an option.
		  */ 
		if (argc == 1 || argv[1][0] == '/')   
		    {
			fprintf(stderr,"TIMER_ONEP - OneProng with Timer function\n\n");
			fprintf(stderr,
			"Command Format:  timer_onep netfile [options]\n\n");
			fprintf(stderr," Options\n---------\n");
			fprintf(stderr,"/s=XXX	Stoptime ( default /s=300.0)\n");
			fprintf(stderr,"/l=XXX	LCS	(default /l=0.05)\n");
			fprintf(stderr,"/i=XXX	Index	(default /i=1.5)\n");
			fprintf(stderr,"/n=XXX	Noise	(default /n=0.00001\n");
			fprintf(stderr,"/o=filename, where filename=Outputfilename	(default /o=NETSIM.OUT)\n");
			fprintf(stderr,"/b=NXX-NYY linkbreak between XX and YY, /b=NZZ nodebreak nodeZZ\n");
			fprintf(stderr,"/x=XXX	Transmission speed	(default /x=8.0)\n");
			fprintf(stderr,"/p=XXX	Port CPU Delay	(default /p=0.100)\n");
			fprintf(stderr,"/m=XXX	msg/signature processing delay	(default /m=0.004)\n");
			fprintf(stderr,"/c=XXX	dcs connectd time	(default /c=0.02)\n");
			fprintf(stderr,"/y=X	parallel DCS mode =1 for sequential	(default /y=1)\n");
			fprintf(stderr,"/d=XXX	Debug  100=debug,110=debugPath,101=debugNode (default /d=000)\n");
			fprintf(stderr,"/r=XXX	Repeat	(default /r=25)\n");
			fprintf(stderr,"/a=XXX	Acklimit	(default /a=1)\n");
			fprintf(stderr,"/t=X	Algorithm type	(default /t=1)\n");
			fprintf(stderr,"/k=XXX	Timer Delay (Constant) Turns off Random Delay(which is default)\n");
			fprintf(stderr,"/z=XXX	Random Timer Delay UP Limit, Timer Delay 0-XXX sec	(default=1.0 sec)\n");
			fprintf(stderr,"/q=XXX	Random Timer Delay DOWN Limit, Timer Delay XXX-1.0 sec	(default=0.0 sec)\n");
			fprintf(stderr,"                                  These switches are ignored if /k=XXX is set\n");
			fprintf(stderr, "Example: DFOP nj3.net /b=N00-N01 \n");
			exit(1);
			}


		SaveEvents = false;
		stop = false;


		/* Initialize options. All default values, except FIRST_PARAM,
		     may be overridden by the command line options.  To override, on
		     command line use:
		        /first_letter_of_option=value_of_option 
		  */
		

		options = NULL;

	
		init_symbol(&options, "FIRST_PARAM", NOT_FOUND); /* FIRST_PARAM */
		init_symbol(&options, "s=", "300.0"); /*STOPTIME */
		init_symbol(&options, "l=", "0.05"); /*LCS */
		init_symbol(&options, "i=", "1.5"); /* index */
		init_symbol(&options, "n=", "0.00001"); /* noise */
		init_symbol(&options, "o=", "NETSIM.OUT"); /* output */
		init_symbol(&options, "b=", NOT_FOUND); /* break */
		init_symbol(&options, "x=", "8.0"); /* transmission speed */
		init_symbol(&options, "p=", "0.100");/* port-cpu delay */
		init_symbol(&options, "m=", "0.004");/* msg/signature processing delay */
		init_symbol(&options, "c=", "0.02"); /* dcs connectd time */
		init_symbol(&options, "y=", "1"); /* parallel DCS mode =1 for sequential*/
		init_symbol(&options, "d=", "000"); /* debug */
		init_symbol(&options, "r=", "25"); /* repeat */
		init_symbol(&options, "a=", "1"); /* ack limit */
		init_symbol(&options, "t=", "1"); /* algorithm type */
		/* AHA */
		init_symbol(&options, "k=", "RD");/* Timer Delay (constant)*/
		init_symbol(&options, "z=", "1.0");/* Random Delay Limit */
		init_symbol(&options, "q=", "0.0");/* Random Delay Limit */

		*result = '\0';

		
		/* Reads in the command line */


		cli(result, &options, '/');

	
		
		/* Normally, nothing is printed */
		if (*result != '\0')
		    puts(result);

		find_symbol_def(string, options, "k=");
		if (!strcmp(string, "RD")) 
		    {
			RANDOMDELAY = 1;
			find_symbol_def(str, options, "z=");
			i = (sscanf(str,"%f", &RAND_DEL_LIMIT) == 0);
			find_symbol_def(str, options, "q=");
			i = (sscanf(str,"%f", &RAND_DEL_DOWN_LIMIT) == 0);
			
			}
		else 
		    {
			RANDOMDELAY = 0;
			find_symbol_def(str, options, "k=");
			i = (sscanf(str,"%f", &DELAY) == 0);

			}



		find_symbol_def(scenario, options, "b=");
		if (!strcmp(scenario, NOT_FOUND)) 
		    {
			fprintf(stderr, 
			"Required field /b=<id-id> not found on command line!\n");
			exit(1);
			}
		
		failureScenario = decideFailureType(scenario);

		

		/* Filling in program variables based on values entered above with the 
		   init_symbol function calls and NOT replaced in the cli function call. */
		i = (sscanf(find_symbol_def(STR2, options, "s="), 
		"%g", &STOP_TIME) == 0);

		i = (sscanf(find_symbol_def(STR2, options, "l="), "%g",
		&LINK_CHECK_SCALEFACTOR) == 0);

		i = (sscanf(find_symbol_def(STR2, options, "i="),
		"%g", &REFRACINDEX) == 0);

		i = (sscanf(find_symbol_def(STR2, options, "n="), 
		"%g", &NOISE_SIZE) == 0);

		i = (sscanf(find_symbol_def(STR2, options, "x="), "%f", 
		&TX_SPEED) == 0);

		i = (sscanf(find_symbol_def(STR2, options, "p="), 
		"%g", &PPTOVERRIDE) == 0);

		i = (sscanf(find_symbol_def(STR2, options, "m="), 
		"%g", &MPTOVERRIDE) == 0);

		i = (sscanf(find_symbol_def(STR2, options, "c="), "%f", 
		&CROSS_CONNECT_TIME) == 0);

		i = (sscanf(find_symbol_def(STR2, options, "y="), "%d", 
		&PARALLEL_DCS) == 0);

		i = (sscanf(find_symbol_def(STR2, options, "r="), "%d",
		&MAX_HOPS) == 0); 

		i = (sscanf(find_symbol_def(STR2, options, "a="), "%d",
		&ACK_LIMIT) == 0); 

		i = (sscanf(find_symbol_def(STR2, options, "t="), "%d",
		&TYPE) == 0); 

		

		find_symbol_def(outfilename, options, "o=");

		

		find_symbol_def(infilename, options, "FIRST_PARAM");

		if ((log_ = fopen(outfilename, "w")) == NULL)
		    {
			fprintf(stderr, "Could not open file %s in main\n", outfilename);
			exit(1);
			}

		nodename = NULL;

		if (SaveEvents)
		    set_save_mode();
	
	

		init_list(&net);
		
	
		
		init_nodes(infilename);



		find_symbol_def(debug_info, options, "d=");


		if (debug_info[0] || debug_info[1] || debug_info[2])
		    Trace_Paths ("PATHTRACE.OUT");




		/* Trace_Lost_Paths("lostpaths.out"); */
		trace_lost_paths("lostpaths.out"); 

		

		log_network();

		/* Chow: events initialized in notify_end_nodes 
		if (failureScenario == SINGLE_LINK_BREAK) {
			setupLinkBreakEvent(0.0, break1str, break2str);
		}
		else if (failureScenario == SINGLE_NODE_FAILURE) {
			disruptrec * now = disrupted;

			setupNodeFailureEvent(0.0, failureNode);
			uplim_rbw = 0;
			while (now != NULL)
			    {
				if (now->restorable == true && now->bw >0)
				    uplim_rbw += now->bw;
				now = now->next;
			}
		} 
		else  if (failureScenario == MULTIPLE_FAILURE) {
			if ((fp = fopen( to_lower(STR1, failureEventFile), "r")) == NULL) {
				fprintf(stderr, "Cannot open input file:  %s\n", STR1);
				exit(1);
			}
			while (fscanf(fp, "%f %s\n", &time, scenario) != EOF) {
				if ((failureEvent=decideFailureType(scenario))==SINGLE_LINK_BREAK) {
					setupLinkBreakEvent(time, break1str, break2str);
				} 
				else if (failureEvent== SINGLE_NODE_FAILURE) {
					setupNodeFailureEvent(time, failureNode);
				}
			}
			fclose(fp);
		}
		*/

		/* Generate header for standard output */
		/* AHA:*/
		/*gettimeofday(&t,0);
		local = localtime((time_t *) &t.tv_sec);*/
		/* localtime returns month as 0 - 11, so must add 1) */
		gettime( &ltime );                /* Get time as long integer. */
        local = localtime( &ltime ); /* Convert to local time. */


		puts("\t\tConnection Table and Message History\n\n");
		printf("Run Date:  %d/%d/%d\nRun Time:  %d:%02d\n",
		local->tm_mon + 1, local->tm_mday, local->tm_year,
		local->tm_hour, local->tm_min);
		printf("Network:  %s\nBreak:  %s\n", infilename, scenario);
		printf("Message/Signature Process Delay:  %f seconds\n", MPTOVERRIDE);
		printf("Port-CPU Delay:  %f seconds\n", PPTOVERRIDE);
		printf("DCS Cross Connect Time:  %f seconds\n", CROSS_CONNECT_TIME); 
		if (PARALLEL_DCS==1) 
			printf("DCS operates in sequential mode\n"); 
		else if (PARALLEL_DCS==0) 
			printf("DCS operates in batch mode, all bw in a request connected in one connect time.\n");
		else 
			printf("DCS operate in parallel mode with k servers.\n");
		
		printf("Transmission Speed:  %f kbps\n",  TX_SPEED);
		printf("ACK Limit:  %d\n", ACK_LIMIT);
		if(RANDOMDELAY)
		{
			printf("Random Timer Delay with random_delay_up_limit:%f\n\n",RAND_DEL_LIMIT);
			printf("Random Timer Delay with random_delay_down_limit:%f\n\n",RAND_DEL_DOWN_LIMIT);

		}
		else
			printf("Constant Timer Delay with delay=%f\n\n",DELAY);

		/* Determine debugging options selected.  Open required files and
		     generate headers for those files. */  
		if (debug_info[0] != '0')
			{
		    debug = true;

		
			}
		else
			debug = false;
		if (debug_info[1] != '0')
		    debug_path = true;
		else
			debug_path = false;
		if (debug_info[2] != '0')
		    debug_node = true;
		else 
		    debug_node = false;

		if (debug_node && debug_path)
		    {
			i = 4;
			j = 0;
			while(debug_info[i] != '-')
			    path_num[j++] = debug_info[i++];
			selected = atoi(path_num);
			if (selected < 1 || selected > max_path)
			    debug_path = false;
			choosen = atoi(&(debug_info[++i]));
			if (choosen < 1 || choosen > netsize)
			    debug_node = false;
			} /* end if debug_node and debug_path */
		else if (debug_path)
		    {
			selected = atoi(&(debug_info[4]));
			if (selected < 1 || selected > max_path)
			    debug_path = false;
			}
		else if (debug_node)
		    {
			choosen = atoi(&(debug_info[4]));
			if (choosen < 1 || choosen > netsize)
			    debug_node = false;
			}

		/* If a link break occurs, create output files to contain the 
		     Connection Tables of the nodes at the ends of the breaks. */
		if (debug && length_scenario >4)
		    {
			if ((break1_out = fopen("BREAK1.OUT", "w")) == NULL)
			    {
				puts("Cannot open file:  BREAK1.OUT");
				exit(1);
				}
			else
				{
				fprintf(break1_out, "Connection Table & Message History for Node:  %s",
					break1str);

				fprintf(break1_out,"\n\nRun Date:  %d/%d/%d\nRun Time:  %d:%02d\n",
					local->tm_mon + 1, local->tm_mday, local->tm_year,
					local->tm_hour, local->tm_min);

				fprintf(break1_out, "Network:  %s\nBreak:  %s\n", infilename,
					scenario);

				fprintf(break1_out, 
					"Message/signature Process Delay:  %f seconds\n", MPTOVERRIDE);

				fprintf(break1_out, 
					"Port-CPU Delay:  %f seconds\n", PPTOVERRIDE);

				fprintf(break1_out, "DCS Cross Connect Time:  %f seconds\n",
					CROSS_CONNECT_TIME);

				if (PARALLEL_DCS==1) 
					fprintf(break1_out, "DCS operates in sequential mode\n"); 
				else if (PARALLEL_DCS==0) 
					fprintf(break1_out, "DCS operates in batch mode, all bw in a request connected in one connect time.\n");
				else 
					fprintf(break1_out, "DCS operate in parallel mode with k servers.\n");
				fprintf(break1_out, "Transmission Speed:  %f kbps\n",
					TX_SPEED);

				fprintf(break1_out, "ACK Limit:  %d\n\n", ACK_LIMIT);

				nextnode = (node *) recall(&net, break1);
				
				Print_Node_Table(nextnode);
				}
			if ((break2_out = fopen("BREAK2.OUT", "w")) == NULL)
			    {
				puts("Cannot open file:  BREAK2.OUT");
				exit(1);
				}
			else
				{
				fprintf(break2_out, "Connection Table & Message History for Node:  %s",
					break2str);
				fprintf(break2_out,"\n\nRun Date:  %d/%d/%d\nRun Time:  %d:%02d\n",
					local->tm_mon + 1, local->tm_mday, local->tm_year,
					local->tm_hour, local->tm_min);
				fprintf(break2_out, "Network:  %s\nBreak:  %s\n", infilename,
					scenario);
				fprintf(break2_out, 
					"Message/Signature Process Delay:  %f seconds\n", MPTOVERRIDE);
				fprintf(break2_out, 
					"Port-CPU Delay:  %f seconds\n", PPTOVERRIDE);
				fprintf(break2_out, "DCS Cross Connect Time:  %f seconds\n",
					CROSS_CONNECT_TIME);
				if (PARALLEL_DCS==1) 
					fprintf(break2_out, "DCS operates in sequential mode\n"); 
				else if (PARALLEL_DCS==0) 
					fprintf(break2_out, "DCS operates in batch mode, all bw in a request connected in one connect time.\n");
				else 
					fprintf(break2_out, "DCS operate in parallel mode with k servers.\n");
				fprintf(break2_out, "Transmission Speed:  %f kbps\n", TX_SPEED);
				fprintf(break2_out, "ACK Limit:  %d\n\n", ACK_LIMIT);
				nextnode = (node *) recall(&net, break2);
				Print_Node_Table(nextnode);
				}
			} /* end debug & lenf=gth scenario */

		if (debug_node)
		    {
			if ((choosen_out = fopen("CHOOSEN.OUT", "w")) == NULL)
			    {
				puts("Cannot open file:  CHOOSEN.OUT");
				exit(1);
				}
			else
				{
				char choosen_str[4];

				Get_Node_Name(choosen, choosen_str);
				
				fprintf(choosen_out, "Connection Table & Message History for Node:  %s"
				    , choosen_str);

				fprintf(choosen_out,"\n\nRun Date:  %d/%d/%d\nRun Time:  %d:%02d\n",
				local->tm_mon + 1, local->tm_mday, local->tm_year,
				local->tm_hour, local->tm_min);

				fprintf(choosen_out, "Network:  %s\nBreak:  %s\n",
				infilename, scenario);

				fprintf(choosen_out, 
					"Message/Signature Process Delay:  %f seconds\n", MPTOVERRIDE);

				fprintf(choosen_out, 
					"Port-CPU Delay:  %f seconds\n", PPTOVERRIDE);

				fprintf(choosen_out, "DCS Cross Connect Time:  %f seconds\n", 
				CROSS_CONNECT_TIME);

				if (PARALLEL_DCS==1) 
					fprintf(choosen_out, "DCS operates in sequential mode\n"); 
				else if (PARALLEL_DCS==0) 
					fprintf(choosen_out, "DCS operates in batch mode, all bw in a request connected in one connect time.\n");
				else 
					fprintf(choosen_out, "DCS operate in parallel mode with k servers.\n");
				fprintf(choosen_out, "Transmission Speed:  %f kbps\n",
				TX_SPEED);

				fprintf(choosen_out, "ACK Limit:  %d\n\n", ACK_LIMIT);

				nextnode = (node *) recall(&net, choosen);

				Print_Node_Table(nextnode);
				}
			}/* end debug node*/

		if (debug_path)
		    {
			if ((selected_out = fopen("SELECTED.OUT", "w")) == NULL)
			    {
				puts("Cannot open file:  SELECTED.OUT");
				exit(1);
				}
			else
				{
				fprintf(selected_out, "Connection Table & Message History for Path:  "
				    );

				fprintf(selected_out,"%3d\n\nRun Date:  %d/%d/%d\nRun Time:  %d:%02d"
				    , selected, 
				local->tm_mon + 1, local->tm_mday, local->tm_year,
				local->tm_hour, local->tm_min);

				fprintf(selected_out, "\nNetwork:  %s\nBreak:  %s\n",
				infilename, scenario);

				fprintf(selected_out, "Message/Signature Process Delay:  %f seconds\n",
					MPTOVERRIDE);        

				fprintf(selected_out, "Port-CPU Delay:  %f seconds\n",
					PPTOVERRIDE);        

				fprintf(selected_out, "DCS Cross Connect Time:  %f seconds\n",
				CROSS_CONNECT_TIME);        

				fprintf(selected_out, 
				"Transmission Speed:  %f kbps\n", TX_SPEED);

				fprintf(selected_out, "ACK Limit: %d\n\n", ACK_LIMIT);

				fprintf(selected_out,
				"For simplicity in generating this file, the sent");

				fprintf(selected_out,
				" messages precede\nthe received message.\n\n");

				}
			}/*end debug path*/

		if (debug)
		    {

		



			if ((event_out = fopen("EVENT.OUT", "w")) == NULL)
			    {
				puts("Cannot open file:  EVENT.OUT");
				exit(1);
				}
			else
				{
				


				fprintf(event_out, "Message and Bandwidth Tallies\n\n");

			

				fprintf(event_out,"Run Date:  %d/%d/%d\nRun Time:  %d:%02d", 
				local->tm_mon + 1, local->tm_mday, local->tm_year,
				local->tm_hour, local->tm_min);

				fprintf(event_out, "\nNetwork:  %s\nBreak:  %s\n",
				infilename, scenario);

				fprintf(event_out, "Message/Signature Process Delay:  %f seconds\n",
					MPTOVERRIDE);        

				fprintf(event_out, "Port-CPU Delay:  %f seconds\n", PPTOVERRIDE);        
				fprintf(event_out, "DCS Cross Connect Time:  %f seconds\n",
				CROSS_CONNECT_TIME);        
				fprintf(event_out, 
				"Transmission Speed:  %f kbps\n", TX_SPEED);
				fprintf(event_out, "ACK Limit: %d\n\n", ACK_LIMIT);



				}
		
			}

		/* Main loop that gets and processes messages, updates output files.
		  */

	
		q=0;
		
		do 
		    {
			

			ee = next_event();
			eventCount++;
			if (debug && !(eventCount % 100))
			    {
				fprintf(event_out, 
				"Event #%d\tFAILs:  %d\tPACKs:  %d\tACKs:  %d\tCONFs:  %d",
				eventCount, FAILCount, PACKCount,
				ACKCount, CONFCount);
				fprintf(event_out, "\n\t\tPDONEs:  %d\tbw:  %d\n", PDONECount, 
				bwCount); 

			



				}

			if (ee != NULL) 
			    {
				nextnode = (node *)recall(&net, ee->next);


				/* AHA */
				if(CheckTableForError(nextnode))
					{
					printf("nextnode contained errors\n");
					exit(1);
					}




				if (debug)  
					{
					find_symbol_def (STR8, ee->msg, "TYPE");
					if ((!strncmp (STR8, "ACK", 3)) ||
				    (!strncmp (STR8, "CONF", 4)) || 
				    (!strcmp(STR8, "PDONE")) ||
				    (!strncmp (STR8, "PACK", 4)) || (!strncmp(STR8, "FAIL", 4)))
						{
						sprintf(STR12, "%s  %s  %s -> %s of Type: %s ID: %s PACK: %s",
						simtime2str(STR1, ee->time),
						simtime2str(STR2, (float)bounds(0.0, nextnode->time - ee->time, 
						1.0e30)),
						find_symbol_def(STR5, nodename, int2str_(STR4, ee->src)),
						find_symbol_def(STR6, nodename, int2str_(STR7, ee->next)),
						find_symbol_def(STR8, ee->msg, "TYPE"),
						find_symbol_def(STR9, ee->msg, "ID"), 
						find_symbol_def(STR13, ee->msg, "PACK"));     
						}
					else
						{
						sprintf(STR12, "%s  %s  %s -> %s of Type: %s ",
						simtime2str(STR1, ee->time),
						simtime2str(STR2, (float)bounds(0.0, nextnode->time - ee->time, 
						1.0e30)),
						find_symbol_def(STR5, nodename, int2str_(STR4, ee->src)),
						find_symbol_def(STR6, nodename, int2str_(STR7, ee->next)),
						find_symbol_def(STR8, ee->msg, "TYPE"));
						}
				/* print event info */
					if (strlen(STR12) > 80)
						{
						int length = strlen(STR12);

						strncpy(temp,STR12,80);
						temp[80] = '\n';
						temp[81] = '\0';
						if (length < 161)
							strcat(temp, &(STR12[80]));
						else
							{
							strncpy(tempstr, &(STR12[80]), 80);
							strcat(temp, tempstr);
							temp[161] = '\n';
							temp[162] = '\0';
							if (length < 241)
								strcat(temp, &(STR12[160]));
							else
								{
								strncpy(tempstr, &(STR12[160]), 80);
								strcat(temp,tempstr);
								temp[242] = '\n';
								temp[243] = '\0';
								strcat(temp, &(STR12[240]));
								}
							}
						strcpy(STR12,temp);
						}
					simlog(STR12);

					if (endtime < ee->time && ee->time < STOP_TIME)
						endtime = ee->time;

				/* extract & print path info */
					find_symbol_def(pathlist, ee->msg, "PATH");
					sprintf(STR1, "                    by%s", pathlist);
					simlog(STR1);
					
					}/* end debug */

				/* do event processing at node */

				if (debug) 
					{ 
					Print_Table (nextnode);
					printf("\n%s\n\n", STR12);
					}
				if (debug && length_scenario > 4)
				    if (break1 == ee->next)
						fprintf(break1_out, "\n%s\n\n", STR12);
					else if (break2 == ee->next) 
						fprintf(break2_out, "\n%s\n\n", STR12);
				if (debug_node && choosen == ee->next)      
				    fprintf(choosen_out, "\n%s\n\n", STR12); 
				
			

				Process(ee, ee->next, nextnode);
				if (debug_path && selected_path)
				    {
					fprintf(selected_out, "\n%s\n\n", STR12);
					Print_Table(nextnode);
					selected_path = false;  
					}
				if (debug) 
					{
					Print_Table (nextnode);

				
					}
				if (debug && failureScenario == SINGLE_LINK_BREAK)
					{
				    if (break1 == ee->next || break2 == ee->next) 
						{

					

						Print_Node_Table(nextnode);
					



						}
					}
				if (debug_node && choosen == ee->next)
				   /*  Print_Node_Table(nextnode); */
				   /* release dynamic memeory */
					if (!SaveEvents) 
						{ 
						release_all_symbols(&ee->msg);
						Free(ee);
						}
		
				} 
				else stop = true;
			} 
			while (!stop);

		for (i = 1; i <= netsize; i++) 
		{

		

			nextnode = (node *)recall(&net, i);
			Pack_Table(nextnode);
			Clean_Table(nextnode);
		}

		sprintf(STR12, "%scut.l", scenario);
		Trace_Restored_Paths (STR12, endtime);  

		sprintf (STR12, "\nMessage Volume was:  %d\n", msgCount);
		simlog (STR12);
		printf ("\nMessage Volume was:  %d\n", msgCount);

		/* Close the output files. */
		fflush(log_);
		P_ioresult = 0;
		if (log_ != NULL)
		    {
			fclose(log_);
			log_ = NULL;
			} /* end if log_ */


		if (debug && length_scenario > 4)
		    {
			if (break1_out)
			    {
				fflush(break1_out);
				fclose(break1_out);
				break1_out = NULL;
				}
			if (break2_out)
			    {
				fflush(break2_out);
				fclose(break2_out);
				break2_out = NULL;
				}
			} /* end if debug */

		if (debug_node && choosen_out)
		    {
			fflush(choosen_out);
			fclose(choosen_out);
			choosen_out = NULL;
			}

		if (debug_path && selected_out)
		    {
			fflush(selected_out);
			fclose(selected_out);
			selected_out = NULL;
			}

		if (debug)
		    {
			fprintf(event_out, 
			"\nFinal\n\tFAILs:  %d\tPACKs:  %d\tACKs:  %d\tCONFs:  %d\n",
			FAILCount, PACKCount, ACKCount, CONFCount);  
			fprintf(event_out, 
			"\tPDONEs:  %d\tbws:  %d\n\nNumber of Messages:  %d\n", 
			PDONECount, bwCount, msgCount);
			fflush(event_out);
			fclose(event_out);
			event_out = NULL;
			}
		printf("END OF SIMULATION\n");
		exit(0);
		}
		/* End. */
